# SPDX-FileCopyrightText: © 2025 Tenstorrent AI ULC
#
# SPDX-License-Identifier: Apache-2.0
import torch
import torch_ttnn
import pytest
import pickle
import ttnn
from pathlib import Path
from tests.utils import calculate_accuracy, render_metric_string_list_to_input_args_kwargs


class AtenModule(torch.nn.Module):
    def __init__(self):
        super().__init__()

    def forward(self, *args, **kwargs):
        return torch.ops.aten.add.Tensor(*args, **kwargs)


metrics = []


def save_pickle(obj, base_path, filename):
    p = Path(base_path)
    p.mkdir(parents=True, exist_ok=True)
    pickle_out_path = p / f"{filename}.pickle"
    with open(pickle_out_path, "wb") as f:
        pickle.dump(obj, f)


def teardown_module(module):
    print(metrics)
    save_pickle(metrics, "metrics-autogen-op/ALL", "aten.add.Tensor")


@pytest.mark.parametrize(
    "input_strings",
    [
        ["Tensor<[1, 256, 1024]> self = ?", "Tensor<[1, 256, 1024]> other = ?"],
        ["Tensor<[1, 16, 256, 256]> self = ?", "Tensor<[1, 1, 1, 256]> other = ?"],
        ["Tensor<[1, 32, 1536]> self = ?", "Tensor<[1, 32, 1536]> other = ?"],
        ["Tensor<[1, 32, 6144]> self = ?", "Tensor other = 1"],
        ["Tensor<[1, 32, 6144]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 50, 768]> self = ?", "Tensor<[1, 50, 768]> other = ?"],
        ["Tensor<[2, 7, 512]> self = ?", "Tensor<[1, 7, 512]> other = ?"],
        ["Tensor<[2, 8, 7, 7]> self = ?", "Tensor<[2, 1, 7, 7]> other = ?"],
        ["Tensor<[2, 7, 512]> self = ?", "Tensor<[2, 7, 512]> other = ?"],
        ["Tensor<[2, 512]> self = ?", "Tensor<[2, 512]> other = ?"],
        ["Tensor<[1, 512]> self = ?", "Tensor<[1, 512]> other = ?"],
        ["Tensor<[1, 768]> self = ?", "Tensor<[1, 768]> other = ?"],
        ["Tensor<[2, 7, 2048]> self = ?", "Tensor<[2, 7, 2048]> other = ?"],
        ["Tensor<[1, 50, 3072]> self = ?", "Tensor<[1, 50, 3072]> other = ?"],
        ["Tensor<[1, 64, 1, 1]> self = ?", "Tensor other = 1e-05"],
        ["Tensor<[1, 64, 360, 640]> self = ?", "Tensor<[1, 64, 1, 1]> other = ?"],
        ["Tensor<[1, 64, 180, 320]> self = ?", "Tensor<[1, 64, 1, 1]> other = ?"],
        ["Tensor<[1, 256, 1, 1]> self = ?", "Tensor other = 1e-05"],
        ["Tensor<[1, 256, 180, 320]> self = ?", "Tensor<[1, 256, 1, 1]> other = ?"],
        ["Tensor<[1, 256, 180, 320]> self = ?", "Tensor<[1, 256, 180, 320]> other = ?"],
        ["Tensor<[1, 128, 1, 1]> self = ?", "Tensor other = 1e-05"],
        ["Tensor<[1, 128, 180, 320]> self = ?", "Tensor<[1, 128, 1, 1]> other = ?"],
        ["Tensor<[1, 128, 90, 160]> self = ?", "Tensor<[1, 128, 1, 1]> other = ?"],
        ["Tensor<[1, 512, 1, 1]> self = ?", "Tensor other = 1e-05"],
        ["Tensor<[1, 512, 90, 160]> self = ?", "Tensor<[1, 512, 1, 1]> other = ?"],
        ["Tensor<[1, 512, 90, 160]> self = ?", "Tensor<[1, 512, 90, 160]> other = ?"],
        ["Tensor<[1, 256, 90, 160]> self = ?", "Tensor<[1, 256, 1, 1]> other = ?"],
        ["Tensor<[1, 256, 45, 80]> self = ?", "Tensor<[1, 256, 1, 1]> other = ?"],
        ["Tensor<[1, 1024, 1, 1]> self = ?", "Tensor other = 1e-05"],
        ["Tensor<[1, 1024, 45, 80]> self = ?", "Tensor<[1, 1024, 1, 1]> other = ?"],
        ["Tensor<[1, 1024, 45, 80]> self = ?", "Tensor<[1, 1024, 45, 80]> other = ?"],
        ["Tensor<[1, 512, 45, 80]> self = ?", "Tensor<[1, 512, 1, 1]> other = ?"],
        ["Tensor<[1, 512, 23, 40]> self = ?", "Tensor<[1, 512, 1, 1]> other = ?"],
        ["Tensor<[1, 2048, 1, 1]> self = ?", "Tensor other = 1e-05"],
        ["Tensor<[1, 2048, 23, 40]> self = ?", "Tensor<[1, 2048, 1, 1]> other = ?"],
        ["Tensor<[1, 2048, 23, 40]> self = ?", "Tensor<[1, 2048, 23, 40]> other = ?"],
        ["Tensor<[1, 1, 40]> self = ?", "Tensor other = 1e-06"],
        ["Tensor<[1, 23, 1]> self = ?", "Tensor other = 1e-06"],
        ["Tensor<[920, 1, 256]> self = ?", "Tensor<[920, 1, 256]> other = ?"],
        ["Tensor<[920, 1, 256]> self = ?", "Tensor<[256]> other = ?"],
        ["Tensor<[100, 1, 256]> self = ?", "Tensor<[100, 1, 256]> other = ?"],
        ["Tensor<[1, 25, 768]> self = ?", "Tensor<[1, 25, 768]> other = ?"],
        ["Tensor<[1, 12, 25, 25]> self = ?", "Tensor<[1, 1, 1, 25]> other = ?"],
        ["Tensor<[1, 15, 1]> self = ?", "Tensor other = 1e-06"],
        ["Tensor<[15, 15]> self = ?", "Tensor other = 0"],
        ["Tensor<[15, 15]> self = ?", "Tensor other = 8"],
        ["Tensor<[15, 15]> self = ?", "Tensor<[15, 15]> other = ?"],
        ["Tensor<[1, 6, 15, 15]> self = ?", "Tensor<[1, 1, 1, 15]> other = ?"],
        ["Tensor<[1, 6, 15, 15]> self = ?", "Tensor<[1, 6, 15, 15]> other = ?"],
        ["Tensor<[1, 15, 512]> self = ?", "Tensor<[1, 15, 512]> other = ?"],
        ["Tensor<[1, 15, 1024]> self = ?", "Tensor<[1, 15, 1024]> other = ?"],
        ["Tensor<[1, 15, 1024]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 1e-06"],
        ["Tensor<[1, 1]> self = ?", "Tensor other = 16"],
        ["Tensor<[1, 1]> self = ?", "Tensor other = 0"],
        ["Tensor<[1, 6, 1, 1]> self = ?", "Tensor<[1, 1, 1, 1]> other = ?"],
        ["Tensor<[1, 6, 1, 1]> self = ?", "Tensor<[1, 6, 1, 1]> other = ?"],
        ["Tensor<[1, 1, 512]> self = ?", "Tensor<[1, 1, 512]> other = ?"],
        ["Tensor<[1, 6, 1, 15]> self = ?", "Tensor<[1, 1, 1, 15]> other = ?"],
        ["Tensor<[1, 6, 1, 15]> self = ?", "Tensor<[1, 6, 1, 15]> other = ?"],
        ["Tensor<[1, 1, 1024]> self = ?", "Tensor<[1, 1, 1024]> other = ?"],
        ["Tensor<[1, 1, 1024]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[2, 2]> self = ?", "Tensor other = 16"],
        ["Tensor<[2, 2]> self = ?", "Tensor other = 0"],
        ["Tensor<[1, 6, 1, 2]> self = ?", "Tensor<[1, 1, 1, 2]> other = ?"],
        ["Tensor<[1, 6, 1, 2]> self = ?", "Tensor<[1, 6, 1, 2]> other = ?"],
        ["Tensor<[s0 + 1, s0 + 1]> self = ?", "Tensor other = 16"],
        ["Tensor<[s0 + 1, s0 + 1]> self = ?", "Tensor other = 0"],
        ["Tensor<[1, 6, 1, s0 + 1]> self = ?", "Tensor<[1, 1, 1, s0 + 1]> other = ?"],
        ["Tensor<[1, 6, 1, s0 + 1]> self = ?", "Tensor<[1, 6, 1, s0 + 1]> other = ?"],
        ["Tensor<[17, 17]> self = ?", "Tensor other = 16"],
        ["Tensor<[17, 17]> self = ?", "Tensor other = 0"],
        ["Tensor<[1, 6, 1, 17]> self = ?", "Tensor<[1, 1, 1, 17]> other = ?"],
        ["Tensor<[1, 6, 1, 17]> self = ?", "Tensor<[1, 6, 1, 17]> other = ?"],
        ["Tensor<[1, 19200, 64]> self = ?", "Tensor<[1, 19200, 64]> other = ?"],
        ["Tensor<[1, 4800, 128]> self = ?", "Tensor<[1, 4800, 128]> other = ?"],
        ["Tensor<[1, 1200, 320]> self = ?", "Tensor<[1, 1200, 320]> other = ?"],
        ["Tensor<[1, 300, 512]> self = ?", "Tensor<[1, 300, 512]> other = ?"],
        ["Tensor<[1, 64, 30, 40]> self = ?", "Tensor<[1, 64, 30, 40]> other = ?"],
        ["Tensor<[1, 64, 60, 80]> self = ?", "Tensor<[1, 64, 60, 80]> other = ?"],
        ["Tensor<[1, 64, 120, 160]> self = ?", "Tensor<[1, 64, 120, 160]> other = ?"],
        ["Tensor<[1, 64, 240, 320]> self = ?", "Tensor<[1, 64, 240, 320]> other = ?"],
        ["Tensor<[1, 64, 480, 640]> self = ?", "Tensor<[1, 64, 480, 640]> other = ?"],
        ["Tensor<[1, 7, 768]> self = ?", "Tensor<[1, 7, 768]> other = ?"],
        ["Tensor<[1, 12, 7, 7]> self = ?", "Tensor<[1, 1, 1, 7]> other = ?"],
        ["Tensor<[1, 7, 3072]> self = ?", "Tensor<[1, 7, 3072]> other = ?"],
        ["Tensor<[1, 7, 3072]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 45, 768]> self = ?", "Tensor<[1, 45, 768]> other = ?"],
        ["Tensor<[1, 12, 45, 45]> self = ?", "Tensor<[1, 1, 45, 45]> other = ?"],
        ["Tensor<[1, 45, 3072]> self = ?", "Tensor<[1, 45, 3072]> other = ?"],
        ["Tensor<[1, 45, 3072]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 1, 768]> self = ?", "Tensor<[1, 1, 768]> other = ?"],
        ["Tensor<[1, 12, 1, 46]> self = ?", "Tensor<[1, 1, 1, 46]> other = ?"],
        ["Tensor<[1, 1, 3072]> self = ?", "Tensor<[1, 1, 3072]> other = ?"],
        ["Tensor<[1, 1, 3072]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 12, 1, s10 + 1]> self = ?", "Tensor<[1, 1, 1, s10 + 1]> other = ?"],
        ["Tensor<[]> self = ?", "Tensor other = 1"],
        ["Tensor<[1, 160, 7, 7]> self = ?", "Tensor<[1, 160, 7, 7]> other = ?"],
        ["Tensor<[1, 272, 7, 7]> self = ?", "Tensor<[1, 272, 7, 7]> other = ?"],
        ["Tensor<[1, 640, 7, 7]> self = ?", "Tensor<[1, 640, 7, 7]> other = ?"],
        ["Tensor<[1, 40, 14, 14]> self = ?", "Tensor<[1, 40, 14, 14]> other = ?"],
        ["Tensor<[1, 68, 14, 14]> self = ?", "Tensor<[1, 68, 14, 14]> other = ?"],
        ["Tensor<[1, 116, 14, 14]> self = ?", "Tensor<[1, 116, 14, 14]> other = ?"],
        ["Tensor<[1, 196, 14, 14]> self = ?", "Tensor<[1, 196, 14, 14]> other = ?"],
        ["Tensor<[1, 320, 14, 14]> self = ?", "Tensor<[1, 320, 14, 14]> other = ?"],
        ["Tensor<[1, 20, 28, 28]> self = ?", "Tensor<[1, 20, 28, 28]> other = ?"],
        ["Tensor<[1, 34, 28, 28]> self = ?", "Tensor<[1, 34, 28, 28]> other = ?"],
        ["Tensor<[1, 58, 28, 28]> self = ?", "Tensor<[1, 58, 28, 28]> other = ?"],
        ["Tensor<[1, 98, 28, 28]> self = ?", "Tensor<[1, 98, 28, 28]> other = ?"],
        ["Tensor<[1, 256, 28, 28]> self = ?", "Tensor<[1, 256, 28, 28]> other = ?"],
        ["Tensor<[1, 16, 28, 28]> self = ?", "Tensor<[1, 16, 28, 28]> other = ?"],
        ["Tensor<[1, 28, 28, 28]> self = ?", "Tensor<[1, 28, 28, 28]> other = ?"],
        ["Tensor<[1, 46, 28, 28]> self = ?", "Tensor<[1, 46, 28, 28]> other = ?"],
        ["Tensor<[1, 78, 28, 28]> self = ?", "Tensor<[1, 78, 28, 28]> other = ?"],
        ["Tensor<[1, 128, 28, 28]> self = ?", "Tensor<[1, 128, 28, 28]> other = ?"],
        ["Tensor<[1, 14, 56, 56]> self = ?", "Tensor<[1, 14, 56, 56]> other = ?"],
        ["Tensor<[1, 24, 56, 56]> self = ?", "Tensor<[1, 24, 56, 56]> other = ?"],
        ["Tensor<[1, 40, 56, 56]> self = ?", "Tensor<[1, 40, 56, 56]> other = ?"],
        ["Tensor<[1, 64, 56, 56]> self = ?", "Tensor<[1, 64, 56, 56]> other = ?"],
        ["Tensor<[1, 256, 512]> self = ?", "Tensor<[1, 256, 512]> other = ?"],
        ["Tensor<[1, 3, 320, 320]> self = ?", "Tensor<[1, 3, 320, 320]> other = ?"],
        ["Tensor<[1, 16, 160, 160]> self = ?", "Tensor<[1, 16, 160, 160]> other = ?"],
        ["Tensor<[1, 24, 80, 80]> self = ?", "Tensor<[1, 24, 80, 80]> other = ?"],
        ["Tensor<[1, 40, 40, 40]> self = ?", "Tensor<[1, 40, 40, 40]> other = ?"],
        ["Tensor<[1, 80, 20, 20]> self = ?", "Tensor<[1, 80, 20, 20]> other = ?"],
        ["Tensor<[1, 112, 20, 20]> self = ?", "Tensor<[1, 112, 20, 20]> other = ?"],
        ["Tensor<[1, 80, 10, 10]> self = ?", "Tensor<[1, 80, 10, 10]> other = ?"],
        ["Tensor<[3234, 2]> self = ?", "Tensor<[3234, 2]> other = ?"],
        ["Tensor<[1, 32, 28, 28]> self = ?", "Tensor<[1, 32, 28, 28]> other = ?"],
        ["Tensor<[1, 64, 14, 14]> self = ?", "Tensor<[1, 64, 14, 14]> other = ?"],
        ["Tensor<[1, 96, 14, 14]> self = ?", "Tensor<[1, 96, 14, 14]> other = ?"],
        ["Tensor<[1, 59]> self = ?", "Tensor other = 2"],
        ["Tensor<[1, 59, 1024]> self = ?", "Tensor<[1, 59, 1024]> other = ?"],
        ["Tensor<[1, 16, 59, 59]> self = ?", "Tensor<[1, 1, 59, 59]> other = ?"],
        ["Tensor<[59, 1024]> self = ?", "Tensor<[59, 1024]> other = ?"],
        ["Tensor<[1, 1]> self = ?", "Tensor other = 2"],
        ["Tensor<[1, 16, 1, 60]> self = ?", "Tensor<[1, 1, 1, 60]> other = ?"],
        ["Tensor<[1, 1024]> self = ?", "Tensor<[1, 1024]> other = ?"],
        ["Tensor<[1, 16, 1, s10 + 1]> self = ?", "Tensor<[1, 1, 1, s10 + 1]> other = ?"],
        ["Tensor<[1, 185, 28, 28]> self = ?", "Tensor<[1, 185, 28, 28]> other = ?"],
        ["Tensor<[1, 2048, 768]> self = ?", "Tensor<[2048, 768]> other = ?"],
        ["Tensor<[1, 8, 256, 2048]> self = ?", "Tensor<[1, 1, 1, 2048]> other = ?"],
        ["Tensor<[1, 256, 1280]> self = ?", "Tensor<[1, 256, 1280]> other = ?"],
        ["Tensor<[1, 2048, 768]> self = ?", "Tensor<[1, 2048, 768]> other = ?"],
        ["Tensor<[2048, 262]> self = ?", "Tensor<[262]> other = ?"],
        ["Tensor<[1, 256, 14, 14]> self = ?", "Tensor<[1, 256, 14, 14]> other = ?"],
        ["Tensor<[1, 512, 7, 7]> self = ?", "Tensor<[1, 512, 7, 7]> other = ?"],
        ["Tensor<[1, 256, 56, 56]> self = ?", "Tensor<[1, 256, 56, 56]> other = ?"],
        ["Tensor<[1, 512, 28, 28]> self = ?", "Tensor<[1, 512, 28, 28]> other = ?"],
        ["Tensor<[1, 1024, 14, 14]> self = ?", "Tensor<[1, 1024, 14, 14]> other = ?"],
        ["Tensor<[1, 2048, 7, 7]> self = ?", "Tensor<[1, 2048, 7, 7]> other = ?"],
        ["Tensor<[1, 10]> self = ?", "Tensor other = 0"],
        ["Tensor<[1, 10]> self = ?", "Tensor other = 1"],
        ["Tensor<[1, 10, 768]> self = ?", "Tensor<[1, 10, 768]> other = ?"],
        ["Tensor<[1, 12, 10, 10]> self = ?", "Tensor<[1, 1, 1, 10]> other = ?"],
        ["Tensor<[1, 16384, 32]> self = ?", "Tensor<[1, 16384, 32]> other = ?"],
        ["Tensor<[1, 4096, 64]> self = ?", "Tensor<[1, 4096, 64]> other = ?"],
        ["Tensor<[1, 1024, 160]> self = ?", "Tensor<[1, 1024, 160]> other = ?"],
        ["Tensor<[1, 256, 256]> self = ?", "Tensor<[1, 256, 256]> other = ?"],
        ["Tensor<[1, 16384, 256]> self = ?", "Tensor<[256]> other = ?"],
        ["Tensor<[1, 256, 128, 128]> self = ?", "Tensor<[1, 256, 128, 128]> other = ?"],
        ["Tensor<[1, 4096, 256]> self = ?", "Tensor<[256]> other = ?"],
        ["Tensor<[1, 1024, 256]> self = ?", "Tensor<[256]> other = ?"],
        ["Tensor<[1, 256, 256]> self = ?", "Tensor<[256]> other = ?"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9857142856344581"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9714285712689161"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9571428559720516"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9428571425378323"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9285714253783226"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9142857119441032"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.8999999985098839"],
        ["Tensor<[1, 256, 16, 16]> self = ?", "Tensor<[1, 256, 16, 16]> other = ?"],
        ["Tensor<[1, 256, 32, 32]> self = ?", "Tensor<[1, 256, 32, 32]> other = ?"],
        ["Tensor<[1, 256, 64, 64]> self = ?", "Tensor<[1, 256, 64, 64]> other = ?"],
        ["Tensor<[1, 160, 32, 32]> self = ?", "Tensor<[1, 160, 32, 32]> other = ?"],
        ["Tensor<[1, 256, 160]> self = ?", "Tensor<[1, 256, 160]> other = ?"],
        ["Tensor<[1, 64, 64, 64]> self = ?", "Tensor<[1, 64, 64, 64]> other = ?"],
        ["Tensor<[1, 256, 64]> self = ?", "Tensor<[1, 256, 64]> other = ?"],
        ["Tensor<[1, 32, 128, 128]> self = ?", "Tensor<[1, 32, 128, 128]> other = ?"],
        ["Tensor<[1, 256, 32]> self = ?", "Tensor<[1, 256, 32]> other = ?"],
        ["Tensor<[1, 8, 768]> self = ?", "Tensor<[1, 8, 768]> other = ?"],
        ["Tensor<[1, 320, 64, 64]> self = ?", "Tensor<[1, 320, 1, 1]> other = ?"],
        ["Tensor<[1, 320, 64, 64]> self = ?", "Tensor<[1, 320, 64, 64]> other = ?"],
        ["Tensor<[1, 4096, 320]> self = ?", "Tensor<[1, 4096, 320]> other = ?"],
        ["Tensor<[1, 640, s0, s1]> self = ?", "Tensor<[1, 640, 1, 1]> other = ?"],
        ["Tensor<[1, 640, s0, s1]> self = ?", "Tensor<[1, 640, s0, s1]> other = ?"],
        ["Tensor<[1, s0*s1, 640]> self = ?", "Tensor<[1, s0*s1, 640]> other = ?"],
        ["Tensor<[1, 1280, s1, s2]> self = ?", "Tensor<[1, 1280, 1, 1]> other = ?"],
        ["Tensor<[1, 1280, s1, s2]> self = ?", "Tensor<[1, 1280, s1, s2]> other = ?"],
        ["Tensor<[1, s1*s2, 1280]> self = ?", "Tensor<[1, s1*s2, 1280]> other = ?"],
        ["Tensor<[1, 1280, s0, s1]> self = ?", "Tensor<[1, 1280, 1, 1]> other = ?"],
        ["Tensor<[1, 1280, s0, s1]> self = ?", "Tensor<[1, 1280, s0, s1]> other = ?"],
        ["Tensor<[1, s0*s1, 1280]> self = ?", "Tensor<[1, s0*s1, 1280]> other = ?"],
        ["Tensor<[1, 1280, 8, 8]> self = ?", "Tensor<[1, 1280, 1, 1]> other = ?"],
        ["Tensor<[1, 1280, 8, 8]> self = ?", "Tensor<[1, 1280, 8, 8]> other = ?"],
        ["Tensor<[2*s0]> self = ?", "Tensor other = 0.0"],
        ["Tensor<[2*s1]> self = ?", "Tensor other = 0.0"],
        ["Tensor<[1, 640, s1, s2]> self = ?", "Tensor<[1, 640, 1, 1]> other = ?"],
        ["Tensor<[1, 640, s1, s2]> self = ?", "Tensor<[1, 640, s1, s2]> other = ?"],
        ["Tensor<[1, s1*s2, 640]> self = ?", "Tensor<[1, s1*s2, 640]> other = ?"],
        ["Tensor<[2*s2]> self = ?", "Tensor other = 0.0"],
        ["Tensor<[1, 320, s1, s2]> self = ?", "Tensor<[1, 320, 1, 1]> other = ?"],
        ["Tensor<[1, 320, s1, s2]> self = ?", "Tensor<[1, 320, s1, s2]> other = ?"],
        ["Tensor<[1, s1*s2, 320]> self = ?", "Tensor<[1, s1*s2, 320]> other = ?"],
        ["Tensor<[1, 128, 64, 64]> self = ?", "Tensor<[1, 128, 64, 64]> other = ?"],
        ["Tensor<[1, 64, 128, 128]> self = ?", "Tensor<[1, 64, 128, 128]> other = ?"],
        ["Tensor<[1, 32, 256, 256]> self = ?", "Tensor<[1, 32, 256, 256]> other = ?"],
        ["Tensor<[1, 128, 112, 112]> self = ?", "Tensor<[1, 128, 112, 112]> other = ?"],
        ["Tensor<[1, 64, 224, 224]> self = ?", "Tensor<[1, 64, 224, 224]> other = ?"],
        ["Tensor<[1, 1500, 768]> self = ?", "Tensor<[1500, 768]> other = ?"],
        ["Tensor<[1, 1500, 768]> self = ?", "Tensor<[1, 1500, 768]> other = ?"],
        ["Tensor<[1, 1, 768]> self = ?", "Tensor<[1, 768]> other = ?"],
        ["Tensor<[1, 4, 768]> self = ?", "Tensor<[4, 768]> other = ?"],
        ["Tensor<[1, 4, 768]> self = ?", "Tensor<[1, 4, 768]> other = ?"],
        ["Tensor<[1, 19, 1024]> self = ?", "Tensor<[1, 19, 1024]> other = ?"],
        ["Tensor<[1, 16, 19, 19]> self = ?", "Tensor<[1, 1, 19, 19]> other = ?"],
        ["Tensor<[1, 1, 32, 1]> self = ?", "Tensor other = 1"],
        ["Tensor<[1, 1, 32, 1]> self = ?", "Tensor other = 2"],
        ["Tensor<[1, 1, 1, 42]> self = ?", "Tensor other = 1"],
        ["Tensor<[1, 1, 1, 42]> self = ?", "Tensor other = 2"],
        ["Tensor<[1, 1, 1, 42]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 1, 1, 42]> self = ?", "Tensor other = -6.0"],
        ["Tensor<[1, 192, 32, 42]> self = ?", "Tensor<[1, 192, 32, 42]> other = ?"],
        ["Tensor<[1, 1, 32, 1]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 1, 32, 1]> self = ?", "Tensor other = -6.0"],
        ["Tensor<[1, 1445, 192]> self = ?", "Tensor<[1, 1445, 192]> other = ?"],
        ["Tensor<[1, 64, 256, 256]> self = ?", "Tensor<[1, 64, 256, 256]> other = ?"],
        ["Tensor<[1, 128, 128, 128]> self = ?", "Tensor<[1, 128, 128, 128]> other = ?"],
        ["Tensor<[1, 512, 32, 32]> self = ?", "Tensor<[1, 512, 32, 32]> other = ?"],
        ["Tensor<[1, 1024, 16, 16]> self = ?", "Tensor<[1, 1024, 16, 16]> other = ?"],
        ["Tensor<[1, 9, 128]> self = ?", "Tensor<[1, 9, 128]> other = ?"],
        ["Tensor<[1, 12, 9, 9]> self = ?", "Tensor<[1, 1, 1, 9]> other = ?"],
        ["Tensor<[1, 9, 768]> self = ?", "Tensor<[1, 9, 768]> other = ?"],
        ["Tensor<[1, 9, 3072]> self = ?", "Tensor<[1, 9, 3072]> other = ?"],
        ["Tensor<[1, 9, 3072]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 9, 128]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 12, 128]> self = ?", "Tensor<[1, 12, 128]> other = ?"],
        ["Tensor<[1, 12, 12, 12]> self = ?", "Tensor<[1, 1, 1, 12]> other = ?"],
        ["Tensor<[1, 12, 768]> self = ?", "Tensor<[1, 12, 768]> other = ?"],
        ["Tensor<[1, 12, 3072]> self = ?", "Tensor<[1, 12, 3072]> other = ?"],
        ["Tensor<[1, 12, 3072]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 16, 9, 9]> self = ?", "Tensor<[1, 1, 1, 9]> other = ?"],
        ["Tensor<[1, 9, 1024]> self = ?", "Tensor<[1, 9, 1024]> other = ?"],
        ["Tensor<[1, 9, 4096]> self = ?", "Tensor<[1, 9, 4096]> other = ?"],
        ["Tensor<[1, 9, 4096]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 9, 2048]> self = ?", "Tensor<[1, 9, 2048]> other = ?"],
        ["Tensor<[1, 9, 8192]> self = ?", "Tensor<[1, 9, 8192]> other = ?"],
        ["Tensor<[1, 9, 8192]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 64, 9, 9]> self = ?", "Tensor<[1, 1, 1, 9]> other = ?"],
        ["Tensor<[1, 9, 16384]> self = ?", "Tensor<[1, 9, 16384]> other = ?"],
        ["Tensor<[1, 9, 16384]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 5, 16, 32]> self = ?", "Tensor<[1, 5, 16, 32]> other = ?"],
        ["Tensor<[1, 16, 5, 5]> self = ?", "Tensor<[1, 1, 1, 5]> other = ?"],
        ["Tensor<[1, 5, 4096]> self = ?", "Tensor<[1, 5, 4096]> other = ?"],
        ["Tensor<[1, 5, 4096]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 5, 1024]> self = ?", "Tensor<[1, 5, 1024]> other = ?"],
        ["Tensor<[1, 1, 16, 32]> self = ?", "Tensor<[1, 1, 16, 32]> other = ?"],
        ["Tensor<[1, 16, 1, 6]> self = ?", "Tensor<[1, 1, 1, 6]> other = ?"],
        ["Tensor<[1, 1, 4096]> self = ?", "Tensor<[1, 1, 4096]> other = ?"],
        ["Tensor<[1, 1, 4096]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 16, 768]> self = ?", "Tensor<[1, 16, 768]> other = ?"],
        ["Tensor<[1, 256, 7, 7]> self = ?", "Tensor<[1, 256, 7, 7]> other = ?"],
        ["Tensor<[1, 32, 112, 112]> self = ?", "Tensor<[1, 32, 112, 112]> other = ?"],
        ["Tensor<[1, 1024, 7, 7]> self = ?", "Tensor<[1, 1024, 7, 7]> other = ?"],
        ["Tensor<[1, 224, 7, 7]> self = ?", "Tensor<[1, 224, 7, 7]> other = ?"],
        ["Tensor<[1, 768, 7, 7]> self = ?", "Tensor<[1, 768, 7, 7]> other = ?"],
        ["Tensor<[1, 768, 14, 14]> self = ?", "Tensor<[1, 768, 14, 14]> other = ?"],
        ["Tensor<[1, 192, 14, 14]> self = ?", "Tensor<[1, 192, 14, 14]> other = ?"],
        ["Tensor<[1, 512, 14, 14]> self = ?", "Tensor<[1, 512, 14, 14]> other = ?"],
        ["Tensor<[1, 160, 28, 28]> self = ?", "Tensor<[1, 160, 28, 28]> other = ?"],
        ["Tensor<[1, 128, 56, 56]> self = ?", "Tensor<[1, 128, 56, 56]> other = ?"],
        ["Tensor<[1, 197, 768]> self = ?", "Tensor<[1, 197, 768]> other = ?"],
        ["Tensor<[1, 16, 112, 112]> self = ?", "Tensor<[1, 16, 112, 112]> other = ?"],
        ["Tensor<[1, 40, 28, 28]> self = ?", "Tensor<[1, 40, 28, 28]> other = ?"],
        ["Tensor<[1, 80, 14, 14]> self = ?", "Tensor<[1, 80, 14, 14]> other = ?"],
        ["Tensor<[1, 112, 14, 14]> self = ?", "Tensor<[1, 112, 14, 14]> other = ?"],
        ["Tensor<[1, 80, 7, 7]> self = ?", "Tensor<[1, 80, 7, 7]> other = ?"],
        ["Tensor<[1, 960, 7, 7]> self = ?", "Tensor<[1, 960, 7, 7]> other = ?"],
        ["Tensor<[1, 480, 7, 7]> self = ?", "Tensor<[1, 480, 7, 7]> other = ?"],
        ["Tensor<[1, 672, 7, 7]> self = ?", "Tensor<[1, 672, 7, 7]> other = ?"],
        ["Tensor<[1, 336, 14, 14]> self = ?", "Tensor<[1, 336, 14, 14]> other = ?"],
        ["Tensor<[1, 56, 14, 14]> self = ?", "Tensor<[1, 56, 14, 14]> other = ?"],
        ["Tensor<[1, 672, 14, 14]> self = ?", "Tensor<[1, 672, 14, 14]> other = ?"],
        ["Tensor<[1, 480, 14, 14]> self = ?", "Tensor<[1, 480, 14, 14]> other = ?"],
        ["Tensor<[1, 240, 14, 14]> self = ?", "Tensor<[1, 240, 14, 14]> other = ?"],
        ["Tensor<[1, 92, 14, 14]> self = ?", "Tensor<[1, 92, 14, 14]> other = ?"],
        ["Tensor<[1, 100, 14, 14]> self = ?", "Tensor<[1, 100, 14, 14]> other = ?"],
        ["Tensor<[1, 120, 28, 28]> self = ?", "Tensor<[1, 120, 28, 28]> other = ?"],
        ["Tensor<[1, 60, 28, 28]> self = ?", "Tensor<[1, 60, 28, 28]> other = ?"],
        ["Tensor<[1, 72, 28, 28]> self = ?", "Tensor<[1, 72, 28, 28]> other = ?"],
        ["Tensor<[1, 36, 56, 56]> self = ?", "Tensor<[1, 36, 56, 56]> other = ?"],
        ["Tensor<[1, 12, 56, 56]> self = ?", "Tensor<[1, 12, 56, 56]> other = ?"],
        ["Tensor<[1, 24, 112, 112]> self = ?", "Tensor<[1, 24, 112, 112]> other = ?"],
        ["Tensor<[1, 8, 112, 112]> self = ?", "Tensor<[1, 8, 112, 112]> other = ?"],
        ["Tensor<[1, 1, 224, 224]> self = ?", "Tensor other = -0.030000000000000027"],
        ["Tensor<[1, 1, 224, 224]> self = ?", "Tensor other = -0.08799999999999997"],
        ["Tensor<[1, 1, 224, 224]> self = ?", "Tensor other = -0.18799999999999994"],
        ["Tensor<[1, 18, 56, 56]> self = ?", "Tensor<[1, 18, 56, 56]> other = ?"],
        ["Tensor<[1, 36, 28, 28]> self = ?", "Tensor<[1, 36, 28, 28]> other = ?"],
        ["Tensor<[1, 72, 14, 14]> self = ?", "Tensor<[1, 72, 14, 14]> other = ?"],
        ["Tensor<[1, 144, 7, 7]> self = ?", "Tensor<[1, 144, 7, 7]> other = ?"],
        ["Tensor<[1, 512, 8, 8]> self = ?", "Tensor<[1, 512, 8, 8]> other = ?"],
        ["Tensor<[1, 1536, 8, 8]> self = ?", "Tensor<[1, 1536, 8, 8]> other = ?"],
        ["Tensor<[1, 384, 8, 8]> self = ?", "Tensor<[1, 384, 8, 8]> other = ?"],
        ["Tensor<[1, 1024, 17, 17]> self = ?", "Tensor<[1, 1024, 17, 17]> other = ?"],
        ["Tensor<[1, 384, 35, 35]> self = ?", "Tensor<[1, 384, 35, 35]> other = ?"],
        ["Tensor<[1, 192, 71, 71]> self = ?", "Tensor<[1, 192, 71, 71]> other = ?"],
        ["Tensor<[1, 160, 73, 73]> self = ?", "Tensor<[1, 160, 73, 73]> other = ?"],
        ["Tensor<[1, 64, 147, 147]> self = ?", "Tensor<[1, 64, 147, 147]> other = ?"],
        ["Tensor<[1, 12, 197, 197]> self = ?", "Tensor<[1, 12, 197, 197]> other = ?"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9909090911969543"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9818181823939085"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9727272726595402"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.963636364787817"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9545454569160938"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.94545454159379"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9363636374473572"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9272727221250534"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9181818142533302"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9090909063816071"],
        ["Tensor<[1, 16, 197, 197]> self = ?", "Tensor<[1, 16, 197, 197]> other = ?"],
        ["Tensor<[1, 197, 1024]> self = ?", "Tensor<[1, 197, 1024]> other = ?"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9956521736457944"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9913043472915888"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9869565209373832"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9826086945831776"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9782608672976494"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9739130418747663"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9695652164518833"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9652173891663551"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.960869561880827"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9565217345952988"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9521739110350609"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.947826087474823"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9434782639145851"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9391304366290569"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9347826093435287"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9304347857832909"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9260869547724724"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9217391312122345"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.917391300201416"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9130434766411781"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9086956530809402"],
        ["Tensor<[1, 1, 1]> self = ?", "Tensor other = 0.9043478220701218"],
        ["Tensor<[1, 768, 384]> self = ?", "Tensor<[384]> other = ?"],
        ["Tensor<[1, 196, 768]> self = ?", "Tensor<[1, 196, 768]> other = ?"],
        ["Tensor<[1, 24, 28, 28]> self = ?", "Tensor<[1, 24, 28, 28]> other = ?"],
        ["Tensor<[1, 48, 14, 14]> self = ?", "Tensor<[1, 48, 14, 14]> other = ?"],
        ["Tensor<[1, 96, 7, 7]> self = ?", "Tensor<[1, 96, 7, 7]> other = ?"],
        ["Tensor<[1, 896, 14, 14]> self = ?", "Tensor<[1, 896, 14, 14]> other = ?"],
        ["Tensor<[1, 72, 56, 56]> self = ?", "Tensor<[1, 72, 56, 56]> other = ?"],
        ["Tensor<[1, 168, 28, 28]> self = ?", "Tensor<[1, 168, 28, 28]> other = ?"],
        ["Tensor<[1, 408, 14, 14]> self = ?", "Tensor<[1, 408, 14, 14]> other = ?"],
        ["Tensor<[1, 912, 7, 7]> self = ?", "Tensor<[1, 912, 7, 7]> other = ?"],
        ["Tensor<[1, 336, 56, 56]> self = ?", "Tensor<[1, 336, 56, 56]> other = ?"],
        ["Tensor<[1, 672, 28, 28]> self = ?", "Tensor<[1, 672, 28, 28]> other = ?"],
        ["Tensor<[1, 1344, 14, 14]> self = ?", "Tensor<[1, 1344, 14, 14]> other = ?"],
        ["Tensor<[1, 2520, 7, 7]> self = ?", "Tensor<[1, 2520, 7, 7]> other = ?"],
        ["Tensor<[1, 96, 56, 56]> self = ?", "Tensor<[1, 96, 56, 56]> other = ?"],
        ["Tensor<[1, 192, 28, 28]> self = ?", "Tensor<[1, 192, 28, 28]> other = ?"],
        ["Tensor<[1, 432, 14, 14]> self = ?", "Tensor<[1, 432, 14, 14]> other = ?"],
        ["Tensor<[1, 1008, 7, 7]> self = ?", "Tensor<[1, 1008, 7, 7]> other = ?"],
        ["Tensor<[1, 32, 56, 56]> self = ?", "Tensor<[1, 32, 56, 56]> other = ?"],
        ["Tensor<[1, 64, 28, 28]> self = ?", "Tensor<[1, 64, 28, 28]> other = ?"],
        ["Tensor<[1, 160, 14, 14]> self = ?", "Tensor<[1, 160, 14, 14]> other = ?"],
        ["Tensor<[1, 400, 7, 7]> self = ?", "Tensor<[1, 400, 7, 7]> other = ?"],
        ["Tensor<[1, 288, 14, 14]> self = ?", "Tensor<[1, 288, 14, 14]> other = ?"],
        ["Tensor<[1, 80, 56, 56]> self = ?", "Tensor<[1, 80, 56, 56]> other = ?"],
        ["Tensor<[1, 240, 28, 28]> self = ?", "Tensor<[1, 240, 28, 28]> other = ?"],
        ["Tensor<[1, 720, 14, 14]> self = ?", "Tensor<[1, 720, 14, 14]> other = ?"],
        ["Tensor<[1, 1920, 7, 7]> self = ?", "Tensor<[1, 1920, 7, 7]> other = ?"],
        ["Tensor<[1, 528, 96, 96]> self = ?", "Tensor<[1, 528, 96, 96]> other = ?"],
        ["Tensor<[1, 1056, 48, 48]> self = ?", "Tensor<[1, 1056, 48, 48]> other = ?"],
        ["Tensor<[1, 2904, 24, 24]> self = ?", "Tensor<[1, 2904, 24, 24]> other = ?"],
        ["Tensor<[1, 7392, 12, 12]> self = ?", "Tensor<[1, 7392, 12, 12]> other = ?"],
        ["Tensor<[1, 224, 56, 56]> self = ?", "Tensor<[1, 224, 56, 56]> other = ?"],
        ["Tensor<[1, 448, 28, 28]> self = ?", "Tensor<[1, 448, 28, 28]> other = ?"],
        ["Tensor<[1, 1232, 14, 14]> self = ?", "Tensor<[1, 1232, 14, 14]> other = ?"],
        ["Tensor<[1, 3024, 7, 7]> self = ?", "Tensor<[1, 3024, 7, 7]> other = ?"],
        ["Tensor<[1, 48, 56, 56]> self = ?", "Tensor<[1, 48, 56, 56]> other = ?"],
        ["Tensor<[1, 888, 7, 7]> self = ?", "Tensor<[1, 888, 7, 7]> other = ?"],
        ["Tensor<[1, 232, 56, 56]> self = ?", "Tensor<[1, 232, 56, 56]> other = ?"],
        ["Tensor<[1, 696, 28, 28]> self = ?", "Tensor<[1, 696, 28, 28]> other = ?"],
        ["Tensor<[1, 1392, 14, 14]> self = ?", "Tensor<[1, 1392, 14, 14]> other = ?"],
        ["Tensor<[1, 3712, 7, 7]> self = ?", "Tensor<[1, 3712, 7, 7]> other = ?"],
        ["Tensor<[1, 216, 28, 28]> self = ?", "Tensor<[1, 216, 28, 28]> other = ?"],
        ["Tensor<[1, 576, 14, 14]> self = ?", "Tensor<[1, 576, 14, 14]> other = ?"],
        ["Tensor<[1, 1512, 7, 7]> self = ?", "Tensor<[1, 1512, 7, 7]> other = ?"],
        ["Tensor<[1, 104, 28, 28]> self = ?", "Tensor<[1, 104, 28, 28]> other = ?"],
        ["Tensor<[1, 208, 14, 14]> self = ?", "Tensor<[1, 208, 14, 14]> other = ?"],
        ["Tensor<[1, 440, 7, 7]> self = ?", "Tensor<[1, 440, 7, 7]> other = ?"],
        ["Tensor<[1, 144, 28, 28]> self = ?", "Tensor<[1, 144, 28, 28]> other = ?"],
        ["Tensor<[1, 784, 7, 7]> self = ?", "Tensor<[1, 784, 7, 7]> other = ?"],
        ["Tensor<[1, 2016, 7, 7]> self = ?", "Tensor<[1, 2016, 7, 7]> other = ?"],
        ["Tensor<[1, 3, 800, 1066]> self = ?", "Tensor<[1, 3, 800, 1066]> other = ?"],
        ["Tensor<[1, 64, 1, 1]> self = ?", "Tensor other = 0.0"],
        ["Tensor<[1, 64, 400, 544]> self = ?", "Tensor<[1, 64, 1, 1]> other = ?"],
        ["Tensor<[1, 64, 200, 272]> self = ?", "Tensor<[1, 64, 1, 1]> other = ?"],
        ["Tensor<[1, 256, 1, 1]> self = ?", "Tensor other = 0.0"],
        ["Tensor<[1, 256, 200, 272]> self = ?", "Tensor<[1, 256, 1, 1]> other = ?"],
        ["Tensor<[1, 256, 200, 272]> self = ?", "Tensor<[1, 256, 200, 272]> other = ?"],
        ["Tensor<[1, 128, 1, 1]> self = ?", "Tensor other = 0.0"],
        ["Tensor<[1, 128, 200, 272]> self = ?", "Tensor<[1, 128, 1, 1]> other = ?"],
        ["Tensor<[1, 128, 100, 136]> self = ?", "Tensor<[1, 128, 1, 1]> other = ?"],
        ["Tensor<[1, 512, 1, 1]> self = ?", "Tensor other = 0.0"],
        ["Tensor<[1, 512, 100, 136]> self = ?", "Tensor<[1, 512, 1, 1]> other = ?"],
        ["Tensor<[1, 512, 100, 136]> self = ?", "Tensor<[1, 512, 100, 136]> other = ?"],
        ["Tensor<[1, 256, 100, 136]> self = ?", "Tensor<[1, 256, 1, 1]> other = ?"],
        ["Tensor<[1, 256, 50, 68]> self = ?", "Tensor<[1, 256, 1, 1]> other = ?"],
        ["Tensor<[1, 1024, 1, 1]> self = ?", "Tensor other = 0.0"],
        ["Tensor<[1, 1024, 50, 68]> self = ?", "Tensor<[1, 1024, 1, 1]> other = ?"],
        ["Tensor<[1, 1024, 50, 68]> self = ?", "Tensor<[1, 1024, 50, 68]> other = ?"],
        ["Tensor<[1, 512, 50, 68]> self = ?", "Tensor<[1, 512, 1, 1]> other = ?"],
        ["Tensor<[1, 512, 25, 34]> self = ?", "Tensor<[1, 512, 1, 1]> other = ?"],
        ["Tensor<[1, 2048, 1, 1]> self = ?", "Tensor other = 0.0"],
        ["Tensor<[1, 2048, 25, 34]> self = ?", "Tensor<[1, 2048, 1, 1]> other = ?"],
        ["Tensor<[1, 2048, 25, 34]> self = ?", "Tensor<[1, 2048, 25, 34]> other = ?"],
        ["Tensor<[1, 256, 50, 68]> self = ?", "Tensor<[1, 256, 50, 68]> other = ?"],
        ["Tensor<[1, 256, 100, 136]> self = ?", "Tensor<[1, 256, 100, 136]> other = ?"],
        ["Tensor<[13600, 1, 4]> self = ?", "Tensor<[1, 9, 4]> other = ?"],
        ["Tensor<[3400, 1, 4]> self = ?", "Tensor<[1, 9, 4]> other = ?"],
        ["Tensor<[850, 1, 4]> self = ?", "Tensor<[1, 9, 4]> other = ?"],
        ["Tensor<[221, 1, 4]> self = ?", "Tensor<[1, 9, 4]> other = ?"],
        ["Tensor<[63, 1, 4]> self = ?", "Tensor<[1, 9, 4]> other = ?"],
        ["Tensor<[1, 24, 768]> self = ?", "Tensor<[1, 24, 768]> other = ?"],
        ["Tensor<[24, 24]> self = ?", "Tensor other = 160"],
        ["Tensor<[12, 24, 24]> self = ?", "Tensor<[12, 24, 24]> other = ?"],
        ["Tensor<[1, 12, 24, 24]> self = ?", "Tensor<[1, 1, 24, 24]> other = ?"],
        ["Tensor<[1, 3, 300, 300]> self = ?", "Tensor<[1, 3, 300, 300]> other = ?"],
        ["Tensor<[8732, 2]> self = ?", "Tensor<[8732, 2]> other = ?"],
        ["Tensor<[64, 4, 49, 49]> self = ?", "Tensor<[1, 4, 49, 49]> other = ?"],
        ["Tensor<[1, 56, 56, 128]> self = ?", "Tensor<[1, 56, 56, 128]> other = ?"],
        ["Tensor<[1, 64, 4, 49, 49]> self = ?", "Tensor<[1, 64, 1, 49, 49]> other = ?"],
        ["Tensor<[16, 8, 49, 49]> self = ?", "Tensor<[1, 8, 49, 49]> other = ?"],
        ["Tensor<[1, 28, 28, 256]> self = ?", "Tensor<[1, 28, 28, 256]> other = ?"],
        ["Tensor<[1, 16, 8, 49, 49]> self = ?", "Tensor<[1, 16, 1, 49, 49]> other = ?"],
        ["Tensor<[4, 16, 49, 49]> self = ?", "Tensor<[1, 16, 49, 49]> other = ?"],
        ["Tensor<[1, 14, 14, 512]> self = ?", "Tensor<[1, 14, 14, 512]> other = ?"],
        ["Tensor<[1, 4, 16, 49, 49]> self = ?", "Tensor<[1, 4, 1, 49, 49]> other = ?"],
        ["Tensor<[1, 32, 49, 49]> self = ?", "Tensor<[1, 32, 49, 49]> other = ?"],
        ["Tensor<[1, 7, 7, 1024]> self = ?", "Tensor<[1, 7, 7, 1024]> other = ?"],
        ["Tensor<[64, 3, 49, 49]> self = ?", "Tensor<[1, 3, 49, 49]> other = ?"],
        ["Tensor<[1, 56, 56, 96]> self = ?", "Tensor<[1, 56, 56, 96]> other = ?"],
        ["Tensor<[1, 64, 3, 49, 49]> self = ?", "Tensor<[1, 64, 1, 49, 49]> other = ?"],
        ["Tensor<[16, 6, 49, 49]> self = ?", "Tensor<[1, 6, 49, 49]> other = ?"],
        ["Tensor<[1, 28, 28, 192]> self = ?", "Tensor<[1, 28, 28, 192]> other = ?"],
        ["Tensor<[1, 16, 6, 49, 49]> self = ?", "Tensor<[1, 16, 1, 49, 49]> other = ?"],
        ["Tensor<[4, 12, 49, 49]> self = ?", "Tensor<[1, 12, 49, 49]> other = ?"],
        ["Tensor<[1, 14, 14, 384]> self = ?", "Tensor<[1, 14, 14, 384]> other = ?"],
        ["Tensor<[1, 4, 12, 49, 49]> self = ?", "Tensor<[1, 4, 1, 49, 49]> other = ?"],
        ["Tensor<[1, 24, 49, 49]> self = ?", "Tensor<[1, 24, 49, 49]> other = ?"],
        ["Tensor<[1, 7, 7, 768]> self = ?", "Tensor<[1, 7, 7, 768]> other = ?"],
        ["Tensor<[64, 4, 64, 64]> self = ?", "Tensor<[1, 4, 64, 64]> other = ?"],
        ["Tensor<[1, 64, 64, 128]> self = ?", "Tensor<[1, 64, 64, 128]> other = ?"],
        ["Tensor<[1, 64, 4, 64, 64]> self = ?", "Tensor<[1, 64, 1, 64, 64]> other = ?"],
        ["Tensor<[16, 8, 64, 64]> self = ?", "Tensor<[1, 8, 64, 64]> other = ?"],
        ["Tensor<[1, 32, 32, 256]> self = ?", "Tensor<[1, 32, 32, 256]> other = ?"],
        ["Tensor<[1, 16, 8, 64, 64]> self = ?", "Tensor<[1, 16, 1, 64, 64]> other = ?"],
        ["Tensor<[4, 16, 64, 64]> self = ?", "Tensor<[1, 16, 64, 64]> other = ?"],
        ["Tensor<[1, 16, 16, 512]> self = ?", "Tensor<[1, 16, 16, 512]> other = ?"],
        ["Tensor<[1, 4, 16, 64, 64]> self = ?", "Tensor<[1, 4, 1, 64, 64]> other = ?"],
        ["Tensor<[1, 32, 64, 64]> self = ?", "Tensor<[1, 32, 64, 64]> other = ?"],
        ["Tensor<[1, 8, 8, 1024]> self = ?", "Tensor<[1, 8, 8, 1024]> other = ?"],
        ["Tensor<[64, 3, 64, 64]> self = ?", "Tensor<[1, 3, 64, 64]> other = ?"],
        ["Tensor<[1, 64, 64, 96]> self = ?", "Tensor<[1, 64, 64, 96]> other = ?"],
        ["Tensor<[1, 64, 3, 64, 64]> self = ?", "Tensor<[1, 64, 1, 64, 64]> other = ?"],
        ["Tensor<[16, 6, 64, 64]> self = ?", "Tensor<[1, 6, 64, 64]> other = ?"],
        ["Tensor<[1, 32, 32, 192]> self = ?", "Tensor<[1, 32, 32, 192]> other = ?"],
        ["Tensor<[1, 16, 6, 64, 64]> self = ?", "Tensor<[1, 16, 1, 64, 64]> other = ?"],
        ["Tensor<[4, 12, 64, 64]> self = ?", "Tensor<[1, 12, 64, 64]> other = ?"],
        ["Tensor<[1, 16, 16, 384]> self = ?", "Tensor<[1, 16, 16, 384]> other = ?"],
        ["Tensor<[1, 4, 12, 64, 64]> self = ?", "Tensor<[1, 4, 1, 64, 64]> other = ?"],
        ["Tensor<[1, 24, 64, 64]> self = ?", "Tensor<[1, 24, 64, 64]> other = ?"],
        ["Tensor<[1, 8, 8, 768]> self = ?", "Tensor<[1, 8, 8, 768]> other = ?"],
        ["Tensor<[1, 10, 1]> self = ?", "Tensor other = 1e-06"],
        ["Tensor<[10, 10]> self = ?", "Tensor other = 0"],
        ["Tensor<[10, 10]> self = ?", "Tensor other = 8"],
        ["Tensor<[10, 10]> self = ?", "Tensor<[10, 10]> other = ?"],
        ["Tensor<[1, 12, 10, 10]> self = ?", "Tensor<[1, 12, 10, 10]> other = ?"],
        ["Tensor<[1, 12, 1, 1]> self = ?", "Tensor<[1, 1, 1, 1]> other = ?"],
        ["Tensor<[1, 12, 1, 1]> self = ?", "Tensor<[1, 12, 1, 1]> other = ?"],
        ["Tensor<[1, 12, 1, 10]> self = ?", "Tensor<[1, 1, 1, 10]> other = ?"],
        ["Tensor<[1, 12, 1, 10]> self = ?", "Tensor<[1, 12, 1, 10]> other = ?"],
        ["Tensor<[1, 12, 1, 2]> self = ?", "Tensor<[1, 1, 1, 2]> other = ?"],
        ["Tensor<[1, 12, 1, 2]> self = ?", "Tensor<[1, 12, 1, 2]> other = ?"],
        ["Tensor<[1, 12, 1, s0 + 1]> self = ?", "Tensor<[1, 1, 1, s0 + 1]> other = ?"],
        ["Tensor<[1, 12, 1, s0 + 1]> self = ?", "Tensor<[1, 12, 1, s0 + 1]> other = ?"],
        ["Tensor<[1, 16, 10, 10]> self = ?", "Tensor<[1, 1, 1, 10]> other = ?"],
        ["Tensor<[1, 16, 10, 10]> self = ?", "Tensor<[1, 16, 10, 10]> other = ?"],
        ["Tensor<[1, 10, 1024]> self = ?", "Tensor<[1, 10, 1024]> other = ?"],
        ["Tensor<[1, 16, 1, 1]> self = ?", "Tensor<[1, 1, 1, 1]> other = ?"],
        ["Tensor<[1, 16, 1, 1]> self = ?", "Tensor<[1, 16, 1, 1]> other = ?"],
        ["Tensor<[1, 16, 1, 10]> self = ?", "Tensor<[1, 1, 1, 10]> other = ?"],
        ["Tensor<[1, 16, 1, 10]> self = ?", "Tensor<[1, 16, 1, 10]> other = ?"],
        ["Tensor<[1, 16, 1, 2]> self = ?", "Tensor<[1, 1, 1, 2]> other = ?"],
        ["Tensor<[1, 16, 1, 2]> self = ?", "Tensor<[1, 16, 1, 2]> other = ?"],
        ["Tensor<[1, 16, 1, s0 + 1]> self = ?", "Tensor<[1, 1, 1, s0 + 1]> other = ?"],
        ["Tensor<[1, 16, 1, s0 + 1]> self = ?", "Tensor<[1, 16, 1, s0 + 1]> other = ?"],
        ["Tensor<[1, 8, 10, 10]> self = ?", "Tensor<[1, 1, 1, 10]> other = ?"],
        ["Tensor<[1, 8, 10, 10]> self = ?", "Tensor<[1, 8, 10, 10]> other = ?"],
        ["Tensor<[1, 10, 512]> self = ?", "Tensor<[1, 10, 512]> other = ?"],
        ["Tensor<[1, 8, 1, 1]> self = ?", "Tensor<[1, 1, 1, 1]> other = ?"],
        ["Tensor<[1, 8, 1, 1]> self = ?", "Tensor<[1, 8, 1, 1]> other = ?"],
        ["Tensor<[1, 8, 1, 10]> self = ?", "Tensor<[1, 1, 1, 10]> other = ?"],
        ["Tensor<[1, 8, 1, 10]> self = ?", "Tensor<[1, 8, 1, 10]> other = ?"],
        ["Tensor<[1, 8, 1, 2]> self = ?", "Tensor<[1, 1, 1, 2]> other = ?"],
        ["Tensor<[1, 8, 1, 2]> self = ?", "Tensor<[1, 8, 1, 2]> other = ?"],
        ["Tensor<[1, 8, 1, s0 + 1]> self = ?", "Tensor<[1, 1, 1, s0 + 1]> other = ?"],
        ["Tensor<[1, 8, 1, s0 + 1]> self = ?", "Tensor<[1, 8, 1, s0 + 1]> other = ?"],
        ["Tensor<[1, 192, 7, 7]> self = ?", "Tensor<[1, 192, 7, 7]> other = ?"],
        ["Tensor<[1, 24, 60, 60]> self = ?", "Tensor<[1, 24, 60, 60]> other = ?"],
        ["Tensor<[1, 40, 30, 30]> self = ?", "Tensor<[1, 40, 30, 30]> other = ?"],
        ["Tensor<[1, 80, 15, 15]> self = ?", "Tensor<[1, 80, 15, 15]> other = ?"],
        ["Tensor<[1, 112, 15, 15]> self = ?", "Tensor<[1, 112, 15, 15]> other = ?"],
        ["Tensor<[1, 192, 8, 8]> self = ?", "Tensor<[1, 192, 8, 8]> other = ?"],
        ["Tensor<[1, 24, 65, 65]> self = ?", "Tensor<[1, 24, 65, 65]> other = ?"],
        ["Tensor<[1, 48, 33, 33]> self = ?", "Tensor<[1, 48, 33, 33]> other = ?"],
        ["Tensor<[1, 88, 17, 17]> self = ?", "Tensor<[1, 88, 17, 17]> other = ?"],
        ["Tensor<[1, 120, 17, 17]> self = ?", "Tensor<[1, 120, 17, 17]> other = ?"],
        ["Tensor<[1, 208, 9, 9]> self = ?", "Tensor<[1, 208, 9, 9]> other = ?"],
        ["Tensor<[1, 32, 75, 75]> self = ?", "Tensor<[1, 32, 75, 75]> other = ?"],
        ["Tensor<[1, 48, 38, 38]> self = ?", "Tensor<[1, 48, 38, 38]> other = ?"],
        ["Tensor<[1, 96, 19, 19]> self = ?", "Tensor<[1, 96, 19, 19]> other = ?"],
        ["Tensor<[1, 136, 19, 19]> self = ?", "Tensor<[1, 136, 19, 19]> other = ?"],
        ["Tensor<[1, 232, 10, 10]> self = ?", "Tensor<[1, 232, 10, 10]> other = ?"],
        ["Tensor<[1, 32, 95, 95]> self = ?", "Tensor<[1, 32, 95, 95]> other = ?"],
        ["Tensor<[1, 56, 48, 48]> self = ?", "Tensor<[1, 56, 48, 48]> other = ?"],
        ["Tensor<[1, 112, 24, 24]> self = ?", "Tensor<[1, 112, 24, 24]> other = ?"],
        ["Tensor<[1, 160, 24, 24]> self = ?", "Tensor<[1, 160, 24, 24]> other = ?"],
        ["Tensor<[1, 272, 12, 12]> self = ?", "Tensor<[1, 272, 12, 12]> other = ?"],
        ["Tensor<[1, 14, 128]> self = ?", "Tensor<[1, 14, 128]> other = ?"],
        ["Tensor<[1, 12, 14, 14]> self = ?", "Tensor<[1, 1, 1, 14]> other = ?"],
        ["Tensor<[1, 14, 768]> self = ?", "Tensor<[1, 14, 768]> other = ?"],
        ["Tensor<[1, 14, 3072]> self = ?", "Tensor<[1, 14, 3072]> other = ?"],
        ["Tensor<[1, 14, 3072]> self = ?", "Tensor other = 1.0"],
        ["Tensor<[1, 1370, 1280]> self = ?", "Tensor<[1, 1370, 1280]> other = ?"],
        ["Tensor<[1, 50, 1024]> self = ?", "Tensor<[1, 50, 1024]> other = ?"],
        ["Tensor<[1, 128, 75, 75]> self = ?", "Tensor<[1, 128, 75, 75]> other = ?"],
        ["Tensor<[1, 256, 75, 75]> self = ?", "Tensor<[1, 256, 75, 75]> other = ?"],
        ["Tensor<[1, 256, 38, 38]> self = ?", "Tensor<[1, 256, 38, 38]> other = ?"],
        ["Tensor<[1, 728, 38, 38]> self = ?", "Tensor<[1, 728, 38, 38]> other = ?"],
        ["Tensor<[1, 728, 19, 19]> self = ?", "Tensor<[1, 728, 19, 19]> other = ?"],
        ["Tensor<[1, 1024, 10, 10]> self = ?", "Tensor<[1, 1024, 10, 10]> other = ?"],
        ["Tensor<[1, 64, 150, 150]> self = ?", "Tensor<[1, 64, 150, 150]> other = ?"],
    ],
)
def test_aten(device, input_strings, input_var_only_native, input_var_check_accu, input_var_check_ttnn):
    metric = {
        "opname": "aten.add.Tensor",
        "input_strings": input_strings,
        "native_run": "N/A",
        "run": "N/A",
        "accuracy": "N/A",
        "convert_to_ttnn": "N/A",
        "ttnn_fallbacks_to_host_count": "N/A",
    }
    m = AtenModule()
    input_args, input_kwargs, status = render_metric_string_list_to_input_args_kwargs("aten.add.Tensor", input_strings)
    if status == False:
        pytest.skip("Invalid input strings")
    try:
        result_before = m.forward(*input_args, **input_kwargs)
        metric["native_run"] = True
    except Exception as e:
        print(f"Failed to run native. Raised exception: {e}")
        metric["native_run"] = False

    if metric["native_run"] == True:
        result_after = None
        option = torch_ttnn.TorchTtnnOption(device=device)
        # option.gen_graphviz = True
        # The compilation is lazy, so we need to run forward once to trigger the compilation
        m = torch.compile(m, backend=torch_ttnn.backend, options=option)
        try:
            ttnn.graph.begin_graph_capture()
            result_after = m.forward(*input_args, **input_kwargs)
            # option._out_fx_graphs[0].print_tabular()
            metric["run"] = True
        except Exception as e:
            print(f"Failed to run. Raised exception: {e}")
            metric["run"] = False
        finally:
            trace = ttnn.graph.end_graph_capture()
            call_stack = ttnn.graph.extract_calltrace(trace)
            if metric["run"] == True:
                print(call_stack)
                expected_to_host_count = 0
                if result_after is None:
                    expected_to_host_count = 0
                elif isinstance(result_after, torch.Tensor):
                    expected_to_host_count = 1
                elif isinstance(result_after, (list, dict)):
                    expected_to_host_count = len(result_after)
                else:
                    print(f"Unexpected result_after type: {type(result_after)}")

                to_host_count = sum(["Tensor::cpu" in str(node) for node in call_stack])
                fallbacks_to_host_count = to_host_count - expected_to_host_count
                print(f"expected_to_host_count: {expected_to_host_count}")
                print(f"to_host_count: {to_host_count}")
                print(f"fallbacks_to_host_count: {fallbacks_to_host_count}")
                metric["ttnn_fallbacks_to_host_count"] = fallbacks_to_host_count

    if metric["run"] == True:
        try:
            # Check inference result
            metric["accuracy"] = calculate_accuracy(result_before, result_after)
        except Exception as e:
            print(f"Failed to check inference result. Raised exception: {e}")

        try:
            # Check the graph has be rewritten and contain ttnn ops
            nodes = list(option._out_fx_graphs[0].nodes)
            if not any(["aten." in str(node.target) for node in nodes]):
                metric["convert_to_ttnn"] = True
            else:
                metric["convert_to_ttnn"] = False
        except Exception as e:
            print(f"Failed to check the graph has ttnn op. Raised exception: {e}")

    metrics.append(metric)

    if not input_var_only_native:
        assert metric["run"] == True
        if input_var_check_accu:
            assert metric["accuracy"] >= 0.99
        if input_var_check_ttnn:
            assert metric["convert_to_ttnn"] == True
