# SPDX-FileCopyrightText: © 2025 Tenstorrent AI ULC
#
# SPDX-License-Identifier: Apache-2.0
import torch
import torch_ttnn
import pytest
import pickle
import ttnn
from pathlib import Path
from tests.utils import calculate_accuracy, render_metric_string_list_to_input_args_kwargs


class AtenModule(torch.nn.Module):
    def __init__(self):
        super().__init__()

    def forward(self, *args, **kwargs):
        return torch.ops.aten.native_batch_norm_backward.default(*args, **kwargs)


metrics = []


def save_pickle(obj, base_path, filename):
    p = Path(base_path)
    p.mkdir(parents=True, exist_ok=True)
    pickle_out_path = p / f"{filename}.pickle"
    with open(pickle_out_path, "wb") as f:
        pickle.dump(obj, f)


def teardown_module(module):
    print(metrics)
    save_pickle(metrics, "metrics-autogen-op/ALL", "aten.native_batch_norm_backward.default")


@pytest.mark.parametrize(
    "input_strings",
    [
        [
            "Tensor<[1, 1024, 7, 7]> grad_out = ?",
            "Tensor<[1, 1024, 7, 7]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> running_mean = ?",
            "Optional[Tensor]<[1024]> running_var = ?",
            "Optional[Tensor]<[1024]> save_mean = ?",
            "Optional[Tensor]<[1024]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 462, 7, 7]> grad_out = ?",
            "Tensor<[1, 462, 7, 7]> input = ?",
            "Optional[Tensor]<[462]> weight = ?",
            "Optional[Tensor]<[462]> running_mean = ?",
            "Optional[Tensor]<[462]> running_var = ?",
            "Optional[Tensor]<[462]> save_mean = ?",
            "Optional[Tensor]<[462]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 160, 7, 7]> grad_out = ?",
            "Tensor<[1, 160, 7, 7]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> running_mean = ?",
            "Optional[Tensor]<[160]> running_var = ?",
            "Optional[Tensor]<[160]> save_mean = ?",
            "Optional[Tensor]<[160]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 272, 7, 7]> grad_out = ?",
            "Tensor<[1, 272, 7, 7]> input = ?",
            "Optional[Tensor]<[272]> weight = ?",
            "Optional[Tensor]<[272]> running_mean = ?",
            "Optional[Tensor]<[272]> running_var = ?",
            "Optional[Tensor]<[272]> save_mean = ?",
            "Optional[Tensor]<[272]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 640, 14, 14]> grad_out = ?",
            "Tensor<[1, 640, 14, 14]> input = ?",
            "Optional[Tensor]<[640]> weight = ?",
            "Optional[Tensor]<[640]> running_mean = ?",
            "Optional[Tensor]<[640]> running_var = ?",
            "Optional[Tensor]<[640]> save_mean = ?",
            "Optional[Tensor]<[640]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 334, 14, 14]> grad_out = ?",
            "Tensor<[1, 334, 14, 14]> input = ?",
            "Optional[Tensor]<[334]> weight = ?",
            "Optional[Tensor]<[334]> running_mean = ?",
            "Optional[Tensor]<[334]> running_var = ?",
            "Optional[Tensor]<[334]> save_mean = ?",
            "Optional[Tensor]<[334]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_out = ?",
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> running_mean = ?",
            "Optional[Tensor]<[40]> running_var = ?",
            "Optional[Tensor]<[40]> save_mean = ?",
            "Optional[Tensor]<[40]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 68, 14, 14]> grad_out = ?",
            "Tensor<[1, 68, 14, 14]> input = ?",
            "Optional[Tensor]<[68]> weight = ?",
            "Optional[Tensor]<[68]> running_mean = ?",
            "Optional[Tensor]<[68]> running_var = ?",
            "Optional[Tensor]<[68]> save_mean = ?",
            "Optional[Tensor]<[68]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 116, 14, 14]> grad_out = ?",
            "Tensor<[1, 116, 14, 14]> input = ?",
            "Optional[Tensor]<[116]> weight = ?",
            "Optional[Tensor]<[116]> running_mean = ?",
            "Optional[Tensor]<[116]> running_var = ?",
            "Optional[Tensor]<[116]> save_mean = ?",
            "Optional[Tensor]<[116]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 196, 14, 14]> grad_out = ?",
            "Tensor<[1, 196, 14, 14]> input = ?",
            "Optional[Tensor]<[196]> weight = ?",
            "Optional[Tensor]<[196]> running_mean = ?",
            "Optional[Tensor]<[196]> running_var = ?",
            "Optional[Tensor]<[196]> save_mean = ?",
            "Optional[Tensor]<[196]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 320, 28, 28]> grad_out = ?",
            "Tensor<[1, 320, 28, 28]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> running_mean = ?",
            "Optional[Tensor]<[320]> running_var = ?",
            "Optional[Tensor]<[320]> save_mean = ?",
            "Optional[Tensor]<[320]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 168, 28, 28]> grad_out = ?",
            "Tensor<[1, 168, 28, 28]> input = ?",
            "Optional[Tensor]<[168]> weight = ?",
            "Optional[Tensor]<[168]> running_mean = ?",
            "Optional[Tensor]<[168]> running_var = ?",
            "Optional[Tensor]<[168]> save_mean = ?",
            "Optional[Tensor]<[168]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_out = ?",
            "Tensor<[1, 20, 28, 28]> input = ?",
            "Optional[Tensor]<[20]> weight = ?",
            "Optional[Tensor]<[20]> running_mean = ?",
            "Optional[Tensor]<[20]> running_var = ?",
            "Optional[Tensor]<[20]> save_mean = ?",
            "Optional[Tensor]<[20]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 34, 28, 28]> grad_out = ?",
            "Tensor<[1, 34, 28, 28]> input = ?",
            "Optional[Tensor]<[34]> weight = ?",
            "Optional[Tensor]<[34]> running_mean = ?",
            "Optional[Tensor]<[34]> running_var = ?",
            "Optional[Tensor]<[34]> save_mean = ?",
            "Optional[Tensor]<[34]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 58, 28, 28]> grad_out = ?",
            "Tensor<[1, 58, 28, 28]> input = ?",
            "Optional[Tensor]<[58]> weight = ?",
            "Optional[Tensor]<[58]> running_mean = ?",
            "Optional[Tensor]<[58]> running_var = ?",
            "Optional[Tensor]<[58]> save_mean = ?",
            "Optional[Tensor]<[58]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 98, 28, 28]> grad_out = ?",
            "Tensor<[1, 98, 28, 28]> input = ?",
            "Optional[Tensor]<[98]> weight = ?",
            "Optional[Tensor]<[98]> running_mean = ?",
            "Optional[Tensor]<[98]> running_var = ?",
            "Optional[Tensor]<[98]> save_mean = ?",
            "Optional[Tensor]<[98]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 28, 28]> grad_out = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 134, 28, 28]> grad_out = ?",
            "Tensor<[1, 134, 28, 28]> input = ?",
            "Optional[Tensor]<[134]> weight = ?",
            "Optional[Tensor]<[134]> running_mean = ?",
            "Optional[Tensor]<[134]> running_var = ?",
            "Optional[Tensor]<[134]> save_mean = ?",
            "Optional[Tensor]<[134]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 16, 28, 28]> grad_out = ?",
            "Tensor<[1, 16, 28, 28]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> running_mean = ?",
            "Optional[Tensor]<[16]> running_var = ?",
            "Optional[Tensor]<[16]> save_mean = ?",
            "Optional[Tensor]<[16]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 28, 28, 28]> grad_out = ?",
            "Tensor<[1, 28, 28, 28]> input = ?",
            "Optional[Tensor]<[28]> weight = ?",
            "Optional[Tensor]<[28]> running_mean = ?",
            "Optional[Tensor]<[28]> running_var = ?",
            "Optional[Tensor]<[28]> save_mean = ?",
            "Optional[Tensor]<[28]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 46, 28, 28]> grad_out = ?",
            "Tensor<[1, 46, 28, 28]> input = ?",
            "Optional[Tensor]<[46]> weight = ?",
            "Optional[Tensor]<[46]> running_mean = ?",
            "Optional[Tensor]<[46]> running_var = ?",
            "Optional[Tensor]<[46]> save_mean = ?",
            "Optional[Tensor]<[46]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 78, 28, 28]> grad_out = ?",
            "Tensor<[1, 78, 28, 28]> input = ?",
            "Optional[Tensor]<[78]> weight = ?",
            "Optional[Tensor]<[78]> running_mean = ?",
            "Optional[Tensor]<[78]> running_var = ?",
            "Optional[Tensor]<[78]> save_mean = ?",
            "Optional[Tensor]<[78]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 56, 56]> grad_out = ?",
            "Tensor<[1, 128, 56, 56]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> running_mean = ?",
            "Optional[Tensor]<[128]> running_var = ?",
            "Optional[Tensor]<[128]> save_mean = ?",
            "Optional[Tensor]<[128]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 68, 56, 56]> grad_out = ?",
            "Tensor<[1, 68, 56, 56]> input = ?",
            "Optional[Tensor]<[68]> weight = ?",
            "Optional[Tensor]<[68]> running_mean = ?",
            "Optional[Tensor]<[68]> running_var = ?",
            "Optional[Tensor]<[68]> save_mean = ?",
            "Optional[Tensor]<[68]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 14, 56, 56]> grad_out = ?",
            "Tensor<[1, 14, 56, 56]> input = ?",
            "Optional[Tensor]<[14]> weight = ?",
            "Optional[Tensor]<[14]> running_mean = ?",
            "Optional[Tensor]<[14]> running_var = ?",
            "Optional[Tensor]<[14]> save_mean = ?",
            "Optional[Tensor]<[14]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 24, 56, 56]> grad_out = ?",
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> running_mean = ?",
            "Optional[Tensor]<[24]> running_var = ?",
            "Optional[Tensor]<[24]> save_mean = ?",
            "Optional[Tensor]<[24]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 40, 56, 56]> grad_out = ?",
            "Tensor<[1, 40, 56, 56]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> running_mean = ?",
            "Optional[Tensor]<[40]> running_var = ?",
            "Optional[Tensor]<[40]> save_mean = ?",
            "Optional[Tensor]<[40]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 112, 112]> grad_out = ?",
            "Tensor<[1, 64, 112, 112]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> running_mean = ?",
            "Optional[Tensor]<[64]> running_var = ?",
            "Optional[Tensor]<[64]> save_mean = ?",
            "Optional[Tensor]<[64]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 112, 112]> grad_out = ?",
            "Tensor<[1, 32, 112, 112]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_out = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> running_mean = ?",
            "Optional[Tensor]<[128]> running_var = ?",
            "Optional[Tensor]<[128]> save_mean = ?",
            "Optional[Tensor]<[128]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_out = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> running_mean = ?",
            "Optional[Tensor]<[512]> running_var = ?",
            "Optional[Tensor]<[512]> save_mean = ?",
            "Optional[Tensor]<[512]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 56, 56]> grad_out = ?",
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> running_mean = ?",
            "Optional[Tensor]<[64]> running_var = ?",
            "Optional[Tensor]<[64]> save_mean = ?",
            "Optional[Tensor]<[64]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 7, 7]> grad_out = ?",
            "Tensor<[1, 512, 7, 7]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> running_mean = ?",
            "Optional[Tensor]<[512]> running_var = ?",
            "Optional[Tensor]<[512]> save_mean = ?",
            "Optional[Tensor]<[512]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_out = ?",
            "Tensor<[1, 256, 14, 14]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 2048, 7, 7]> grad_out = ?",
            "Tensor<[1, 2048, 7, 7]> input = ?",
            "Optional[Tensor]<[2048]> weight = ?",
            "Optional[Tensor]<[2048]> running_mean = ?",
            "Optional[Tensor]<[2048]> running_var = ?",
            "Optional[Tensor]<[2048]> save_mean = ?",
            "Optional[Tensor]<[2048]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 14, 14]> grad_out = ?",
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> running_mean = ?",
            "Optional[Tensor]<[512]> running_var = ?",
            "Optional[Tensor]<[512]> save_mean = ?",
            "Optional[Tensor]<[512]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1024, 14, 14]> grad_out = ?",
            "Tensor<[1, 1024, 14, 14]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> running_mean = ?",
            "Optional[Tensor]<[1024]> running_var = ?",
            "Optional[Tensor]<[1024]> save_mean = ?",
            "Optional[Tensor]<[1024]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 56, 56]> grad_out = ?",
            "Tensor<[1, 256, 56, 56]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 128, 128]> grad_out = ?",
            "Tensor<[1, 256, 128, 128]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 256, 256]> grad_out = ?",
            "Tensor<[1, 32, 256, 256]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 128, 128]> grad_out = ?",
            "Tensor<[1, 64, 128, 128]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> running_mean = ?",
            "Optional[Tensor]<[64]> running_var = ?",
            "Optional[Tensor]<[64]> save_mean = ?",
            "Optional[Tensor]<[64]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 64, 64]> grad_out = ?",
            "Tensor<[1, 128, 64, 64]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> running_mean = ?",
            "Optional[Tensor]<[128]> running_var = ?",
            "Optional[Tensor]<[128]> save_mean = ?",
            "Optional[Tensor]<[128]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 32, 32]> grad_out = ?",
            "Tensor<[1, 256, 32, 32]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 16, 16]> grad_out = ?",
            "Tensor<[1, 512, 16, 16]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> running_mean = ?",
            "Optional[Tensor]<[512]> running_var = ?",
            "Optional[Tensor]<[512]> save_mean = ?",
            "Optional[Tensor]<[512]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 224, 224]> grad_out = ?",
            "Tensor<[1, 64, 224, 224]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> running_mean = ?",
            "Optional[Tensor]<[64]> running_var = ?",
            "Optional[Tensor]<[64]> save_mean = ?",
            "Optional[Tensor]<[64]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 112, 112]> grad_out = ?",
            "Tensor<[1, 128, 112, 112]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> running_mean = ?",
            "Optional[Tensor]<[128]> running_var = ?",
            "Optional[Tensor]<[128]> save_mean = ?",
            "Optional[Tensor]<[128]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 16, 224, 224]> grad_out = ?",
            "Tensor<[1, 16, 224, 224]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> running_mean = ?",
            "Optional[Tensor]<[16]> running_var = ?",
            "Optional[Tensor]<[16]> save_mean = ?",
            "Optional[Tensor]<[16]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 224, 7, 7]> grad_out = ?",
            "Tensor<[1, 224, 7, 7]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> running_mean = ?",
            "Optional[Tensor]<[224]> running_var = ?",
            "Optional[Tensor]<[224]> save_mean = ?",
            "Optional[Tensor]<[224]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 768, 14, 14]> grad_out = ?",
            "Tensor<[1, 768, 14, 14]> input = ?",
            "Optional[Tensor]<[768]> weight = ?",
            "Optional[Tensor]<[768]> running_mean = ?",
            "Optional[Tensor]<[768]> running_var = ?",
            "Optional[Tensor]<[768]> save_mean = ?",
            "Optional[Tensor]<[768]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 192, 14, 14]> grad_out = ?",
            "Tensor<[1, 192, 14, 14]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> running_mean = ?",
            "Optional[Tensor]<[192]> running_var = ?",
            "Optional[Tensor]<[192]> save_mean = ?",
            "Optional[Tensor]<[192]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 160, 28, 28]> grad_out = ?",
            "Tensor<[1, 160, 28, 28]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> running_mean = ?",
            "Optional[Tensor]<[160]> running_var = ?",
            "Optional[Tensor]<[160]> save_mean = ?",
            "Optional[Tensor]<[160]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 960, 7, 7]> grad_out = ?",
            "Tensor<[1, 960, 7, 7]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> running_mean = ?",
            "Optional[Tensor]<[960]> running_var = ?",
            "Optional[Tensor]<[960]> save_mean = ?",
            "Optional[Tensor]<[960]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 80, 7, 7]> grad_out = ?",
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> running_mean = ?",
            "Optional[Tensor]<[80]> running_var = ?",
            "Optional[Tensor]<[80]> save_mean = ?",
            "Optional[Tensor]<[80]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_out = ?",
            "Tensor<[1, 480, 7, 7]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> running_mean = ?",
            "Optional[Tensor]<[480]> running_var = ?",
            "Optional[Tensor]<[480]> save_mean = ?",
            "Optional[Tensor]<[480]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 112, 7, 7]> grad_out = ?",
            "Tensor<[1, 112, 7, 7]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> running_mean = ?",
            "Optional[Tensor]<[112]> running_var = ?",
            "Optional[Tensor]<[112]> save_mean = ?",
            "Optional[Tensor]<[112]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_out = ?",
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> running_mean = ?",
            "Optional[Tensor]<[672]> running_var = ?",
            "Optional[Tensor]<[672]> save_mean = ?",
            "Optional[Tensor]<[672]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 336, 14, 14]> grad_out = ?",
            "Tensor<[1, 336, 14, 14]> input = ?",
            "Optional[Tensor]<[336]> weight = ?",
            "Optional[Tensor]<[336]> running_mean = ?",
            "Optional[Tensor]<[336]> running_var = ?",
            "Optional[Tensor]<[336]> save_mean = ?",
            "Optional[Tensor]<[336]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 56, 14, 14]> grad_out = ?",
            "Tensor<[1, 56, 14, 14]> input = ?",
            "Optional[Tensor]<[56]> weight = ?",
            "Optional[Tensor]<[56]> running_mean = ?",
            "Optional[Tensor]<[56]> running_var = ?",
            "Optional[Tensor]<[56]> save_mean = ?",
            "Optional[Tensor]<[56]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 112, 14, 14]> grad_out = ?",
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> running_mean = ?",
            "Optional[Tensor]<[112]> running_var = ?",
            "Optional[Tensor]<[112]> save_mean = ?",
            "Optional[Tensor]<[112]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 80, 14, 14]> grad_out = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> running_mean = ?",
            "Optional[Tensor]<[80]> running_var = ?",
            "Optional[Tensor]<[80]> save_mean = ?",
            "Optional[Tensor]<[80]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_out = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> running_mean = ?",
            "Optional[Tensor]<[240]> running_var = ?",
            "Optional[Tensor]<[240]> save_mean = ?",
            "Optional[Tensor]<[240]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 92, 14, 14]> grad_out = ?",
            "Tensor<[1, 92, 14, 14]> input = ?",
            "Optional[Tensor]<[92]> weight = ?",
            "Optional[Tensor]<[92]> running_mean = ?",
            "Optional[Tensor]<[92]> running_var = ?",
            "Optional[Tensor]<[92]> save_mean = ?",
            "Optional[Tensor]<[92]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 100, 14, 14]> grad_out = ?",
            "Tensor<[1, 100, 14, 14]> input = ?",
            "Optional[Tensor]<[100]> weight = ?",
            "Optional[Tensor]<[100]> running_mean = ?",
            "Optional[Tensor]<[100]> running_var = ?",
            "Optional[Tensor]<[100]> save_mean = ?",
            "Optional[Tensor]<[100]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 120, 28, 28]> grad_out = ?",
            "Tensor<[1, 120, 28, 28]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> running_mean = ?",
            "Optional[Tensor]<[120]> running_var = ?",
            "Optional[Tensor]<[120]> save_mean = ?",
            "Optional[Tensor]<[120]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 60, 28, 28]> grad_out = ?",
            "Tensor<[1, 60, 28, 28]> input = ?",
            "Optional[Tensor]<[60]> weight = ?",
            "Optional[Tensor]<[60]> running_mean = ?",
            "Optional[Tensor]<[60]> running_var = ?",
            "Optional[Tensor]<[60]> save_mean = ?",
            "Optional[Tensor]<[60]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 40, 28, 28]> grad_out = ?",
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> running_mean = ?",
            "Optional[Tensor]<[40]> running_var = ?",
            "Optional[Tensor]<[40]> save_mean = ?",
            "Optional[Tensor]<[40]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 24, 28, 28]> grad_out = ?",
            "Tensor<[1, 24, 28, 28]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> running_mean = ?",
            "Optional[Tensor]<[24]> running_var = ?",
            "Optional[Tensor]<[24]> save_mean = ?",
            "Optional[Tensor]<[24]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 72, 28, 28]> grad_out = ?",
            "Tensor<[1, 72, 28, 28]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> running_mean = ?",
            "Optional[Tensor]<[72]> running_var = ?",
            "Optional[Tensor]<[72]> save_mean = ?",
            "Optional[Tensor]<[72]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 36, 56, 56]> grad_out = ?",
            "Tensor<[1, 36, 56, 56]> input = ?",
            "Optional[Tensor]<[36]> weight = ?",
            "Optional[Tensor]<[36]> running_mean = ?",
            "Optional[Tensor]<[36]> running_var = ?",
            "Optional[Tensor]<[36]> save_mean = ?",
            "Optional[Tensor]<[36]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 12, 56, 56]> grad_out = ?",
            "Tensor<[1, 12, 56, 56]> input = ?",
            "Optional[Tensor]<[12]> weight = ?",
            "Optional[Tensor]<[12]> running_mean = ?",
            "Optional[Tensor]<[12]> running_var = ?",
            "Optional[Tensor]<[12]> save_mean = ?",
            "Optional[Tensor]<[12]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 16, 56, 56]> grad_out = ?",
            "Tensor<[1, 16, 56, 56]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> running_mean = ?",
            "Optional[Tensor]<[16]> running_var = ?",
            "Optional[Tensor]<[16]> save_mean = ?",
            "Optional[Tensor]<[16]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 48, 56, 56]> grad_out = ?",
            "Tensor<[1, 48, 56, 56]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> running_mean = ?",
            "Optional[Tensor]<[48]> running_var = ?",
            "Optional[Tensor]<[48]> save_mean = ?",
            "Optional[Tensor]<[48]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 24, 112, 112]> grad_out = ?",
            "Tensor<[1, 24, 112, 112]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> running_mean = ?",
            "Optional[Tensor]<[24]> running_var = ?",
            "Optional[Tensor]<[24]> save_mean = ?",
            "Optional[Tensor]<[24]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 8, 112, 112]> grad_out = ?",
            "Tensor<[1, 8, 112, 112]> input = ?",
            "Optional[Tensor]<[8]> weight = ?",
            "Optional[Tensor]<[8]> running_mean = ?",
            "Optional[Tensor]<[8]> running_var = ?",
            "Optional[Tensor]<[8]> save_mean = ?",
            "Optional[Tensor]<[8]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 16, 112, 112]> grad_out = ?",
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> running_mean = ?",
            "Optional[Tensor]<[16]> running_var = ?",
            "Optional[Tensor]<[16]> save_mean = ?",
            "Optional[Tensor]<[16]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 960, 3, 3]> grad_out = ?",
            "Tensor<[1, 960, 3, 3]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> running_mean = ?",
            "Optional[Tensor]<[960]> running_var = ?",
            "Optional[Tensor]<[960]> save_mean = ?",
            "Optional[Tensor]<[960]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 184, 7, 7]> grad_out = ?",
            "Tensor<[1, 184, 7, 7]> input = ?",
            "Optional[Tensor]<[184]> weight = ?",
            "Optional[Tensor]<[184]> running_mean = ?",
            "Optional[Tensor]<[184]> running_var = ?",
            "Optional[Tensor]<[184]> save_mean = ?",
            "Optional[Tensor]<[184]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 200, 7, 7]> grad_out = ?",
            "Tensor<[1, 200, 7, 7]> input = ?",
            "Optional[Tensor]<[200]> weight = ?",
            "Optional[Tensor]<[200]> running_mean = ?",
            "Optional[Tensor]<[200]> running_var = ?",
            "Optional[Tensor]<[200]> save_mean = ?",
            "Optional[Tensor]<[200]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 120, 14, 14]> grad_out = ?",
            "Tensor<[1, 120, 14, 14]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> running_mean = ?",
            "Optional[Tensor]<[120]> running_var = ?",
            "Optional[Tensor]<[120]> save_mean = ?",
            "Optional[Tensor]<[120]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 7, 7]> grad_out = ?",
            "Tensor<[1, 256, 7, 7]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 14, 14]> grad_out = ?",
            "Tensor<[1, 128, 14, 14]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> running_mean = ?",
            "Optional[Tensor]<[128]> running_var = ?",
            "Optional[Tensor]<[128]> save_mean = ?",
            "Optional[Tensor]<[128]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 28, 28]> grad_out = ?",
            "Tensor<[1, 64, 28, 28]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> running_mean = ?",
            "Optional[Tensor]<[64]> running_var = ?",
            "Optional[Tensor]<[64]> save_mean = ?",
            "Optional[Tensor]<[64]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 56, 56]> grad_out = ?",
            "Tensor<[1, 32, 56, 56]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 7, 7]> grad_out = ?",
            "Tensor<[1, 144, 7, 7]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 36, 14, 14]> grad_out = ?",
            "Tensor<[1, 36, 14, 14]> input = ?",
            "Optional[Tensor]<[36]> weight = ?",
            "Optional[Tensor]<[36]> running_mean = ?",
            "Optional[Tensor]<[36]> running_var = ?",
            "Optional[Tensor]<[36]> save_mean = ?",
            "Optional[Tensor]<[36]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 18, 14, 14]> grad_out = ?",
            "Tensor<[1, 18, 14, 14]> input = ?",
            "Optional[Tensor]<[18]> weight = ?",
            "Optional[Tensor]<[18]> running_mean = ?",
            "Optional[Tensor]<[18]> running_var = ?",
            "Optional[Tensor]<[18]> save_mean = ?",
            "Optional[Tensor]<[18]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 18, 28, 28]> grad_out = ?",
            "Tensor<[1, 18, 28, 28]> input = ?",
            "Optional[Tensor]<[18]> weight = ?",
            "Optional[Tensor]<[18]> running_mean = ?",
            "Optional[Tensor]<[18]> running_var = ?",
            "Optional[Tensor]<[18]> save_mean = ?",
            "Optional[Tensor]<[18]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 72, 7, 7]> grad_out = ?",
            "Tensor<[1, 72, 7, 7]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> running_mean = ?",
            "Optional[Tensor]<[72]> running_var = ?",
            "Optional[Tensor]<[72]> save_mean = ?",
            "Optional[Tensor]<[72]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 72, 14, 14]> grad_out = ?",
            "Tensor<[1, 72, 14, 14]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> running_mean = ?",
            "Optional[Tensor]<[72]> running_var = ?",
            "Optional[Tensor]<[72]> save_mean = ?",
            "Optional[Tensor]<[72]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 36, 7, 7]> grad_out = ?",
            "Tensor<[1, 36, 7, 7]> input = ?",
            "Optional[Tensor]<[36]> weight = ?",
            "Optional[Tensor]<[36]> running_mean = ?",
            "Optional[Tensor]<[36]> running_var = ?",
            "Optional[Tensor]<[36]> save_mean = ?",
            "Optional[Tensor]<[36]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 36, 28, 28]> grad_out = ?",
            "Tensor<[1, 36, 28, 28]> input = ?",
            "Optional[Tensor]<[36]> weight = ?",
            "Optional[Tensor]<[36]> running_mean = ?",
            "Optional[Tensor]<[36]> running_var = ?",
            "Optional[Tensor]<[36]> save_mean = ?",
            "Optional[Tensor]<[36]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 18, 7, 7]> grad_out = ?",
            "Tensor<[1, 18, 7, 7]> input = ?",
            "Optional[Tensor]<[18]> weight = ?",
            "Optional[Tensor]<[18]> running_mean = ?",
            "Optional[Tensor]<[18]> running_var = ?",
            "Optional[Tensor]<[18]> save_mean = ?",
            "Optional[Tensor]<[18]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 18, 56, 56]> grad_out = ?",
            "Tensor<[1, 18, 56, 56]> input = ?",
            "Optional[Tensor]<[18]> weight = ?",
            "Optional[Tensor]<[18]> running_mean = ?",
            "Optional[Tensor]<[18]> running_var = ?",
            "Optional[Tensor]<[18]> save_mean = ?",
            "Optional[Tensor]<[18]> save_invstd = ?",
            "bool train = True",
            "float eps = 1e-05",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 8, 8]> grad_out = ?",
            "Tensor<[1, 256, 8, 8]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 8, 8]> grad_out = ?",
            "Tensor<[1, 512, 8, 8]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> running_mean = ?",
            "Optional[Tensor]<[512]> running_var = ?",
            "Optional[Tensor]<[512]> save_mean = ?",
            "Optional[Tensor]<[512]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 448, 8, 8]> grad_out = ?",
            "Tensor<[1, 448, 8, 8]> input = ?",
            "Optional[Tensor]<[448]> weight = ?",
            "Optional[Tensor]<[448]> running_mean = ?",
            "Optional[Tensor]<[448]> running_var = ?",
            "Optional[Tensor]<[448]> save_mean = ?",
            "Optional[Tensor]<[448]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 384, 8, 8]> grad_out = ?",
            "Tensor<[1, 384, 8, 8]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> running_mean = ?",
            "Optional[Tensor]<[384]> running_var = ?",
            "Optional[Tensor]<[384]> save_mean = ?",
            "Optional[Tensor]<[384]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 320, 8, 8]> grad_out = ?",
            "Tensor<[1, 320, 8, 8]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> running_mean = ?",
            "Optional[Tensor]<[320]> running_var = ?",
            "Optional[Tensor]<[320]> save_mean = ?",
            "Optional[Tensor]<[320]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 320, 17, 17]> grad_out = ?",
            "Tensor<[1, 320, 17, 17]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> running_mean = ?",
            "Optional[Tensor]<[320]> running_var = ?",
            "Optional[Tensor]<[320]> save_mean = ?",
            "Optional[Tensor]<[320]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 17, 17]> grad_out = ?",
            "Tensor<[1, 256, 17, 17]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 192, 8, 8]> grad_out = ?",
            "Tensor<[1, 192, 8, 8]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> running_mean = ?",
            "Optional[Tensor]<[192]> running_var = ?",
            "Optional[Tensor]<[192]> save_mean = ?",
            "Optional[Tensor]<[192]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 192, 17, 17]> grad_out = ?",
            "Tensor<[1, 192, 17, 17]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> running_mean = ?",
            "Optional[Tensor]<[192]> running_var = ?",
            "Optional[Tensor]<[192]> save_mean = ?",
            "Optional[Tensor]<[192]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 17, 17]> grad_out = ?",
            "Tensor<[1, 128, 17, 17]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> running_mean = ?",
            "Optional[Tensor]<[128]> running_var = ?",
            "Optional[Tensor]<[128]> save_mean = ?",
            "Optional[Tensor]<[128]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 224, 17, 17]> grad_out = ?",
            "Tensor<[1, 224, 17, 17]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> running_mean = ?",
            "Optional[Tensor]<[224]> running_var = ?",
            "Optional[Tensor]<[224]> save_mean = ?",
            "Optional[Tensor]<[224]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 384, 17, 17]> grad_out = ?",
            "Tensor<[1, 384, 17, 17]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> running_mean = ?",
            "Optional[Tensor]<[384]> running_var = ?",
            "Optional[Tensor]<[384]> save_mean = ?",
            "Optional[Tensor]<[384]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 224, 35, 35]> grad_out = ?",
            "Tensor<[1, 224, 35, 35]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> running_mean = ?",
            "Optional[Tensor]<[224]> running_var = ?",
            "Optional[Tensor]<[224]> save_mean = ?",
            "Optional[Tensor]<[224]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 192, 35, 35]> grad_out = ?",
            "Tensor<[1, 192, 35, 35]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> running_mean = ?",
            "Optional[Tensor]<[192]> running_var = ?",
            "Optional[Tensor]<[192]> save_mean = ?",
            "Optional[Tensor]<[192]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 35, 35]> grad_out = ?",
            "Tensor<[1, 96, 35, 35]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 35, 35]> grad_out = ?",
            "Tensor<[1, 64, 35, 35]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> running_mean = ?",
            "Optional[Tensor]<[64]> running_var = ?",
            "Optional[Tensor]<[64]> save_mean = ?",
            "Optional[Tensor]<[64]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 71, 71]> grad_out = ?",
            "Tensor<[1, 96, 71, 71]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 73, 73]> grad_out = ?",
            "Tensor<[1, 64, 73, 73]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> running_mean = ?",
            "Optional[Tensor]<[64]> running_var = ?",
            "Optional[Tensor]<[64]> save_mean = ?",
            "Optional[Tensor]<[64]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 73, 73]> grad_out = ?",
            "Tensor<[1, 96, 73, 73]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 147, 147]> grad_out = ?",
            "Tensor<[1, 64, 147, 147]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> running_mean = ?",
            "Optional[Tensor]<[64]> running_var = ?",
            "Optional[Tensor]<[64]> save_mean = ?",
            "Optional[Tensor]<[64]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 147, 147]> grad_out = ?",
            "Tensor<[1, 32, 147, 147]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 149, 149]> grad_out = ?",
            "Tensor<[1, 32, 149, 149]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1280, 7, 7]> grad_out = ?",
            "Tensor<[1, 1280, 7, 7]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> running_mean = ?",
            "Optional[Tensor]<[1280]> running_var = ?",
            "Optional[Tensor]<[1280]> save_mean = ?",
            "Optional[Tensor]<[1280]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 320, 7, 7]> grad_out = ?",
            "Tensor<[1, 320, 7, 7]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> running_mean = ?",
            "Optional[Tensor]<[320]> running_var = ?",
            "Optional[Tensor]<[320]> save_mean = ?",
            "Optional[Tensor]<[320]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1152, 7, 7]> grad_out = ?",
            "Tensor<[1, 1152, 7, 7]> input = ?",
            "Optional[Tensor]<[1152]> weight = ?",
            "Optional[Tensor]<[1152]> running_mean = ?",
            "Optional[Tensor]<[1152]> running_var = ?",
            "Optional[Tensor]<[1152]> save_mean = ?",
            "Optional[Tensor]<[1152]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 192, 7, 7]> grad_out = ?",
            "Tensor<[1, 192, 7, 7]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> running_mean = ?",
            "Optional[Tensor]<[192]> running_var = ?",
            "Optional[Tensor]<[192]> save_mean = ?",
            "Optional[Tensor]<[192]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_out = ?",
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> running_mean = ?",
            "Optional[Tensor]<[672]> running_var = ?",
            "Optional[Tensor]<[672]> save_mean = ?",
            "Optional[Tensor]<[672]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 672, 14, 14]> grad_out = ?",
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> running_mean = ?",
            "Optional[Tensor]<[672]> running_var = ?",
            "Optional[Tensor]<[672]> save_mean = ?",
            "Optional[Tensor]<[672]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 112, 14, 14]> grad_out = ?",
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> running_mean = ?",
            "Optional[Tensor]<[112]> running_var = ?",
            "Optional[Tensor]<[112]> save_mean = ?",
            "Optional[Tensor]<[112]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 480, 14, 14]> grad_out = ?",
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> running_mean = ?",
            "Optional[Tensor]<[480]> running_var = ?",
            "Optional[Tensor]<[480]> save_mean = ?",
            "Optional[Tensor]<[480]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 80, 14, 14]> grad_out = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> running_mean = ?",
            "Optional[Tensor]<[80]> running_var = ?",
            "Optional[Tensor]<[80]> save_mean = ?",
            "Optional[Tensor]<[80]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_out = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> running_mean = ?",
            "Optional[Tensor]<[240]> running_var = ?",
            "Optional[Tensor]<[240]> save_mean = ?",
            "Optional[Tensor]<[240]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 240, 28, 28]> grad_out = ?",
            "Tensor<[1, 240, 28, 28]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> running_mean = ?",
            "Optional[Tensor]<[240]> running_var = ?",
            "Optional[Tensor]<[240]> save_mean = ?",
            "Optional[Tensor]<[240]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 40, 28, 28]> grad_out = ?",
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> running_mean = ?",
            "Optional[Tensor]<[40]> running_var = ?",
            "Optional[Tensor]<[40]> save_mean = ?",
            "Optional[Tensor]<[40]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 28, 28]> grad_out = ?",
            "Tensor<[1, 144, 28, 28]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 56, 56]> grad_out = ?",
            "Tensor<[1, 144, 56, 56]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 24, 56, 56]> grad_out = ?",
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> running_mean = ?",
            "Optional[Tensor]<[24]> running_var = ?",
            "Optional[Tensor]<[24]> save_mean = ?",
            "Optional[Tensor]<[24]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 56, 56]> grad_out = ?",
            "Tensor<[1, 96, 56, 56]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 112, 112]> grad_out = ?",
            "Tensor<[1, 96, 112, 112]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 16, 112, 112]> grad_out = ?",
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> running_mean = ?",
            "Optional[Tensor]<[16]> running_var = ?",
            "Optional[Tensor]<[16]> save_mean = ?",
            "Optional[Tensor]<[16]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 112, 112]> grad_out = ?",
            "Tensor<[1, 32, 112, 112]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1280, 8, 8]> grad_out = ?",
            "Tensor<[1, 1280, 8, 8]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> running_mean = ?",
            "Optional[Tensor]<[1280]> running_var = ?",
            "Optional[Tensor]<[1280]> save_mean = ?",
            "Optional[Tensor]<[1280]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1152, 8, 8]> grad_out = ?",
            "Tensor<[1, 1152, 8, 8]> input = ?",
            "Optional[Tensor]<[1152]> weight = ?",
            "Optional[Tensor]<[1152]> running_mean = ?",
            "Optional[Tensor]<[1152]> running_var = ?",
            "Optional[Tensor]<[1152]> save_mean = ?",
            "Optional[Tensor]<[1152]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 672, 8, 8]> grad_out = ?",
            "Tensor<[1, 672, 8, 8]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> running_mean = ?",
            "Optional[Tensor]<[672]> running_var = ?",
            "Optional[Tensor]<[672]> save_mean = ?",
            "Optional[Tensor]<[672]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 672, 15, 15]> grad_out = ?",
            "Tensor<[1, 672, 15, 15]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> running_mean = ?",
            "Optional[Tensor]<[672]> running_var = ?",
            "Optional[Tensor]<[672]> save_mean = ?",
            "Optional[Tensor]<[672]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 112, 15, 15]> grad_out = ?",
            "Tensor<[1, 112, 15, 15]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> running_mean = ?",
            "Optional[Tensor]<[112]> running_var = ?",
            "Optional[Tensor]<[112]> save_mean = ?",
            "Optional[Tensor]<[112]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 480, 15, 15]> grad_out = ?",
            "Tensor<[1, 480, 15, 15]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> running_mean = ?",
            "Optional[Tensor]<[480]> running_var = ?",
            "Optional[Tensor]<[480]> save_mean = ?",
            "Optional[Tensor]<[480]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 80, 15, 15]> grad_out = ?",
            "Tensor<[1, 80, 15, 15]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> running_mean = ?",
            "Optional[Tensor]<[80]> running_var = ?",
            "Optional[Tensor]<[80]> save_mean = ?",
            "Optional[Tensor]<[80]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 240, 15, 15]> grad_out = ?",
            "Tensor<[1, 240, 15, 15]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> running_mean = ?",
            "Optional[Tensor]<[240]> running_var = ?",
            "Optional[Tensor]<[240]> save_mean = ?",
            "Optional[Tensor]<[240]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 240, 30, 30]> grad_out = ?",
            "Tensor<[1, 240, 30, 30]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> running_mean = ?",
            "Optional[Tensor]<[240]> running_var = ?",
            "Optional[Tensor]<[240]> save_mean = ?",
            "Optional[Tensor]<[240]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 40, 30, 30]> grad_out = ?",
            "Tensor<[1, 40, 30, 30]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> running_mean = ?",
            "Optional[Tensor]<[40]> running_var = ?",
            "Optional[Tensor]<[40]> save_mean = ?",
            "Optional[Tensor]<[40]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 30, 30]> grad_out = ?",
            "Tensor<[1, 144, 30, 30]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 60, 60]> grad_out = ?",
            "Tensor<[1, 144, 60, 60]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 24, 60, 60]> grad_out = ?",
            "Tensor<[1, 24, 60, 60]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> running_mean = ?",
            "Optional[Tensor]<[24]> running_var = ?",
            "Optional[Tensor]<[24]> save_mean = ?",
            "Optional[Tensor]<[24]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 60, 60]> grad_out = ?",
            "Tensor<[1, 96, 60, 60]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 120, 120]> grad_out = ?",
            "Tensor<[1, 96, 120, 120]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 16, 120, 120]> grad_out = ?",
            "Tensor<[1, 16, 120, 120]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> running_mean = ?",
            "Optional[Tensor]<[16]> running_var = ?",
            "Optional[Tensor]<[16]> save_mean = ?",
            "Optional[Tensor]<[16]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 120, 120]> grad_out = ?",
            "Tensor<[1, 32, 120, 120]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1280, 9, 9]> grad_out = ?",
            "Tensor<[1, 1280, 9, 9]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> running_mean = ?",
            "Optional[Tensor]<[1280]> running_var = ?",
            "Optional[Tensor]<[1280]> save_mean = ?",
            "Optional[Tensor]<[1280]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 352, 9, 9]> grad_out = ?",
            "Tensor<[1, 352, 9, 9]> input = ?",
            "Optional[Tensor]<[352]> weight = ?",
            "Optional[Tensor]<[352]> running_mean = ?",
            "Optional[Tensor]<[352]> running_var = ?",
            "Optional[Tensor]<[352]> save_mean = ?",
            "Optional[Tensor]<[352]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1248, 9, 9]> grad_out = ?",
            "Tensor<[1, 1248, 9, 9]> input = ?",
            "Optional[Tensor]<[1248]> weight = ?",
            "Optional[Tensor]<[1248]> running_mean = ?",
            "Optional[Tensor]<[1248]> running_var = ?",
            "Optional[Tensor]<[1248]> save_mean = ?",
            "Optional[Tensor]<[1248]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 208, 9, 9]> grad_out = ?",
            "Tensor<[1, 208, 9, 9]> input = ?",
            "Optional[Tensor]<[208]> weight = ?",
            "Optional[Tensor]<[208]> running_mean = ?",
            "Optional[Tensor]<[208]> running_var = ?",
            "Optional[Tensor]<[208]> save_mean = ?",
            "Optional[Tensor]<[208]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 720, 9, 9]> grad_out = ?",
            "Tensor<[1, 720, 9, 9]> input = ?",
            "Optional[Tensor]<[720]> weight = ?",
            "Optional[Tensor]<[720]> running_mean = ?",
            "Optional[Tensor]<[720]> running_var = ?",
            "Optional[Tensor]<[720]> save_mean = ?",
            "Optional[Tensor]<[720]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 720, 17, 17]> grad_out = ?",
            "Tensor<[1, 720, 17, 17]> input = ?",
            "Optional[Tensor]<[720]> weight = ?",
            "Optional[Tensor]<[720]> running_mean = ?",
            "Optional[Tensor]<[720]> running_var = ?",
            "Optional[Tensor]<[720]> save_mean = ?",
            "Optional[Tensor]<[720]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 120, 17, 17]> grad_out = ?",
            "Tensor<[1, 120, 17, 17]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> running_mean = ?",
            "Optional[Tensor]<[120]> running_var = ?",
            "Optional[Tensor]<[120]> save_mean = ?",
            "Optional[Tensor]<[120]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 528, 17, 17]> grad_out = ?",
            "Tensor<[1, 528, 17, 17]> input = ?",
            "Optional[Tensor]<[528]> weight = ?",
            "Optional[Tensor]<[528]> running_mean = ?",
            "Optional[Tensor]<[528]> running_var = ?",
            "Optional[Tensor]<[528]> save_mean = ?",
            "Optional[Tensor]<[528]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 88, 17, 17]> grad_out = ?",
            "Tensor<[1, 88, 17, 17]> input = ?",
            "Optional[Tensor]<[88]> weight = ?",
            "Optional[Tensor]<[88]> running_mean = ?",
            "Optional[Tensor]<[88]> running_var = ?",
            "Optional[Tensor]<[88]> save_mean = ?",
            "Optional[Tensor]<[88]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 288, 17, 17]> grad_out = ?",
            "Tensor<[1, 288, 17, 17]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> running_mean = ?",
            "Optional[Tensor]<[288]> running_var = ?",
            "Optional[Tensor]<[288]> save_mean = ?",
            "Optional[Tensor]<[288]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 288, 33, 33]> grad_out = ?",
            "Tensor<[1, 288, 33, 33]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> running_mean = ?",
            "Optional[Tensor]<[288]> running_var = ?",
            "Optional[Tensor]<[288]> save_mean = ?",
            "Optional[Tensor]<[288]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 48, 33, 33]> grad_out = ?",
            "Tensor<[1, 48, 33, 33]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> running_mean = ?",
            "Optional[Tensor]<[48]> running_var = ?",
            "Optional[Tensor]<[48]> save_mean = ?",
            "Optional[Tensor]<[48]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 33, 33]> grad_out = ?",
            "Tensor<[1, 144, 33, 33]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 65, 65]> grad_out = ?",
            "Tensor<[1, 144, 65, 65]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 24, 65, 65]> grad_out = ?",
            "Tensor<[1, 24, 65, 65]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> running_mean = ?",
            "Optional[Tensor]<[24]> running_var = ?",
            "Optional[Tensor]<[24]> save_mean = ?",
            "Optional[Tensor]<[24]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 65, 65]> grad_out = ?",
            "Tensor<[1, 96, 65, 65]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 130, 130]> grad_out = ?",
            "Tensor<[1, 96, 130, 130]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 16, 130, 130]> grad_out = ?",
            "Tensor<[1, 16, 130, 130]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> running_mean = ?",
            "Optional[Tensor]<[16]> running_var = ?",
            "Optional[Tensor]<[16]> save_mean = ?",
            "Optional[Tensor]<[16]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 130, 130]> grad_out = ?",
            "Tensor<[1, 32, 130, 130]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1280, 10, 10]> grad_out = ?",
            "Tensor<[1, 1280, 10, 10]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> running_mean = ?",
            "Optional[Tensor]<[1280]> running_var = ?",
            "Optional[Tensor]<[1280]> save_mean = ?",
            "Optional[Tensor]<[1280]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 384, 10, 10]> grad_out = ?",
            "Tensor<[1, 384, 10, 10]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> running_mean = ?",
            "Optional[Tensor]<[384]> running_var = ?",
            "Optional[Tensor]<[384]> save_mean = ?",
            "Optional[Tensor]<[384]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1392, 10, 10]> grad_out = ?",
            "Tensor<[1, 1392, 10, 10]> input = ?",
            "Optional[Tensor]<[1392]> weight = ?",
            "Optional[Tensor]<[1392]> running_mean = ?",
            "Optional[Tensor]<[1392]> running_var = ?",
            "Optional[Tensor]<[1392]> save_mean = ?",
            "Optional[Tensor]<[1392]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 232, 10, 10]> grad_out = ?",
            "Tensor<[1, 232, 10, 10]> input = ?",
            "Optional[Tensor]<[232]> weight = ?",
            "Optional[Tensor]<[232]> running_mean = ?",
            "Optional[Tensor]<[232]> running_var = ?",
            "Optional[Tensor]<[232]> save_mean = ?",
            "Optional[Tensor]<[232]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 816, 10, 10]> grad_out = ?",
            "Tensor<[1, 816, 10, 10]> input = ?",
            "Optional[Tensor]<[816]> weight = ?",
            "Optional[Tensor]<[816]> running_mean = ?",
            "Optional[Tensor]<[816]> running_var = ?",
            "Optional[Tensor]<[816]> save_mean = ?",
            "Optional[Tensor]<[816]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 816, 19, 19]> grad_out = ?",
            "Tensor<[1, 816, 19, 19]> input = ?",
            "Optional[Tensor]<[816]> weight = ?",
            "Optional[Tensor]<[816]> running_mean = ?",
            "Optional[Tensor]<[816]> running_var = ?",
            "Optional[Tensor]<[816]> save_mean = ?",
            "Optional[Tensor]<[816]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 136, 19, 19]> grad_out = ?",
            "Tensor<[1, 136, 19, 19]> input = ?",
            "Optional[Tensor]<[136]> weight = ?",
            "Optional[Tensor]<[136]> running_mean = ?",
            "Optional[Tensor]<[136]> running_var = ?",
            "Optional[Tensor]<[136]> save_mean = ?",
            "Optional[Tensor]<[136]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 576, 19, 19]> grad_out = ?",
            "Tensor<[1, 576, 19, 19]> input = ?",
            "Optional[Tensor]<[576]> weight = ?",
            "Optional[Tensor]<[576]> running_mean = ?",
            "Optional[Tensor]<[576]> running_var = ?",
            "Optional[Tensor]<[576]> save_mean = ?",
            "Optional[Tensor]<[576]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 96, 19, 19]> grad_out = ?",
            "Tensor<[1, 96, 19, 19]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> running_mean = ?",
            "Optional[Tensor]<[96]> running_var = ?",
            "Optional[Tensor]<[96]> save_mean = ?",
            "Optional[Tensor]<[96]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 288, 19, 19]> grad_out = ?",
            "Tensor<[1, 288, 19, 19]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> running_mean = ?",
            "Optional[Tensor]<[288]> running_var = ?",
            "Optional[Tensor]<[288]> save_mean = ?",
            "Optional[Tensor]<[288]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 288, 38, 38]> grad_out = ?",
            "Tensor<[1, 288, 38, 38]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> running_mean = ?",
            "Optional[Tensor]<[288]> running_var = ?",
            "Optional[Tensor]<[288]> save_mean = ?",
            "Optional[Tensor]<[288]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 48, 38, 38]> grad_out = ?",
            "Tensor<[1, 48, 38, 38]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> running_mean = ?",
            "Optional[Tensor]<[48]> running_var = ?",
            "Optional[Tensor]<[48]> save_mean = ?",
            "Optional[Tensor]<[48]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 192, 38, 38]> grad_out = ?",
            "Tensor<[1, 192, 38, 38]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> running_mean = ?",
            "Optional[Tensor]<[192]> running_var = ?",
            "Optional[Tensor]<[192]> save_mean = ?",
            "Optional[Tensor]<[192]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 192, 75, 75]> grad_out = ?",
            "Tensor<[1, 192, 75, 75]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> running_mean = ?",
            "Optional[Tensor]<[192]> running_var = ?",
            "Optional[Tensor]<[192]> save_mean = ?",
            "Optional[Tensor]<[192]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 75, 75]> grad_out = ?",
            "Tensor<[1, 32, 75, 75]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 75, 75]> grad_out = ?",
            "Tensor<[1, 144, 75, 75]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 150, 150]> grad_out = ?",
            "Tensor<[1, 144, 150, 150]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 24, 150, 150]> grad_out = ?",
            "Tensor<[1, 24, 150, 150]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> running_mean = ?",
            "Optional[Tensor]<[24]> running_var = ?",
            "Optional[Tensor]<[24]> save_mean = ?",
            "Optional[Tensor]<[24]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 150, 150]> grad_out = ?",
            "Tensor<[1, 32, 150, 150]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1280, 12, 12]> grad_out = ?",
            "Tensor<[1, 1280, 12, 12]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> running_mean = ?",
            "Optional[Tensor]<[1280]> running_var = ?",
            "Optional[Tensor]<[1280]> save_mean = ?",
            "Optional[Tensor]<[1280]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 448, 12, 12]> grad_out = ?",
            "Tensor<[1, 448, 12, 12]> input = ?",
            "Optional[Tensor]<[448]> weight = ?",
            "Optional[Tensor]<[448]> running_mean = ?",
            "Optional[Tensor]<[448]> running_var = ?",
            "Optional[Tensor]<[448]> save_mean = ?",
            "Optional[Tensor]<[448]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1632, 12, 12]> grad_out = ?",
            "Tensor<[1, 1632, 12, 12]> input = ?",
            "Optional[Tensor]<[1632]> weight = ?",
            "Optional[Tensor]<[1632]> running_mean = ?",
            "Optional[Tensor]<[1632]> running_var = ?",
            "Optional[Tensor]<[1632]> save_mean = ?",
            "Optional[Tensor]<[1632]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 272, 12, 12]> grad_out = ?",
            "Tensor<[1, 272, 12, 12]> input = ?",
            "Optional[Tensor]<[272]> weight = ?",
            "Optional[Tensor]<[272]> running_mean = ?",
            "Optional[Tensor]<[272]> running_var = ?",
            "Optional[Tensor]<[272]> save_mean = ?",
            "Optional[Tensor]<[272]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 960, 12, 12]> grad_out = ?",
            "Tensor<[1, 960, 12, 12]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> running_mean = ?",
            "Optional[Tensor]<[960]> running_var = ?",
            "Optional[Tensor]<[960]> save_mean = ?",
            "Optional[Tensor]<[960]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 960, 24, 24]> grad_out = ?",
            "Tensor<[1, 960, 24, 24]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> running_mean = ?",
            "Optional[Tensor]<[960]> running_var = ?",
            "Optional[Tensor]<[960]> save_mean = ?",
            "Optional[Tensor]<[960]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 160, 24, 24]> grad_out = ?",
            "Tensor<[1, 160, 24, 24]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> running_mean = ?",
            "Optional[Tensor]<[160]> running_var = ?",
            "Optional[Tensor]<[160]> save_mean = ?",
            "Optional[Tensor]<[160]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 672, 24, 24]> grad_out = ?",
            "Tensor<[1, 672, 24, 24]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> running_mean = ?",
            "Optional[Tensor]<[672]> running_var = ?",
            "Optional[Tensor]<[672]> save_mean = ?",
            "Optional[Tensor]<[672]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 112, 24, 24]> grad_out = ?",
            "Tensor<[1, 112, 24, 24]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> running_mean = ?",
            "Optional[Tensor]<[112]> running_var = ?",
            "Optional[Tensor]<[112]> save_mean = ?",
            "Optional[Tensor]<[112]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 336, 24, 24]> grad_out = ?",
            "Tensor<[1, 336, 24, 24]> input = ?",
            "Optional[Tensor]<[336]> weight = ?",
            "Optional[Tensor]<[336]> running_mean = ?",
            "Optional[Tensor]<[336]> running_var = ?",
            "Optional[Tensor]<[336]> save_mean = ?",
            "Optional[Tensor]<[336]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 336, 48, 48]> grad_out = ?",
            "Tensor<[1, 336, 48, 48]> input = ?",
            "Optional[Tensor]<[336]> weight = ?",
            "Optional[Tensor]<[336]> running_mean = ?",
            "Optional[Tensor]<[336]> running_var = ?",
            "Optional[Tensor]<[336]> save_mean = ?",
            "Optional[Tensor]<[336]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 56, 48, 48]> grad_out = ?",
            "Tensor<[1, 56, 48, 48]> input = ?",
            "Optional[Tensor]<[56]> weight = ?",
            "Optional[Tensor]<[56]> running_mean = ?",
            "Optional[Tensor]<[56]> running_var = ?",
            "Optional[Tensor]<[56]> save_mean = ?",
            "Optional[Tensor]<[56]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 192, 48, 48]> grad_out = ?",
            "Tensor<[1, 192, 48, 48]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> running_mean = ?",
            "Optional[Tensor]<[192]> running_var = ?",
            "Optional[Tensor]<[192]> save_mean = ?",
            "Optional[Tensor]<[192]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 192, 95, 95]> grad_out = ?",
            "Tensor<[1, 192, 95, 95]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> running_mean = ?",
            "Optional[Tensor]<[192]> running_var = ?",
            "Optional[Tensor]<[192]> save_mean = ?",
            "Optional[Tensor]<[192]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 95, 95]> grad_out = ?",
            "Tensor<[1, 32, 95, 95]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 95, 95]> grad_out = ?",
            "Tensor<[1, 144, 95, 95]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 144, 190, 190]> grad_out = ?",
            "Tensor<[1, 144, 190, 190]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> running_mean = ?",
            "Optional[Tensor]<[144]> running_var = ?",
            "Optional[Tensor]<[144]> save_mean = ?",
            "Optional[Tensor]<[144]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 24, 190, 190]> grad_out = ?",
            "Tensor<[1, 24, 190, 190]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> running_mean = ?",
            "Optional[Tensor]<[24]> running_var = ?",
            "Optional[Tensor]<[24]> save_mean = ?",
            "Optional[Tensor]<[24]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 190, 190]> grad_out = ?",
            "Tensor<[1, 32, 190, 190]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> running_mean = ?",
            "Optional[Tensor]<[32]> running_var = ?",
            "Optional[Tensor]<[32]> save_mean = ?",
            "Optional[Tensor]<[32]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 2048, 10, 10]> grad_out = ?",
            "Tensor<[1, 2048, 10, 10]> input = ?",
            "Optional[Tensor]<[2048]> weight = ?",
            "Optional[Tensor]<[2048]> running_mean = ?",
            "Optional[Tensor]<[2048]> running_var = ?",
            "Optional[Tensor]<[2048]> save_mean = ?",
            "Optional[Tensor]<[2048]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1536, 10, 10]> grad_out = ?",
            "Tensor<[1, 1536, 10, 10]> input = ?",
            "Optional[Tensor]<[1536]> weight = ?",
            "Optional[Tensor]<[1536]> running_mean = ?",
            "Optional[Tensor]<[1536]> running_var = ?",
            "Optional[Tensor]<[1536]> save_mean = ?",
            "Optional[Tensor]<[1536]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1024, 10, 10]> grad_out = ?",
            "Tensor<[1, 1024, 10, 10]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> running_mean = ?",
            "Optional[Tensor]<[1024]> running_var = ?",
            "Optional[Tensor]<[1024]> save_mean = ?",
            "Optional[Tensor]<[1024]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1024, 19, 19]> grad_out = ?",
            "Tensor<[1, 1024, 19, 19]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> running_mean = ?",
            "Optional[Tensor]<[1024]> running_var = ?",
            "Optional[Tensor]<[1024]> save_mean = ?",
            "Optional[Tensor]<[1024]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 728, 19, 19]> grad_out = ?",
            "Tensor<[1, 728, 19, 19]> input = ?",
            "Optional[Tensor]<[728]> weight = ?",
            "Optional[Tensor]<[728]> running_mean = ?",
            "Optional[Tensor]<[728]> running_var = ?",
            "Optional[Tensor]<[728]> save_mean = ?",
            "Optional[Tensor]<[728]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 728, 38, 38]> grad_out = ?",
            "Tensor<[1, 728, 38, 38]> input = ?",
            "Optional[Tensor]<[728]> weight = ?",
            "Optional[Tensor]<[728]> running_mean = ?",
            "Optional[Tensor]<[728]> running_var = ?",
            "Optional[Tensor]<[728]> save_mean = ?",
            "Optional[Tensor]<[728]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 38, 38]> grad_out = ?",
            "Tensor<[1, 256, 38, 38]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 75, 75]> grad_out = ?",
            "Tensor<[1, 256, 75, 75]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> running_mean = ?",
            "Optional[Tensor]<[256]> running_var = ?",
            "Optional[Tensor]<[256]> save_mean = ?",
            "Optional[Tensor]<[256]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 75, 75]> grad_out = ?",
            "Tensor<[1, 128, 75, 75]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> running_mean = ?",
            "Optional[Tensor]<[128]> running_var = ?",
            "Optional[Tensor]<[128]> save_mean = ?",
            "Optional[Tensor]<[128]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 150, 150]> grad_out = ?",
            "Tensor<[1, 128, 150, 150]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> running_mean = ?",
            "Optional[Tensor]<[128]> running_var = ?",
            "Optional[Tensor]<[128]> save_mean = ?",
            "Optional[Tensor]<[128]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 150, 150]> grad_out = ?",
            "Tensor<[1, 64, 150, 150]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> running_mean = ?",
            "Optional[Tensor]<[64]> running_var = ?",
            "Optional[Tensor]<[64]> save_mean = ?",
            "Optional[Tensor]<[64]> save_invstd = ?",
            "bool train = True",
            "float eps = 0.001",
            "List[bool] output_mask = [True, True, True]",
        ],
    ],
)
def test_aten(device, input_strings, input_var_only_native, input_var_check_accu, input_var_check_ttnn):
    metric = {
        "opname": "aten.native_batch_norm_backward.default",
        "input_strings": input_strings,
        "native_run": "N/A",
        "run": "N/A",
        "accuracy": "N/A",
        "convert_to_ttnn": "N/A",
        "ttnn_fallbacks_to_host_count": "N/A",
    }
    m = AtenModule()
    input_args, input_kwargs, status = render_metric_string_list_to_input_args_kwargs(
        "aten.native_batch_norm_backward.default", input_strings
    )
    if status == False:
        pytest.skip("Invalid input strings")
    try:
        result_before = m.forward(*input_args, **input_kwargs)
        metric["native_run"] = True
    except Exception as e:
        print(f"Failed to run native. Raised exception: {e}")
        metric["native_run"] = False

    if metric["native_run"] == True:
        result_after = None
        option = torch_ttnn.TorchTtnnOption(device=device)
        # option.gen_graphviz = True
        # The compilation is lazy, so we need to run forward once to trigger the compilation
        m = torch.compile(m, backend=torch_ttnn.backend, options=option)
        try:
            ttnn.graph.begin_graph_capture()
            result_after = m.forward(*input_args, **input_kwargs)
            # option._out_fx_graphs[0].print_tabular()
            metric["run"] = True
        except Exception as e:
            print(f"Failed to run. Raised exception: {e}")
            metric["run"] = False
        finally:
            trace = ttnn.graph.end_graph_capture()
            call_stack = ttnn.graph.extract_calltrace(trace)
            if metric["run"] == True:
                print(call_stack)
                expected_to_host_count = 0
                if result_after is None:
                    expected_to_host_count = 0
                elif isinstance(result_after, torch.Tensor):
                    expected_to_host_count = 1
                elif isinstance(result_after, (list, dict)):
                    expected_to_host_count = len(result_after)
                else:
                    print(f"Unexpected result_after type: {type(result_after)}")

                to_host_count = sum(["Tensor::cpu" in str(node) for node in call_stack])
                fallbacks_to_host_count = to_host_count - expected_to_host_count
                print(f"expected_to_host_count: {expected_to_host_count}")
                print(f"to_host_count: {to_host_count}")
                print(f"fallbacks_to_host_count: {fallbacks_to_host_count}")
                metric["ttnn_fallbacks_to_host_count"] = fallbacks_to_host_count

    if metric["run"] == True:
        try:
            # Check inference result
            metric["accuracy"] = calculate_accuracy(result_before, result_after)
        except Exception as e:
            print(f"Failed to check inference result. Raised exception: {e}")

        try:
            # Check the graph has be rewritten and contain ttnn ops
            nodes = list(option._out_fx_graphs[0].nodes)
            if not any(["aten." in str(node.target) for node in nodes]):
                metric["convert_to_ttnn"] = True
            else:
                metric["convert_to_ttnn"] = False
        except Exception as e:
            print(f"Failed to check the graph has ttnn op. Raised exception: {e}")

    metrics.append(metric)

    if not input_var_only_native:
        assert metric["run"] == True
        if input_var_check_accu:
            assert metric["accuracy"] >= 0.99
        if input_var_check_ttnn:
            assert metric["convert_to_ttnn"] == True
