# SPDX-FileCopyrightText: © 2025 Tenstorrent AI ULC
#
# SPDX-License-Identifier: Apache-2.0
import torch
import torch_ttnn
import pytest
import pickle
import ttnn
from pathlib import Path
from tests.utils import calculate_accuracy, render_metric_string_list_to_input_args_kwargs


class AtenModule(torch.nn.Module):
    def __init__(self):
        super().__init__()

    def forward(self, *args, **kwargs):
        return torch.ops.aten.view.default(*args, **kwargs)


metrics = []


def save_pickle(obj, base_path, filename):
    p = Path(base_path)
    p.mkdir(parents=True, exist_ok=True)
    pickle_out_path = p / f"{filename}.pickle"
    with open(pickle_out_path, "wb") as f:
        pickle.dump(obj, f)


def teardown_module(module):
    print(metrics)
    save_pickle(metrics, "metrics-autogen-op/ALL", "aten.view.default")


@pytest.mark.parametrize(
    "input_strings",
    [
        ["Tensor<[1, 784]> self = ?", "List[int] size = [784]"],
        ["Tensor<[1, 128]> self = ?", "List[int] size = [128]"],
        ["Tensor<[1, 64]> self = ?", "List[int] size = [64]"],
        ["Tensor<[1, 12]> self = ?", "List[int] size = [12]"],
        ["Tensor<[1, 3]> self = ?", "List[int] size = [3]"],
        ["Tensor<[1, 256, 1024]> self = ?", "List[int] size = [256, 1024]"],
        ["Tensor<[256, 1024]> self = ?", "List[int] size = [1, 256, 1024]"],
        ["Tensor<[1, 256, 1024]> self = ?", "List[int] size = [1, 256, 16, 64]"],
        ["Tensor<[1, 16, 256, 64]> self = ?", "List[int] size = [16, 256, 64]"],
        ["Tensor<[1, 16, 64, 256]> self = ?", "List[int] size = [16, 64, 256]"],
        ["Tensor<[16, 256, 256]> self = ?", "List[int] size = [1, 16, 256, 256]"],
        ["Tensor<[1, 16, 256, 256]> self = ?", "List[int] size = [16, 256, 256]"],
        ["Tensor<[16, 256, 64]> self = ?", "List[int] size = [1, 16, 256, 64]"],
        ["Tensor<[1, 256, 16, 64]> self = ?", "List[int] size = [1, 256, 1024]"],
        ["Tensor<[256, 4096]> self = ?", "List[int] size = [1, 256, 4096]"],
        ["Tensor<[1, 256, 4096]> self = ?", "List[int] size = [256, 4096]"],
        ["Tensor<[256, 2]> self = ?", "List[int] size = [1, 256, 2]"],
        ["Tensor<[1, 16, 32]> self = ?", "List[int] size = [16, 1, 32]"],
        ["Tensor<[1, 32, 1536]> self = ?", "List[int] size = [32, 1536]"],
        ["Tensor<[32, 4608]> self = ?", "List[int] size = [1, 32, 4608]"],
        ["Tensor<[1, 32, 4608]> self = ?", "List[int] size = [1, 32, 16, 3, 96]"],
        ["Tensor<[1, 16, 32, 96]> self = ?", "List[int] size = [16, 32, 96]"],
        ["Tensor<[1, 16, 96, 32]> self = ?", "List[int] size = [16, 96, 32]"],
        ["Tensor<[16, 32, 32]> self = ?", "List[int] size = [1, 16, 32, 32]"],
        ["Tensor<[1, 16, 32, 32]> self = ?", "List[int] size = [16, 32, 32]"],
        ["Tensor<[16, 32, 96]> self = ?", "List[int] size = [1, 16, 32, 96]"],
        ["Tensor<[32, 1536]> self = ?", "List[int] size = [1, 32, 1536]"],
        ["Tensor<[32, 6144]> self = ?", "List[int] size = [1, 32, 6144]"],
        ["Tensor<[1, 32, 6144]> self = ?", "List[int] size = [32, 6144]"],
        ["Tensor<[32, 250880]> self = ?", "List[int] size = [1, 32, 250880]"],
        ["Tensor<[1, 768, 7, 7]> self = ?", "List[int] size = [1, 768, 49]"],
        ["Tensor<[1, 50, 768]> self = ?", "List[int] size = [50, 768]"],
        ["Tensor<[50, 768]> self = ?", "List[int] size = [1, 50, 768]"],
        ["Tensor<[1, 50, 768]> self = ?", "List[int] size = [1, -1, 12, 64]"],
        ["Tensor<[1, 50, 768]> self = ?", "List[int] size = [1, 50, 12, 64]"],
        ["Tensor<[1, 12, 50, 64]> self = ?", "List[int] size = [12, -1, 64]"],
        ["Tensor<[12, 50, 64]> self = ?", "List[int] size = [1, 12, 50, 64]"],
        ["Tensor<[50, 3072]> self = ?", "List[int] size = [1, 50, 3072]"],
        ["Tensor<[1, 50, 3072]> self = ?", "List[int] size = [50, 3072]"],
        ["Tensor<[2, 7]> self = ?", "List[int] size = [-1, 7]"],
        ["Tensor<[2, 7, 512]> self = ?", "List[int] size = [14, 512]"],
        ["Tensor<[14, 512]> self = ?", "List[int] size = [2, 7, 512]"],
        ["Tensor<[2, 7, 512]> self = ?", "List[int] size = [2, -1, 8, 64]"],
        ["Tensor<[2, 7, 512]> self = ?", "List[int] size = [2, 7, 8, 64]"],
        ["Tensor<[2, 8, 7, 64]> self = ?", "List[int] size = [16, -1, 64]"],
        ["Tensor<[16, 7, 7]> self = ?", "List[int] size = [2, 8, 7, 7]"],
        ["Tensor<[2, 8, 7, 7]> self = ?", "List[int] size = [16, 7, 7]"],
        ["Tensor<[16, 7, 64]> self = ?", "List[int] size = [2, 8, 7, 64]"],
        ["Tensor<[14, 2048]> self = ?", "List[int] size = [2, 7, 2048]"],
        ["Tensor<[2, 7, 2048]> self = ?", "List[int] size = [14, 2048]"],
        ["Tensor<[1, 512]> self = ?", "List[int] size = [512]"],
        ["Tensor<[1, 2048]> self = ?", "List[int] size = [2048]"],
        ["Tensor<[2, 7, 8, 64]> self = ?", "List[int] size = [2, 7, 512]"],
        ["Tensor<[1, 768]> self = ?", "List[int] size = [768]"],
        ["Tensor<[1, 3072]> self = ?", "List[int] size = [3072]"],
        ["Tensor<[1, 12, 50, 64]> self = ?", "List[int] size = [12, 50, 64]"],
        ["Tensor<[1, 50, 12, 64]> self = ?", "List[int] size = [1, 50, 768]"],
        ["Tensor<[1, 1, 768]> self = ?", "List[int] size = [768]"],
        ["Tensor<[1, 768, 49]> self = ?", "List[int] size = [1, 768, 7, 7]"],
        ["Tensor<[64]> self = ?", "List[int] size = [1, -1, 1, 1]"],
        ["Tensor<[256]> self = ?", "List[int] size = [1, -1, 1, 1]"],
        ["Tensor<[128]> self = ?", "List[int] size = [1, -1, 1, 1]"],
        ["Tensor<[512]> self = ?", "List[int] size = [1, -1, 1, 1]"],
        ["Tensor<[1024]> self = ?", "List[int] size = [1, -1, 1, 1]"],
        ["Tensor<[2048]> self = ?", "List[int] size = [1, -1, 1, 1]"],
        ["Tensor<[1, 23, 40, 64, 2]> self = ?", "List[int] size = [1, 23, 40, 128]"],
        ["Tensor<[1, 256, 23, 40]> self = ?", "List[int] size = [1, 256, 920]"],
        ["Tensor<[920, 1, 256]> self = ?", "List[int] size = [920, 256]"],
        ["Tensor<[920, 256]> self = ?", "List[int] size = [920, 1, 256]"],
        ["Tensor<[920, 1, 256]> self = ?", "List[int] size = [920, 8, 32]"],
        ["Tensor<[1, 920]> self = ?", "List[int] size = [1, 1, 1, 920]"],
        ["Tensor<[1, 8, 1, 920]> self = ?", "List[int] size = [8, 1, 920]"],
        ["Tensor<[920, 8, 32]> self = ?", "List[int] size = [920, 256]"],
        ["Tensor<[920, 2048]> self = ?", "List[int] size = [920, 1, 2048]"],
        ["Tensor<[920, 1, 2048]> self = ?", "List[int] size = [920, 2048]"],
        ["Tensor<[100, 1, 256]> self = ?", "List[int] size = [100, 256]"],
        ["Tensor<[100, 256]> self = ?", "List[int] size = [100, 1, 256]"],
        ["Tensor<[100, 1, 256]> self = ?", "List[int] size = [100, 8, 32]"],
        ["Tensor<[100, 8, 32]> self = ?", "List[int] size = [100, 256]"],
        ["Tensor<[100, 2048]> self = ?", "List[int] size = [100, 1, 2048]"],
        ["Tensor<[100, 1, 2048]> self = ?", "List[int] size = [100, 2048]"],
        ["Tensor<[6, 1, 100, 256]> self = ?", "List[int] size = [600, 256]"],
        ["Tensor<[600, 92]> self = ?", "List[int] size = [6, 1, 100, 92]"],
        ["Tensor<[600, 256]> self = ?", "List[int] size = [6, 1, 100, 256]"],
        ["Tensor<[600, 4]> self = ?", "List[int] size = [6, 1, 100, 4]"],
        ["Tensor<[1, 25, 768]> self = ?", "List[int] size = [25, 768]"],
        ["Tensor<[25, 768]> self = ?", "List[int] size = [1, 25, 768]"],
        ["Tensor<[1, 25, 768]> self = ?", "List[int] size = [1, 25, 12, 64]"],
        ["Tensor<[1, 12, 25, 64]> self = ?", "List[int] size = [12, 25, 64]"],
        ["Tensor<[1, 12, 64, 25]> self = ?", "List[int] size = [12, 64, 25]"],
        ["Tensor<[12, 25, 25]> self = ?", "List[int] size = [1, 12, 25, 25]"],
        ["Tensor<[1, 12, 25, 25]> self = ?", "List[int] size = [12, 25, 25]"],
        ["Tensor<[12, 25, 64]> self = ?", "List[int] size = [1, 12, 25, 64]"],
        ["Tensor<[1, 25, 12, 64]> self = ?", "List[int] size = [1, 25, 768]"],
        ["Tensor<[25, 3072]> self = ?", "List[int] size = [1, 25, 3072]"],
        ["Tensor<[1, 25, 3072]> self = ?", "List[int] size = [25, 3072]"],
        ["Tensor<[25, 2]> self = ?", "List[int] size = [1, 25, 2]"],
        ["Tensor<[1, 25]> self = ?", "List[int] size = [1, 25]"],
        ["Tensor<[1, 1]> self = ?", "List[int] size = [1]"],
        ["Tensor<[1, 15]> self = ?", "List[int] size = [-1, 15]"],
        ["Tensor<[1, 15, 512]> self = ?", "List[int] size = [15, 512]"],
        ["Tensor<[15, 384]> self = ?", "List[int] size = [1, 15, 384]"],
        ["Tensor<[1, 15, 384]> self = ?", "List[int] size = [1, -1, 6, 64]"],
        ["Tensor<[1, 6, 15, 64]> self = ?", "List[int] size = [6, 15, 64]"],
        ["Tensor<[1, 6, 64, 15]> self = ?", "List[int] size = [6, 64, 15]"],
        ["Tensor<[6, 15, 15]> self = ?", "List[int] size = [1, 6, 15, 15]"],
        ["Tensor<[1, 6, 15, 15]> self = ?", "List[int] size = [6, 15, 15]"],
        ["Tensor<[6, 15, 64]> self = ?", "List[int] size = [1, 6, 15, 64]"],
        ["Tensor<[1, 15, 6, 64]> self = ?", "List[int] size = [1, -1, 384]"],
        ["Tensor<[1, 15, 384]> self = ?", "List[int] size = [15, 384]"],
        ["Tensor<[15, 512]> self = ?", "List[int] size = [1, 15, 512]"],
        ["Tensor<[15, 1024]> self = ?", "List[int] size = [1, 15, 1024]"],
        ["Tensor<[1, 15, 1024]> self = ?", "List[int] size = [15, 1024]"],
        ["Tensor<[1, 1]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[1, 1, 512]> self = ?", "List[int] size = [1, 512]"],
        ["Tensor<[1, 384]> self = ?", "List[int] size = [1, 1, 384]"],
        ["Tensor<[1, 1, 384]> self = ?", "List[int] size = [1, -1, 6, 64]"],
        ["Tensor<[1, 6, 1, 64]> self = ?", "List[int] size = [6, 1, 64]"],
        ["Tensor<[1, 6, 64, 1]> self = ?", "List[int] size = [6, 64, 1]"],
        ["Tensor<[6, 1, 1]> self = ?", "List[int] size = [1, 6, 1, 1]"],
        ["Tensor<[1, 6, 1, 1]> self = ?", "List[int] size = [6, 1, 1]"],
        ["Tensor<[6, 1, 64]> self = ?", "List[int] size = [1, 6, 1, 64]"],
        ["Tensor<[1, 1, 6, 64]> self = ?", "List[int] size = [1, -1, 384]"],
        ["Tensor<[1, 1, 384]> self = ?", "List[int] size = [1, 384]"],
        ["Tensor<[1, 512]> self = ?", "List[int] size = [1, 1, 512]"],
        ["Tensor<[6, 1, 15]> self = ?", "List[int] size = [1, 6, 1, 15]"],
        ["Tensor<[1, 6, 1, 15]> self = ?", "List[int] size = [6, 1, 15]"],
        ["Tensor<[1, 1024]> self = ?", "List[int] size = [1, 1, 1024]"],
        ["Tensor<[1, 1, 1024]> self = ?", "List[int] size = [1, 1024]"],
        ["Tensor<[1, 32128]> self = ?", "List[int] size = [1, 1, 32128]"],
        ["Tensor<[1, 6, 64, 2]> self = ?", "List[int] size = [6, 64, 2]"],
        ["Tensor<[6, 1, 2]> self = ?", "List[int] size = [1, 6, 1, 2]"],
        ["Tensor<[1, 6, 1, 2]> self = ?", "List[int] size = [6, 1, 2]"],
        ["Tensor<[1, 6, 2, 64]> self = ?", "List[int] size = [6, 2, 64]"],
        ["Tensor<[1, 6, 64, s0 + 1]> self = ?", "List[int] size = [6, 64, <s0 + 1>]"],
        ["Tensor<[6, 1, s0 + 1]> self = ?", "List[int] size = [1, 6, 1, <s0 + 1>]"],
        ["Tensor<[1, 6, 1, s0 + 1]> self = ?", "List[int] size = [6, 1, <s0 + 1>]"],
        ["Tensor<[1, 6, s0 + 1, 64]> self = ?", "List[int] size = [6, <s0 + 1>, 64]"],
        ["Tensor<[1, 6, 64, 17]> self = ?", "List[int] size = [6, 64, 17]"],
        ["Tensor<[6, 1, 17]> self = ?", "List[int] size = [1, 6, 1, 17]"],
        ["Tensor<[1, 6, 1, 17]> self = ?", "List[int] size = [6, 1, 17]"],
        ["Tensor<[1, 6, 17, 64]> self = ?", "List[int] size = [6, 17, 64]"],
        ["Tensor<[1, 64, 120, 160]> self = ?", "List[int] size = [1, 64, 19200]"],
        ["Tensor<[1, 19200, 64]> self = ?", "List[int] size = [19200, 64]"],
        ["Tensor<[19200, 64]> self = ?", "List[int] size = [1, 19200, 64]"],
        ["Tensor<[1, 19200, 64]> self = ?", "List[int] size = [1, 19200, 1, 64]"],
        ["Tensor<[1, 64, 19200]> self = ?", "List[int] size = [1, 64, 120, 160]"],
        ["Tensor<[1, 64, 15, 20]> self = ?", "List[int] size = [1, 64, 300]"],
        ["Tensor<[1, 300, 64]> self = ?", "List[int] size = [300, 64]"],
        ["Tensor<[300, 64]> self = ?", "List[int] size = [1, 300, 64]"],
        ["Tensor<[1, 300, 64]> self = ?", "List[int] size = [1, 300, 1, 64]"],
        ["Tensor<[1, 1, 19200, 64]> self = ?", "List[int] size = [1, 19200, 64]"],
        ["Tensor<[1, 1, 64, 300]> self = ?", "List[int] size = [1, 64, 300]"],
        ["Tensor<[1, 19200, 300]> self = ?", "List[int] size = [1, 1, 19200, 300]"],
        ["Tensor<[1, 1, 19200, 300]> self = ?", "List[int] size = [1, 19200, 300]"],
        ["Tensor<[1, 1, 300, 64]> self = ?", "List[int] size = [1, 300, 64]"],
        ["Tensor<[1, 19200, 64]> self = ?", "List[int] size = [1, 1, 19200, 64]"],
        ["Tensor<[1, 19200, 1, 64]> self = ?", "List[int] size = [1, 19200, 64]"],
        ["Tensor<[19200, 256]> self = ?", "List[int] size = [1, 19200, 256]"],
        ["Tensor<[1, 256, 19200]> self = ?", "List[int] size = [1, 256, 120, 160]"],
        ["Tensor<[1, 256, 120, 160]> self = ?", "List[int] size = [1, 256, 19200]"],
        ["Tensor<[1, 19200, 256]> self = ?", "List[int] size = [19200, 256]"],
        ["Tensor<[1, 19200, 64]> self = ?", "List[int] size = [1, 120, 160, -1]"],
        ["Tensor<[1, 128, 60, 80]> self = ?", "List[int] size = [1, 128, 4800]"],
        ["Tensor<[1, 4800, 128]> self = ?", "List[int] size = [4800, 128]"],
        ["Tensor<[4800, 128]> self = ?", "List[int] size = [1, 4800, 128]"],
        ["Tensor<[1, 4800, 128]> self = ?", "List[int] size = [1, 4800, 2, 64]"],
        ["Tensor<[1, 128, 4800]> self = ?", "List[int] size = [1, 128, 60, 80]"],
        ["Tensor<[1, 128, 15, 20]> self = ?", "List[int] size = [1, 128, 300]"],
        ["Tensor<[1, 300, 128]> self = ?", "List[int] size = [300, 128]"],
        ["Tensor<[300, 128]> self = ?", "List[int] size = [1, 300, 128]"],
        ["Tensor<[1, 300, 128]> self = ?", "List[int] size = [1, 300, 2, 64]"],
        ["Tensor<[1, 2, 4800, 64]> self = ?", "List[int] size = [2, 4800, 64]"],
        ["Tensor<[1, 2, 64, 300]> self = ?", "List[int] size = [2, 64, 300]"],
        ["Tensor<[2, 4800, 300]> self = ?", "List[int] size = [1, 2, 4800, 300]"],
        ["Tensor<[1, 2, 4800, 300]> self = ?", "List[int] size = [2, 4800, 300]"],
        ["Tensor<[1, 2, 300, 64]> self = ?", "List[int] size = [2, 300, 64]"],
        ["Tensor<[2, 4800, 64]> self = ?", "List[int] size = [1, 2, 4800, 64]"],
        ["Tensor<[1, 4800, 2, 64]> self = ?", "List[int] size = [1, 4800, 128]"],
        ["Tensor<[4800, 512]> self = ?", "List[int] size = [1, 4800, 512]"],
        ["Tensor<[1, 512, 4800]> self = ?", "List[int] size = [1, 512, 60, 80]"],
        ["Tensor<[1, 512, 60, 80]> self = ?", "List[int] size = [1, 512, 4800]"],
        ["Tensor<[1, 4800, 512]> self = ?", "List[int] size = [4800, 512]"],
        ["Tensor<[1, 4800, 128]> self = ?", "List[int] size = [1, 60, 80, -1]"],
        ["Tensor<[1, 320, 30, 40]> self = ?", "List[int] size = [1, 320, 1200]"],
        ["Tensor<[1, 1200, 320]> self = ?", "List[int] size = [1200, 320]"],
        ["Tensor<[1200, 320]> self = ?", "List[int] size = [1, 1200, 320]"],
        ["Tensor<[1, 1200, 320]> self = ?", "List[int] size = [1, 1200, 5, 64]"],
        ["Tensor<[1, 320, 1200]> self = ?", "List[int] size = [1, 320, 30, 40]"],
        ["Tensor<[1, 320, 15, 20]> self = ?", "List[int] size = [1, 320, 300]"],
        ["Tensor<[1, 300, 320]> self = ?", "List[int] size = [300, 320]"],
        ["Tensor<[300, 320]> self = ?", "List[int] size = [1, 300, 320]"],
        ["Tensor<[1, 300, 320]> self = ?", "List[int] size = [1, 300, 5, 64]"],
        ["Tensor<[1, 5, 1200, 64]> self = ?", "List[int] size = [5, 1200, 64]"],
        ["Tensor<[1, 5, 64, 300]> self = ?", "List[int] size = [5, 64, 300]"],
        ["Tensor<[5, 1200, 300]> self = ?", "List[int] size = [1, 5, 1200, 300]"],
        ["Tensor<[1, 5, 1200, 300]> self = ?", "List[int] size = [5, 1200, 300]"],
        ["Tensor<[1, 5, 300, 64]> self = ?", "List[int] size = [5, 300, 64]"],
        ["Tensor<[5, 1200, 64]> self = ?", "List[int] size = [1, 5, 1200, 64]"],
        ["Tensor<[1, 1200, 5, 64]> self = ?", "List[int] size = [1, 1200, 320]"],
        ["Tensor<[1200, 1280]> self = ?", "List[int] size = [1, 1200, 1280]"],
        ["Tensor<[1, 1280, 1200]> self = ?", "List[int] size = [1, 1280, 30, 40]"],
        ["Tensor<[1, 1280, 30, 40]> self = ?", "List[int] size = [1, 1280, 1200]"],
        ["Tensor<[1, 1200, 1280]> self = ?", "List[int] size = [1200, 1280]"],
        ["Tensor<[1, 1200, 320]> self = ?", "List[int] size = [1, 30, 40, -1]"],
        ["Tensor<[1, 512, 15, 20]> self = ?", "List[int] size = [1, 512, 300]"],
        ["Tensor<[1, 300, 512]> self = ?", "List[int] size = [300, 512]"],
        ["Tensor<[300, 512]> self = ?", "List[int] size = [1, 300, 512]"],
        ["Tensor<[1, 300, 512]> self = ?", "List[int] size = [1, 300, 8, 64]"],
        ["Tensor<[1, 8, 300, 64]> self = ?", "List[int] size = [8, 300, 64]"],
        ["Tensor<[1, 8, 64, 300]> self = ?", "List[int] size = [8, 64, 300]"],
        ["Tensor<[8, 300, 300]> self = ?", "List[int] size = [1, 8, 300, 300]"],
        ["Tensor<[1, 8, 300, 300]> self = ?", "List[int] size = [8, 300, 300]"],
        ["Tensor<[8, 300, 64]> self = ?", "List[int] size = [1, 8, 300, 64]"],
        ["Tensor<[1, 300, 8, 64]> self = ?", "List[int] size = [1, 300, 512]"],
        ["Tensor<[300, 2048]> self = ?", "List[int] size = [1, 300, 2048]"],
        ["Tensor<[1, 2048, 300]> self = ?", "List[int] size = [1, 2048, 15, 20]"],
        ["Tensor<[1, 2048, 15, 20]> self = ?", "List[int] size = [1, 2048, 300]"],
        ["Tensor<[1, 300, 2048]> self = ?", "List[int] size = [300, 2048]"],
        ["Tensor<[1, 300, 512]> self = ?", "List[int] size = [1, 15, 20, -1]"],
        ["Tensor<[1, 7]> self = ?", "List[int] size = [-1, 7]"],
        ["Tensor<[1, 7]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[1, 7, 768]> self = ?", "List[int] size = [-1, 768]"],
        ["Tensor<[7, 2304]> self = ?", "List[int] size = [1, 7, 2304]"],
        ["Tensor<[1, 7, 768]> self = ?", "List[int] size = [1, 7, 12, 64]"],
        ["Tensor<[1, 12, 7, 64]> self = ?", "List[int] size = [12, 7, 64]"],
        ["Tensor<[1, 12, 64, 7]> self = ?", "List[int] size = [12, 64, 7]"],
        ["Tensor<[12, 7, 7]> self = ?", "List[int] size = [1, 12, 7, 7]"],
        ["Tensor<[1, 12, 7, 7]> self = ?", "List[int] size = [12, 7, 7]"],
        ["Tensor<[12, 7, 64]> self = ?", "List[int] size = [1, 12, 7, 64]"],
        ["Tensor<[1, 7, 12, 64]> self = ?", "List[int] size = [1, 7, 768]"],
        ["Tensor<[7, 768]> self = ?", "List[int] size = [1, 7, 768]"],
        ["Tensor<[7, 3072]> self = ?", "List[int] size = [1, 7, 3072]"],
        ["Tensor<[1, 7, 3072]> self = ?", "List[int] size = [-1, 3072]"],
        ["Tensor<[1, 7, 768]> self = ?", "List[int] size = [-1, 7, 768]"],
        ["Tensor<[1, 7, 768]> self = ?", "List[int] size = [7, 768]"],
        ["Tensor<[7, 2]> self = ?", "List[int] size = [1, 7, 2]"],
        ["Tensor<[1, 1, 45]> self = ?", "List[int] size = [1, 45]"],
        ["Tensor<[1, 45]> self = ?", "List[int] size = [-1, 45]"],
        ["Tensor<[1, 45, 768]> self = ?", "List[int] size = [45, 768]"],
        ["Tensor<[45, 768]> self = ?", "List[int] size = [1, 45, 768]"],
        ["Tensor<[1, 45, 768]> self = ?", "List[int] size = [1, 45, 12, 64]"],
        ["Tensor<[1, 12, 45, 64]> self = ?", "List[int] size = [12, 45, 64]"],
        ["Tensor<[1, 12, 64, 45]> self = ?", "List[int] size = [12, 64, 45]"],
        ["Tensor<[12, 45, 45]> self = ?", "List[int] size = [1, 12, 45, 45]"],
        ["Tensor<[1, 12, 45, 45]> self = ?", "List[int] size = [12, 45, 45]"],
        ["Tensor<[12, 45, 64]> self = ?", "List[int] size = [1, 12, 45, 64]"],
        ["Tensor<[1, 45, 12, 64]> self = ?", "List[int] size = [1, 45, 768]"],
        ["Tensor<[45, 3072]> self = ?", "List[int] size = [1, 45, 3072]"],
        ["Tensor<[1, 45, 3072]> self = ?", "List[int] size = [45, 3072]"],
        ["Tensor<[1, 45, 768]> self = ?", "List[int] size = [-1, 45, 768]"],
        ["Tensor<[45, 50257]> self = ?", "List[int] size = [1, 45, 50257]"],
        ["Tensor<[1, 1, 768]> self = ?", "List[int] size = [1, 768]"],
        ["Tensor<[1, 768]> self = ?", "List[int] size = [1, 1, 768]"],
        ["Tensor<[1, 1, 768]> self = ?", "List[int] size = [1, 1, 12, 64]"],
        ["Tensor<[1, 12, 1, 64]> self = ?", "List[int] size = [12, 1, 64]"],
        ["Tensor<[1, 12, 64, 46]> self = ?", "List[int] size = [12, 64, 46]"],
        ["Tensor<[12, 1, 46]> self = ?", "List[int] size = [1, 12, 1, 46]"],
        ["Tensor<[1, 12, 1, 46]> self = ?", "List[int] size = [12, 1, 46]"],
        ["Tensor<[1, 12, 46, 64]> self = ?", "List[int] size = [12, 46, 64]"],
        ["Tensor<[12, 1, 64]> self = ?", "List[int] size = [1, 12, 1, 64]"],
        ["Tensor<[1, 1, 12, 64]> self = ?", "List[int] size = [1, 1, 768]"],
        ["Tensor<[1, 3072]> self = ?", "List[int] size = [1, 1, 3072]"],
        ["Tensor<[1, 1, 3072]> self = ?", "List[int] size = [1, 3072]"],
        ["Tensor<[1, 1, 768]> self = ?", "List[int] size = [-1, 1, 768]"],
        ["Tensor<[1, 50257]> self = ?", "List[int] size = [1, 1, 50257]"],
        ["Tensor<[1, 12, 64, s10 + 1]> self = ?", "List[int] size = [12, 64, <s10 + 1>]"],
        ["Tensor<[12, 1, s10 + 1]> self = ?", "List[int] size = [1, 12, 1, <s10 + 1>]"],
        ["Tensor<[1, 12, 1, s10 + 1]> self = ?", "List[int] size = [12, 1, <s10 + 1>]"],
        ["Tensor<[1, 12, s10 + 1, 64]> self = ?", "List[int] size = [12, <s10 + 1>, 64]"],
        ["Tensor<[1, 1024, 1, 1]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[1, 1000]> self = ?", "List[int] size = [1000]"],
        ["Tensor<[1, 1024]> self = ?", "List[int] size = [1, 1024, 1, 1]"],
        ["Tensor<[1, 64, 1]> self = ?", "List[int] size = [1, 64, 1]"],
        ["Tensor<[1, 1, 32]> self = ?", "List[int] size = [1, 1, 32]"],
        ["Tensor<[1, 64, 32]> self = ?", "List[int] size = [1, 64, 32]"],
        ["Tensor<[1, 3, 256, 256]> self = ?", "List[int] size = [1, 3, 16, 16, 16, 16]"],
        ["Tensor<[1, 256, 768]> self = ?", "List[int] size = [256, 768]"],
        ["Tensor<[256, 512]> self = ?", "List[int] size = [1, 256, 512]"],
        ["Tensor<[1, 256, 512]> self = ?", "List[int] size = [256, 512]"],
        ["Tensor<[256, 256]> self = ?", "List[int] size = [1, 256, 256]"],
        ["Tensor<[1, 256, 256]> self = ?", "List[int] size = [256, 256]"],
        ["Tensor<[1, 256]> self = ?", "List[int] size = [256]"],
        ["Tensor<[256, 768]> self = ?", "List[int] size = [1, 256, 768]"],
        ["Tensor<[1, 256, 768]> self = ?", "List[int] size = [1, 16, 16, 16, 16, 3]"],
        ["Tensor<[1, 64, 12, 12]> self = ?", "List[int] size = [1, 9216]"],
        ["Tensor<[1, 10]> self = ?", "List[int] size = [10]"],
        ["Tensor<[1, 9216]> self = ?", "List[int] size = [1, 64, 12, 12]"],
        ["Tensor<[1, 24, 20, 20]> self = ?", "List[int] size = [1, -1, 4, 20, 20]"],
        ["Tensor<[1, 24, 10, 10]> self = ?", "List[int] size = [1, -1, 4, 10, 10]"],
        ["Tensor<[1, 24, 5, 5]> self = ?", "List[int] size = [1, -1, 4, 5, 5]"],
        ["Tensor<[1, 24, 3, 3]> self = ?", "List[int] size = [1, -1, 4, 3, 3]"],
        ["Tensor<[1, 24, 2, 2]> self = ?", "List[int] size = [1, -1, 4, 2, 2]"],
        ["Tensor<[1, 24, 1, 1]> self = ?", "List[int] size = [1, -1, 4, 1, 1]"],
        ["Tensor<[1, 1, 1, 6, 4]> self = ?", "List[int] size = [1, -1, 4]"],
        ["Tensor<[1, 546, 20, 20]> self = ?", "List[int] size = [1, -1, 91, 20, 20]"],
        ["Tensor<[1, 546, 10, 10]> self = ?", "List[int] size = [1, -1, 91, 10, 10]"],
        ["Tensor<[1, 546, 5, 5]> self = ?", "List[int] size = [1, -1, 91, 5, 5]"],
        ["Tensor<[1, 546, 3, 3]> self = ?", "List[int] size = [1, -1, 91, 3, 3]"],
        ["Tensor<[1, 546, 2, 2]> self = ?", "List[int] size = [1, -1, 91, 2, 2]"],
        ["Tensor<[1, 546, 1, 1]> self = ?", "List[int] size = [1, -1, 91, 1, 1]"],
        ["Tensor<[1, 1, 1, 6, 91]> self = ?", "List[int] size = [1, -1, 91]"],
        ["Tensor<[20]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[20]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[400, 12]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[10]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[10]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[100, 12]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[5]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[5]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[25, 12]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[3]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[3]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[9, 12]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[2]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[2]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[4, 12]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[1]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[1]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[1, 1]> self = ?", "List[int] size = [-1]"],
        ["Tensor<[1, 12]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[1, 1280, 1, 1]> self = ?", "List[int] size = [1, 1280]"],
        ["Tensor<[1, 59]> self = ?", "List[int] size = [-1, 59]"],
        ["Tensor<[1, 59, 512]> self = ?", "List[int] size = [59, 512]"],
        ["Tensor<[59, 1024]> self = ?", "List[int] size = [1, 59, 1024]"],
        ["Tensor<[1, 59, 1024]> self = ?", "List[int] size = [59, 1024]"],
        ["Tensor<[1, 59, 1024]> self = ?", "List[int] size = [1, -1, 16, 64]"],
        ["Tensor<[1, 59, 1024]> self = ?", "List[int] size = [1, 59, 16, 64]"],
        ["Tensor<[1, 16, 59, 64]> self = ?", "List[int] size = [16, -1, 64]"],
        ["Tensor<[16, 59, 59]> self = ?", "List[int] size = [1, 16, 59, 59]"],
        ["Tensor<[1, 16, 59, 59]> self = ?", "List[int] size = [16, 59, 59]"],
        ["Tensor<[16, 59, 64]> self = ?", "List[int] size = [1, 16, 59, 64]"],
        ["Tensor<[1, 59, 1024]> self = ?", "List[int] size = [-1, 1024]"],
        ["Tensor<[59, 512]> self = ?", "List[int] size = [1, 59, 512]"],
        ["Tensor<[59, 50272]> self = ?", "List[int] size = [1, 59, 50272]"],
        ["Tensor<[1, 1, 1024]> self = ?", "List[int] size = [1, -1, 16, 64]"],
        ["Tensor<[1, 1, 1024]> self = ?", "List[int] size = [1, 1, 16, 64]"],
        ["Tensor<[1, 16, 1, 64]> self = ?", "List[int] size = [16, -1, 64]"],
        ["Tensor<[1, 16, 60, 64]> self = ?", "List[int] size = [16, -1, 64]"],
        ["Tensor<[16, 1, 60]> self = ?", "List[int] size = [1, 16, 1, 60]"],
        ["Tensor<[1, 16, 1, 60]> self = ?", "List[int] size = [16, 1, 60]"],
        ["Tensor<[16, 1, 64]> self = ?", "List[int] size = [1, 16, 1, 64]"],
        ["Tensor<[1, 1, 16, 64]> self = ?", "List[int] size = [1, 1, 1024]"],
        ["Tensor<[1, 1, 1024]> self = ?", "List[int] size = [-1, 1024]"],
        ["Tensor<[1, 50272]> self = ?", "List[int] size = [1, 1, 50272]"],
        ["Tensor<[1, 16, s10 + 1, 64]> self = ?", "List[int] size = [16, -1, 64]"],
        ["Tensor<[16, 1, s10 + 1]> self = ?", "List[int] size = [1, 16, 1, <s10 + 1>]"],
        ["Tensor<[1, 16, 1, s10 + 1]> self = ?", "List[int] size = [16, 1, <s10 + 1>]"],
        ["Tensor<[1, 256, 1280]> self = ?", "List[int] size = [256, 1280]"],
        ["Tensor<[1, 2048, 768]> self = ?", "List[int] size = [2048, 768]"],
        ["Tensor<[2048, 256]> self = ?", "List[int] size = [1, 2048, 256]"],
        ["Tensor<[2048, 1280]> self = ?", "List[int] size = [1, 2048, 1280]"],
        ["Tensor<[1, 256, 256]> self = ?", "List[int] size = [1, 256, 8, 32]"],
        ["Tensor<[1, 2048, 256]> self = ?", "List[int] size = [1, 2048, 8, 32]"],
        ["Tensor<[1, 2048, 1280]> self = ?", "List[int] size = [1, 2048, 8, 160]"],
        ["Tensor<[1, 8, 256, 32]> self = ?", "List[int] size = [8, 256, 32]"],
        ["Tensor<[1, 8, 32, 2048]> self = ?", "List[int] size = [8, 32, 2048]"],
        ["Tensor<[8, 256, 2048]> self = ?", "List[int] size = [1, 8, 256, 2048]"],
        ["Tensor<[1, 8, 256, 2048]> self = ?", "List[int] size = [8, 256, 2048]"],
        ["Tensor<[1, 8, 2048, 160]> self = ?", "List[int] size = [8, 2048, 160]"],
        ["Tensor<[8, 256, 160]> self = ?", "List[int] size = [1, 8, 256, 160]"],
        ["Tensor<[1, 256, 8, 160]> self = ?", "List[int] size = [1, 256, 1280]"],
        ["Tensor<[256, 1280]> self = ?", "List[int] size = [1, 256, 1280]"],
        ["Tensor<[1, 256, 1280]> self = ?", "List[int] size = [1, 256, 8, 160]"],
        ["Tensor<[1, 8, 32, 256]> self = ?", "List[int] size = [8, 32, 256]"],
        ["Tensor<[8, 256, 256]> self = ?", "List[int] size = [1, 8, 256, 256]"],
        ["Tensor<[1, 8, 256, 256]> self = ?", "List[int] size = [8, 256, 256]"],
        ["Tensor<[1, 8, 256, 160]> self = ?", "List[int] size = [8, 256, 160]"],
        ["Tensor<[1, 256, 768]> self = ?", "List[int] size = [1, 256, 8, 96]"],
        ["Tensor<[1, 8, 2048, 32]> self = ?", "List[int] size = [8, 2048, 32]"],
        ["Tensor<[8, 2048, 256]> self = ?", "List[int] size = [1, 8, 2048, 256]"],
        ["Tensor<[1, 8, 2048, 256]> self = ?", "List[int] size = [8, 2048, 256]"],
        ["Tensor<[1, 8, 256, 96]> self = ?", "List[int] size = [8, 256, 96]"],
        ["Tensor<[8, 2048, 96]> self = ?", "List[int] size = [1, 8, 2048, 96]"],
        ["Tensor<[1, 2048, 8, 96]> self = ?", "List[int] size = [1, 2048, 768]"],
        ["Tensor<[2048, 768]> self = ?", "List[int] size = [1, 2048, 768]"],
        ["Tensor<[1, 2048, 768]> self = ?", "List[int] size = [-1, 768]"],
        ["Tensor<[2048, 262]> self = ?", "List[int] size = [1, 2048, 262]"],
        ["Tensor<[1, 512, 1, 1]> self = ?", "List[int] size = [1, 512]"],
        ["Tensor<[1, 512]> self = ?", "List[int] size = [1, 512, 1, 1]"],
        ["Tensor<[1, 2048, 1, 1]> self = ?", "List[int] size = [1, 2048]"],
        ["Tensor<[1, 2048]> self = ?", "List[int] size = [1, 2048, 1, 1]"],
        ["Tensor<[1, 10, 768]> self = ?", "List[int] size = [10, 768]"],
        ["Tensor<[10, 768]> self = ?", "List[int] size = [1, 10, 768]"],
        ["Tensor<[1, 10, 768]> self = ?", "List[int] size = [1, 10, 12, 64]"],
        ["Tensor<[1, 12, 10, 64]> self = ?", "List[int] size = [12, 10, 64]"],
        ["Tensor<[1, 12, 64, 10]> self = ?", "List[int] size = [12, 64, 10]"],
        ["Tensor<[12, 10, 10]> self = ?", "List[int] size = [1, 12, 10, 10]"],
        ["Tensor<[1, 12, 10, 10]> self = ?", "List[int] size = [12, 10, 10]"],
        ["Tensor<[12, 10, 64]> self = ?", "List[int] size = [1, 12, 10, 64]"],
        ["Tensor<[1, 10, 12, 64]> self = ?", "List[int] size = [1, 10, 768]"],
        ["Tensor<[10, 3072]> self = ?", "List[int] size = [1, 10, 3072]"],
        ["Tensor<[1, 10, 3072]> self = ?", "List[int] size = [10, 3072]"],
        ["Tensor<[10, 250002]> self = ?", "List[int] size = [1, 10, 250002]"],
        ["Tensor<[1, 32, 128, 128]> self = ?", "List[int] size = [1, 32, 16384]"],
        ["Tensor<[1, 16384, 32]> self = ?", "List[int] size = [16384, 32]"],
        ["Tensor<[16384, 32]> self = ?", "List[int] size = [1, 16384, 32]"],
        ["Tensor<[1, 16384, 32]> self = ?", "List[int] size = [1, 16384, 1, 32]"],
        ["Tensor<[1, 32, 16384]> self = ?", "List[int] size = [1, 32, 128, 128]"],
        ["Tensor<[1, 32, 16, 16]> self = ?", "List[int] size = [1, 32, 256]"],
        ["Tensor<[1, 256, 32]> self = ?", "List[int] size = [256, 32]"],
        ["Tensor<[256, 32]> self = ?", "List[int] size = [1, 256, 32]"],
        ["Tensor<[1, 256, 32]> self = ?", "List[int] size = [1, 256, 1, 32]"],
        ["Tensor<[1, 1, 16384, 32]> self = ?", "List[int] size = [1, 16384, 32]"],
        ["Tensor<[1, 1, 32, 256]> self = ?", "List[int] size = [1, 32, 256]"],
        ["Tensor<[1, 16384, 256]> self = ?", "List[int] size = [1, 1, 16384, 256]"],
        ["Tensor<[1, 1, 16384, 256]> self = ?", "List[int] size = [1, 16384, 256]"],
        ["Tensor<[1, 1, 256, 32]> self = ?", "List[int] size = [1, 256, 32]"],
        ["Tensor<[1, 16384, 32]> self = ?", "List[int] size = [1, 1, 16384, 32]"],
        ["Tensor<[1, 16384, 1, 32]> self = ?", "List[int] size = [1, 16384, 32]"],
        ["Tensor<[16384, 128]> self = ?", "List[int] size = [1, 16384, 128]"],
        ["Tensor<[1, 128, 16384]> self = ?", "List[int] size = [1, 128, 128, 128]"],
        ["Tensor<[1, 128, 128, 128]> self = ?", "List[int] size = [1, 128, 16384]"],
        ["Tensor<[1, 16384, 128]> self = ?", "List[int] size = [16384, 128]"],
        ["Tensor<[1, 16384, 32]> self = ?", "List[int] size = [1, 128, 128, -1]"],
        ["Tensor<[1, 64, 64, 64]> self = ?", "List[int] size = [1, 64, 4096]"],
        ["Tensor<[1, 4096, 64]> self = ?", "List[int] size = [4096, 64]"],
        ["Tensor<[4096, 64]> self = ?", "List[int] size = [1, 4096, 64]"],
        ["Tensor<[1, 4096, 64]> self = ?", "List[int] size = [1, 4096, 2, 32]"],
        ["Tensor<[1, 64, 4096]> self = ?", "List[int] size = [1, 64, 64, 64]"],
        ["Tensor<[1, 64, 16, 16]> self = ?", "List[int] size = [1, 64, 256]"],
        ["Tensor<[1, 256, 64]> self = ?", "List[int] size = [256, 64]"],
        ["Tensor<[256, 64]> self = ?", "List[int] size = [1, 256, 64]"],
        ["Tensor<[1, 256, 64]> self = ?", "List[int] size = [1, 256, 2, 32]"],
        ["Tensor<[1, 2, 4096, 32]> self = ?", "List[int] size = [2, 4096, 32]"],
        ["Tensor<[1, 2, 32, 256]> self = ?", "List[int] size = [2, 32, 256]"],
        ["Tensor<[2, 4096, 256]> self = ?", "List[int] size = [1, 2, 4096, 256]"],
        ["Tensor<[1, 2, 4096, 256]> self = ?", "List[int] size = [2, 4096, 256]"],
        ["Tensor<[1, 2, 256, 32]> self = ?", "List[int] size = [2, 256, 32]"],
        ["Tensor<[2, 4096, 32]> self = ?", "List[int] size = [1, 2, 4096, 32]"],
        ["Tensor<[1, 4096, 2, 32]> self = ?", "List[int] size = [1, 4096, 64]"],
        ["Tensor<[4096, 256]> self = ?", "List[int] size = [1, 4096, 256]"],
        ["Tensor<[1, 256, 4096]> self = ?", "List[int] size = [1, 256, 64, 64]"],
        ["Tensor<[1, 256, 64, 64]> self = ?", "List[int] size = [1, 256, 4096]"],
        ["Tensor<[1, 4096, 256]> self = ?", "List[int] size = [4096, 256]"],
        ["Tensor<[1, 4096, 64]> self = ?", "List[int] size = [1, 64, 64, -1]"],
        ["Tensor<[1, 160, 32, 32]> self = ?", "List[int] size = [1, 160, 1024]"],
        ["Tensor<[1, 1024, 160]> self = ?", "List[int] size = [1024, 160]"],
        ["Tensor<[1024, 160]> self = ?", "List[int] size = [1, 1024, 160]"],
        ["Tensor<[1, 1024, 160]> self = ?", "List[int] size = [1, 1024, 5, 32]"],
        ["Tensor<[1, 160, 1024]> self = ?", "List[int] size = [1, 160, 32, 32]"],
        ["Tensor<[1, 160, 16, 16]> self = ?", "List[int] size = [1, 160, 256]"],
        ["Tensor<[1, 256, 160]> self = ?", "List[int] size = [256, 160]"],
        ["Tensor<[256, 160]> self = ?", "List[int] size = [1, 256, 160]"],
        ["Tensor<[1, 256, 160]> self = ?", "List[int] size = [1, 256, 5, 32]"],
        ["Tensor<[1, 5, 1024, 32]> self = ?", "List[int] size = [5, 1024, 32]"],
        ["Tensor<[1, 5, 32, 256]> self = ?", "List[int] size = [5, 32, 256]"],
        ["Tensor<[5, 1024, 256]> self = ?", "List[int] size = [1, 5, 1024, 256]"],
        ["Tensor<[1, 5, 1024, 256]> self = ?", "List[int] size = [5, 1024, 256]"],
        ["Tensor<[1, 5, 256, 32]> self = ?", "List[int] size = [5, 256, 32]"],
        ["Tensor<[5, 1024, 32]> self = ?", "List[int] size = [1, 5, 1024, 32]"],
        ["Tensor<[1, 1024, 5, 32]> self = ?", "List[int] size = [1, 1024, 160]"],
        ["Tensor<[1024, 640]> self = ?", "List[int] size = [1, 1024, 640]"],
        ["Tensor<[1, 640, 1024]> self = ?", "List[int] size = [1, 640, 32, 32]"],
        ["Tensor<[1, 640, 32, 32]> self = ?", "List[int] size = [1, 640, 1024]"],
        ["Tensor<[1, 1024, 640]> self = ?", "List[int] size = [1024, 640]"],
        ["Tensor<[1, 1024, 160]> self = ?", "List[int] size = [1, 32, 32, -1]"],
        ["Tensor<[1, 256, 16, 16]> self = ?", "List[int] size = [1, 256, 256]"],
        ["Tensor<[8, 256, 32]> self = ?", "List[int] size = [1, 8, 256, 32]"],
        ["Tensor<[1, 256, 8, 32]> self = ?", "List[int] size = [1, 256, 256]"],
        ["Tensor<[1, 1024, 256]> self = ?", "List[int] size = [1, 1024, 16, 16]"],
        ["Tensor<[1, 1024, 16, 16]> self = ?", "List[int] size = [1, 1024, 256]"],
        ["Tensor<[1, 256, 256]> self = ?", "List[int] size = [1, 16, 16, -1]"],
        ["Tensor<[16384, 256]> self = ?", "List[int] size = [1, 16384, 256]"],
        ["Tensor<[1, 256, 16384]> self = ?", "List[int] size = [1, 256, 128, 128]"],
        ["Tensor<[1024, 256]> self = ?", "List[int] size = [1, 1024, 256]"],
        ["Tensor<[1, 256, 1024]> self = ?", "List[int] size = [1, 256, 32, 32]"],
        ["Tensor<[1, 256, 256]> self = ?", "List[int] size = [1, 256, 16, 16]"],
        ["Tensor<[1, 1, 256]> self = ?", "List[int] size = [256]"],
        ["Tensor<[1, 256, 32, 32]> self = ?", "List[int] size = [1, 256, 1024]"],
        ["Tensor<[1, 1024, 256]> self = ?", "List[int] size = [1024, 256]"],
        ["Tensor<[1, 256, 128, 128]> self = ?", "List[int] size = [1, 256, 16384]"],
        ["Tensor<[1, 16384, 256]> self = ?", "List[int] size = [16384, 256]"],
        ["Tensor<[1, 16, 16, 256]> self = ?", "List[int] size = [1, 256, 256]"],
        ["Tensor<[1, 1024]> self = ?", "List[int] size = [1024]"],
        ["Tensor<[8, 32, 256]> self = ?", "List[int] size = [1, 8, 32, 256]"],
        ["Tensor<[1, 32, 32, 160]> self = ?", "List[int] size = [1, 1024, 160]"],
        ["Tensor<[1, 160]> self = ?", "List[int] size = [160]"],
        ["Tensor<[1, 640]> self = ?", "List[int] size = [640]"],
        ["Tensor<[5, 256, 32]> self = ?", "List[int] size = [1, 5, 256, 32]"],
        ["Tensor<[5, 32, 256]> self = ?", "List[int] size = [1, 5, 32, 256]"],
        ["Tensor<[1, 256, 5, 32]> self = ?", "List[int] size = [1, 256, 160]"],
        ["Tensor<[1, 160, 256]> self = ?", "List[int] size = [1, 160, 16, 16]"],
        ["Tensor<[1, 64, 64, 64]> self = ?", "List[int] size = [1, 4096, 64]"],
        ["Tensor<[2, 256, 32]> self = ?", "List[int] size = [1, 2, 256, 32]"],
        ["Tensor<[2, 32, 256]> self = ?", "List[int] size = [1, 2, 32, 256]"],
        ["Tensor<[1, 256, 2, 32]> self = ?", "List[int] size = [1, 256, 64]"],
        ["Tensor<[1, 64, 256]> self = ?", "List[int] size = [1, 64, 16, 16]"],
        ["Tensor<[1, 128, 128, 32]> self = ?", "List[int] size = [1, 16384, 32]"],
        ["Tensor<[1, 32]> self = ?", "List[int] size = [32]"],
        ["Tensor<[1, 256, 32]> self = ?", "List[int] size = [1, 1, 256, 32]"],
        ["Tensor<[1, 32, 256]> self = ?", "List[int] size = [1, 1, 32, 256]"],
        ["Tensor<[1, 256, 1, 32]> self = ?", "List[int] size = [1, 256, 32]"],
        ["Tensor<[1, 32, 256]> self = ?", "List[int] size = [1, 32, 16, 16]"],
        ["Tensor<[1, 64, 64, 320]> self = ?", "List[int] size = [1, 4096, 320]"],
        ["Tensor<[1, 4096, 320]> self = ?", "List[int] size = [4096, 320]"],
        ["Tensor<[4096, 320]> self = ?", "List[int] size = [1, 4096, 320]"],
        ["Tensor<[1, 4096, 320]> self = ?", "List[int] size = [1, -1, 8, 40]"],
        ["Tensor<[1, 4096, 8, 40]> self = ?", "List[int] size = [1, -1, 320]"],
        ["Tensor<[1, 9, 768]> self = ?", "List[int] size = [9, 768]"],
        ["Tensor<[9, 320]> self = ?", "List[int] size = [1, 9, 320]"],
        ["Tensor<[1, 9, 320]> self = ?", "List[int] size = [1, -1, 8, 40]"],
        ["Tensor<[4096, 2560]> self = ?", "List[int] size = [1, 4096, 2560]"],
        ["Tensor<[1, 4096, 1280]> self = ?", "List[int] size = [4096, 1280]"],
        ["Tensor<[1, 4096, 320]> self = ?", "List[int] size = [1, 64, 64, 320]"],
        ["Tensor<[1, s0, s1, 640]> self = ?", "List[int] size = [1, <s0*s1>, 640]"],
        ["Tensor<[1, s0*s1, 640]> self = ?", "List[int] size = [<s0*s1>, 640]"],
        ["Tensor<[s0*s1, 640]> self = ?", "List[int] size = [1, <s0*s1>, 640]"],
        ["Tensor<[1, s0*s1, 640]> self = ?", "List[int] size = [1, -1, 8, 80]"],
        ["Tensor<[1, s0*s1, 8, 80]> self = ?", "List[int] size = [1, -1, 640]"],
        ["Tensor<[9, 640]> self = ?", "List[int] size = [1, 9, 640]"],
        ["Tensor<[1, 9, 640]> self = ?", "List[int] size = [1, -1, 8, 80]"],
        ["Tensor<[s0*s1, 5120]> self = ?", "List[int] size = [1, <s0*s1>, 5120]"],
        ["Tensor<[1, s0*s1, 2560]> self = ?", "List[int] size = [<s0*s1>, 2560]"],
        ["Tensor<[1, s0*s1, 640]> self = ?", "List[int] size = [1, <s0>, <s1>, 640]"],
        ["Tensor<[1, s1, s2, 1280]> self = ?", "List[int] size = [1, <s1*s2>, 1280]"],
        ["Tensor<[1, s1*s2, 1280]> self = ?", "List[int] size = [<s1*s2>, 1280]"],
        ["Tensor<[s1*s2, 1280]> self = ?", "List[int] size = [1, <s1*s2>, 1280]"],
        ["Tensor<[1, s1*s2, 1280]> self = ?", "List[int] size = [1, -1, 8, 160]"],
        ["Tensor<[1, s1*s2, 8, 160]> self = ?", "List[int] size = [1, -1, 1280]"],
        ["Tensor<[9, 1280]> self = ?", "List[int] size = [1, 9, 1280]"],
        ["Tensor<[1, 9, 1280]> self = ?", "List[int] size = [1, -1, 8, 160]"],
        ["Tensor<[s1*s2, 10240]> self = ?", "List[int] size = [1, <s1*s2>, 10240]"],
        ["Tensor<[1, s1*s2, 5120]> self = ?", "List[int] size = [<s1*s2>, 5120]"],
        ["Tensor<[1, s1*s2, 1280]> self = ?", "List[int] size = [1, <s1>, <s2>, 1280]"],
        ["Tensor<[1, s0, s1, 1280]> self = ?", "List[int] size = [1, <s0*s1>, 1280]"],
        ["Tensor<[1, s0*s1, 1280]> self = ?", "List[int] size = [<s0*s1>, 1280]"],
        ["Tensor<[s0*s1, 1280]> self = ?", "List[int] size = [1, <s0*s1>, 1280]"],
        ["Tensor<[1, s0*s1, 1280]> self = ?", "List[int] size = [1, -1, 8, 160]"],
        ["Tensor<[1, s0*s1, 8, 160]> self = ?", "List[int] size = [1, -1, 1280]"],
        ["Tensor<[s0*s1, 10240]> self = ?", "List[int] size = [1, <s0*s1>, 10240]"],
        ["Tensor<[1, s0*s1, 5120]> self = ?", "List[int] size = [<s0*s1>, 5120]"],
        ["Tensor<[1, s0*s1, 1280]> self = ?", "List[int] size = [1, <s0>, <s1>, 1280]"],
        ["Tensor<[1, s1, s2, 640]> self = ?", "List[int] size = [1, <s1*s2>, 640]"],
        ["Tensor<[1, s1*s2, 640]> self = ?", "List[int] size = [<s1*s2>, 640]"],
        ["Tensor<[s1*s2, 640]> self = ?", "List[int] size = [1, <s1*s2>, 640]"],
        ["Tensor<[1, s1*s2, 640]> self = ?", "List[int] size = [1, -1, 8, 80]"],
        ["Tensor<[1, s1*s2, 8, 80]> self = ?", "List[int] size = [1, -1, 640]"],
        ["Tensor<[s1*s2, 5120]> self = ?", "List[int] size = [1, <s1*s2>, 5120]"],
        ["Tensor<[1, s1*s2, 2560]> self = ?", "List[int] size = [<s1*s2>, 2560]"],
        ["Tensor<[1, s1*s2, 640]> self = ?", "List[int] size = [1, <s1>, <s2>, 640]"],
        ["Tensor<[1, s1, s2, 320]> self = ?", "List[int] size = [1, <s1*s2>, 320]"],
        ["Tensor<[1, s1*s2, 320]> self = ?", "List[int] size = [<s1*s2>, 320]"],
        ["Tensor<[s1*s2, 320]> self = ?", "List[int] size = [1, <s1*s2>, 320]"],
        ["Tensor<[1, s1*s2, 320]> self = ?", "List[int] size = [1, -1, 8, 40]"],
        ["Tensor<[1, s1*s2, 8, 40]> self = ?", "List[int] size = [1, -1, 320]"],
        ["Tensor<[s1*s2, 2560]> self = ?", "List[int] size = [1, <s1*s2>, 2560]"],
        ["Tensor<[1, s1*s2, 320]> self = ?", "List[int] size = [1, <s1>, <s2>, 320]"],
        ["Tensor<[1, 768, 144]> self = ?", "List[int] size = [1, 768, 12, 12]"],
        ["Tensor<[1, 768, 12, 16]> self = ?", "List[int] size = [1, 768, 192]"],
        ["Tensor<[12]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[1, 1500, 768]> self = ?", "List[int] size = [1500, 768]"],
        ["Tensor<[1500, 768]> self = ?", "List[int] size = [1, 1500, 768]"],
        ["Tensor<[1, 1500, 768]> self = ?", "List[int] size = [1, -1, 12, 64]"],
        ["Tensor<[1, 1500, 768]> self = ?", "List[int] size = [1, 1500, 12, 64]"],
        ["Tensor<[1, 1500, 12, 64]> self = ?", "List[int] size = [1, 1500, 768]"],
        ["Tensor<[1500, 3072]> self = ?", "List[int] size = [1, 1500, 3072]"],
        ["Tensor<[1, 1500, 3072]> self = ?", "List[int] size = [1500, 3072]"],
        ["Tensor<[1, 1, 768]> self = ?", "List[int] size = [1, -1, 12, 64]"],
        ["Tensor<[1, 51865]> self = ?", "List[int] size = [1, 1, 51865]"],
        ["Tensor<[1, 4]> self = ?", "List[int] size = [-1, 4]"],
        ["Tensor<[1, 4, 768]> self = ?", "List[int] size = [4, 768]"],
        ["Tensor<[4, 768]> self = ?", "List[int] size = [1, 4, 768]"],
        ["Tensor<[1, 4, 768]> self = ?", "List[int] size = [1, -1, 12, 64]"],
        ["Tensor<[1, 4, 768]> self = ?", "List[int] size = [1, 4, 12, 64]"],
        ["Tensor<[1, 4, 12, 64]> self = ?", "List[int] size = [1, 4, 768]"],
        ["Tensor<[4, 3072]> self = ?", "List[int] size = [1, 4, 3072]"],
        ["Tensor<[1, 4, 3072]> self = ?", "List[int] size = [4, 3072]"],
        ["Tensor<[4, 51865]> self = ?", "List[int] size = [1, 4, 51865]"],
        ["Tensor<[1, 19]> self = ?", "List[int] size = [-1, 19]"],
        ["Tensor<[1, 19]> self = ?", "List[int] size = [-1]"],
        ["Tensor<[19, 1024]> self = ?", "List[int] size = [1, 19, 1024]"],
        ["Tensor<[1, 19, 1024]> self = ?", "List[int] size = [19, 1024]"],
        ["Tensor<[1, 19, 1024]> self = ?", "List[int] size = [1, -1, 16, 64]"],
        ["Tensor<[1, 19, 1024]> self = ?", "List[int] size = [1, 19, 16, 64]"],
        ["Tensor<[1, 16, 19, 64]> self = ?", "List[int] size = [16, -1, 64]"],
        ["Tensor<[16, 19, 19]> self = ?", "List[int] size = [1, 16, 19, 19]"],
        ["Tensor<[1, 16, 19, 19]> self = ?", "List[int] size = [16, 19, 19]"],
        ["Tensor<[16, 19, 64]> self = ?", "List[int] size = [1, 16, 19, 64]"],
        ["Tensor<[19, 4096]> self = ?", "List[int] size = [1, 19, 4096]"],
        ["Tensor<[1, 19, 4096]> self = ?", "List[int] size = [19, 4096]"],
        ["Tensor<[19, 256008]> self = ?", "List[int] size = [1, 19, 256008]"],
        ["Tensor<[1, 19, 256008]> self = ?", "List[int] size = [-1, 256008]"],
        ["Tensor<[1, 192, 32, 42]> self = ?", "List[int] size = [1, 192, 1344]"],
        ["Tensor<[1, 192, 4150]> self = ?", "List[int] size = [1, 192, 50, 83]"],
        ["Tensor<[1, 1445, 192]> self = ?", "List[int] size = [1445, 192]"],
        ["Tensor<[1445, 192]> self = ?", "List[int] size = [1, 1445, 192]"],
        ["Tensor<[1, 1445, 192]> self = ?", "List[int] size = [1, 1445, 3, 64]"],
        ["Tensor<[1, 3, 1445, 64]> self = ?", "List[int] size = [3, 1445, 64]"],
        ["Tensor<[1, 3, 64, 1445]> self = ?", "List[int] size = [3, 64, 1445]"],
        ["Tensor<[3, 1445, 1445]> self = ?", "List[int] size = [1, 3, 1445, 1445]"],
        ["Tensor<[1, 3, 1445, 1445]> self = ?", "List[int] size = [3, 1445, 1445]"],
        ["Tensor<[3, 1445, 64]> self = ?", "List[int] size = [1, 3, 1445, 64]"],
        ["Tensor<[1, 1445, 3, 64]> self = ?", "List[int] size = [1, 1445, 192]"],
        ["Tensor<[1445, 768]> self = ?", "List[int] size = [1, 1445, 768]"],
        ["Tensor<[1, 1445, 768]> self = ?", "List[int] size = [1445, 768]"],
        ["Tensor<[1, 100, 192]> self = ?", "List[int] size = [100, 192]"],
        ["Tensor<[100, 192]> self = ?", "List[int] size = [1, 100, 192]"],
        ["Tensor<[100, 92]> self = ?", "List[int] size = [1, 100, 92]"],
        ["Tensor<[100, 4]> self = ?", "List[int] size = [1, 100, 4]"],
        ["Tensor<[1, 9, 128]> self = ?", "List[int] size = [9, 128]"],
        ["Tensor<[9, 768]> self = ?", "List[int] size = [1, 9, 768]"],
        ["Tensor<[1, 9, 768]> self = ?", "List[int] size = [1, 9, 12, 64]"],
        ["Tensor<[1, 12, 9, 64]> self = ?", "List[int] size = [12, 9, 64]"],
        ["Tensor<[1, 12, 64, 9]> self = ?", "List[int] size = [12, 64, 9]"],
        ["Tensor<[12, 9, 9]> self = ?", "List[int] size = [1, 12, 9, 9]"],
        ["Tensor<[1, 12, 9, 9]> self = ?", "List[int] size = [12, 9, 9]"],
        ["Tensor<[12, 9, 64]> self = ?", "List[int] size = [1, 12, 9, 64]"],
        ["Tensor<[9, 3072]> self = ?", "List[int] size = [1, 9, 3072]"],
        ["Tensor<[1, 9, 3072]> self = ?", "List[int] size = [9, 3072]"],
        ["Tensor<[9, 128]> self = ?", "List[int] size = [1, 9, 128]"],
        ["Tensor<[9, 30000]> self = ?", "List[int] size = [1, 9, 30000]"],
        ["Tensor<[1, 12, 128]> self = ?", "List[int] size = [12, 128]"],
        ["Tensor<[12, 768]> self = ?", "List[int] size = [1, 12, 768]"],
        ["Tensor<[1, 12, 768]> self = ?", "List[int] size = [12, 768]"],
        ["Tensor<[1, 12, 768]> self = ?", "List[int] size = [1, 12, 12, 64]"],
        ["Tensor<[1, 12, 12, 64]> self = ?", "List[int] size = [12, 12, 64]"],
        ["Tensor<[1, 12, 64, 12]> self = ?", "List[int] size = [12, 64, 12]"],
        ["Tensor<[12, 12, 12]> self = ?", "List[int] size = [1, 12, 12, 12]"],
        ["Tensor<[1, 12, 12, 12]> self = ?", "List[int] size = [12, 12, 12]"],
        ["Tensor<[12, 12, 64]> self = ?", "List[int] size = [1, 12, 12, 64]"],
        ["Tensor<[12, 3072]> self = ?", "List[int] size = [1, 12, 3072]"],
        ["Tensor<[1, 12, 3072]> self = ?", "List[int] size = [12, 3072]"],
        ["Tensor<[12, 2]> self = ?", "List[int] size = [1, 12, 2]"],
        ["Tensor<[9, 1024]> self = ?", "List[int] size = [1, 9, 1024]"],
        ["Tensor<[1, 9, 1024]> self = ?", "List[int] size = [9, 1024]"],
        ["Tensor<[1, 9, 1024]> self = ?", "List[int] size = [1, 9, 16, 64]"],
        ["Tensor<[1, 16, 9, 64]> self = ?", "List[int] size = [16, 9, 64]"],
        ["Tensor<[1, 16, 64, 9]> self = ?", "List[int] size = [16, 64, 9]"],
        ["Tensor<[16, 9, 9]> self = ?", "List[int] size = [1, 16, 9, 9]"],
        ["Tensor<[1, 16, 9, 9]> self = ?", "List[int] size = [16, 9, 9]"],
        ["Tensor<[16, 9, 64]> self = ?", "List[int] size = [1, 16, 9, 64]"],
        ["Tensor<[9, 4096]> self = ?", "List[int] size = [1, 9, 4096]"],
        ["Tensor<[1, 9, 4096]> self = ?", "List[int] size = [9, 4096]"],
        ["Tensor<[9, 2048]> self = ?", "List[int] size = [1, 9, 2048]"],
        ["Tensor<[1, 9, 2048]> self = ?", "List[int] size = [9, 2048]"],
        ["Tensor<[1, 9, 2048]> self = ?", "List[int] size = [1, 9, 16, 128]"],
        ["Tensor<[1, 16, 9, 128]> self = ?", "List[int] size = [16, 9, 128]"],
        ["Tensor<[1, 16, 128, 9]> self = ?", "List[int] size = [16, 128, 9]"],
        ["Tensor<[16, 9, 128]> self = ?", "List[int] size = [1, 16, 9, 128]"],
        ["Tensor<[9, 8192]> self = ?", "List[int] size = [1, 9, 8192]"],
        ["Tensor<[1, 9, 8192]> self = ?", "List[int] size = [9, 8192]"],
        ["Tensor<[1, 9, 4096]> self = ?", "List[int] size = [1, 9, 64, 64]"],
        ["Tensor<[1, 64, 9, 64]> self = ?", "List[int] size = [64, 9, 64]"],
        ["Tensor<[1, 64, 64, 9]> self = ?", "List[int] size = [64, 64, 9]"],
        ["Tensor<[64, 9, 9]> self = ?", "List[int] size = [1, 64, 9, 9]"],
        ["Tensor<[1, 64, 9, 9]> self = ?", "List[int] size = [64, 9, 9]"],
        ["Tensor<[64, 9, 64]> self = ?", "List[int] size = [1, 64, 9, 64]"],
        ["Tensor<[9, 16384]> self = ?", "List[int] size = [1, 9, 16384]"],
        ["Tensor<[1, 9, 16384]> self = ?", "List[int] size = [9, 16384]"],
        ["Tensor<[1, 5]> self = ?", "List[int] size = [-1, 5]"],
        ["Tensor<[1, 5]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[1, 5, 1024]> self = ?", "List[int] size = [5, 1024]"],
        ["Tensor<[5, 3072]> self = ?", "List[int] size = [1, 5, 3072]"],
        ["Tensor<[1, 5, 3072]> self = ?", "List[int] size = [1, 5, 4, -1]"],
        ["Tensor<[1, 5, 4, 256]> self = ?", "List[int] size = [1, 5, 4, 4, 64]"],
        ["Tensor<[1, 5, 1, 16, 2]> self = ?", "List[int] size = [1, 5, 1, 32]"],
        ["Tensor<[1, 5, 16, 16, 2]> self = ?", "List[int] size = [1, 5, 16, 32]"],
        ["Tensor<[1, 16, 5, 64]> self = ?", "List[int] size = [16, 5, 64]"],
        ["Tensor<[1, 16, 64, 5]> self = ?", "List[int] size = [16, 64, 5]"],
        ["Tensor<[16, 5, 5]> self = ?", "List[int] size = [1, 16, 5, 5]"],
        ["Tensor<[1, 16, 5, 5]> self = ?", "List[int] size = [16, 5, 5]"],
        ["Tensor<[16, 5, 64]> self = ?", "List[int] size = [1, 16, 5, 64]"],
        ["Tensor<[1, 5, 16, 64]> self = ?", "List[int] size = [1, 5, 1024]"],
        ["Tensor<[5, 1024]> self = ?", "List[int] size = [1, 5, 1024]"],
        ["Tensor<[5, 4096]> self = ?", "List[int] size = [1, 5, 4096]"],
        ["Tensor<[1, 5, 4096]> self = ?", "List[int] size = [5, 4096]"],
        ["Tensor<[1, 5, 1024]> self = ?", "List[int] size = [1, 5, 1024]"],
        ["Tensor<[5, 51200]> self = ?", "List[int] size = [1, 5, 51200]"],
        ["Tensor<[1, 6]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[1, 1, 3072]> self = ?", "List[int] size = [1, 1, 4, -1]"],
        ["Tensor<[1, 1, 4, 256]> self = ?", "List[int] size = [1, 1, 4, 4, 64]"],
        ["Tensor<[1, 1, 1, 16, 2]> self = ?", "List[int] size = [1, 1, 1, 32]"],
        ["Tensor<[1, 1, 16, 16, 2]> self = ?", "List[int] size = [1, 1, 16, 32]"],
        ["Tensor<[1, 16, 1, 64]> self = ?", "List[int] size = [16, 1, 64]"],
        ["Tensor<[1, 16, 64, 6]> self = ?", "List[int] size = [16, 64, 6]"],
        ["Tensor<[16, 1, 6]> self = ?", "List[int] size = [1, 16, 1, 6]"],
        ["Tensor<[1, 16, 1, 6]> self = ?", "List[int] size = [16, 1, 6]"],
        ["Tensor<[1, 16, 6, 64]> self = ?", "List[int] size = [16, 6, 64]"],
        ["Tensor<[1, 4096]> self = ?", "List[int] size = [1, 1, 4096]"],
        ["Tensor<[1, 1, 4096]> self = ?", "List[int] size = [1, 4096]"],
        ["Tensor<[1, 1, 1024]> self = ?", "List[int] size = [1, 1, 1024]"],
        ["Tensor<[1, 51200]> self = ?", "List[int] size = [1, 1, 51200]"],
        ["Tensor<[1, s10 + 1]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[1, 16, 64, s10 + 1]> self = ?", "List[int] size = [16, 64, <s10 + 1>]"],
        ["Tensor<[1, 16, s10 + 1, 64]> self = ?", "List[int] size = [16, <s10 + 1>, 64]"],
        ["Tensor<[1, 1024, 1, 1]> self = ?", "List[int] size = [1, 1024]"],
        ["Tensor<[1, 2208, 1, 1]> self = ?", "List[int] size = [1, 2208]"],
        ["Tensor<[1, 1664, 1, 1]> self = ?", "List[int] size = [1, 1664]"],
        ["Tensor<[1, 1920, 1, 1]> self = ?", "List[int] size = [1, 1920]"],
        ["Tensor<[1, 16, 768]> self = ?", "List[int] size = [16, 768]"],
        ["Tensor<[16, 768]> self = ?", "List[int] size = [1, 16, 768]"],
        ["Tensor<[1, 16, 768]> self = ?", "List[int] size = [1, -1, 12, 64]"],
        ["Tensor<[1, 12, 16, 64]> self = ?", "List[int] size = [12, 16, 64]"],
        ["Tensor<[1, 12, 64, 16]> self = ?", "List[int] size = [12, 64, 16]"],
        ["Tensor<[12, 16, 16]> self = ?", "List[int] size = [1, 12, 16, 16]"],
        ["Tensor<[1, 16]> self = ?", "List[int] size = [1, 1, 1, 16]"],
        ["Tensor<[1, 12, 16, 16]> self = ?", "List[int] size = [12, 16, 16]"],
        ["Tensor<[12, 16, 64]> self = ?", "List[int] size = [1, 12, 16, 64]"],
        ["Tensor<[1, 16, 12, 64]> self = ?", "List[int] size = [1, -1, 768]"],
        ["Tensor<[16, 3072]> self = ?", "List[int] size = [1, 16, 3072]"],
        ["Tensor<[1, 16, 3072]> self = ?", "List[int] size = [16, 3072]"],
        ["Tensor<[1, 1000, 1, 1]> self = ?", "List[int] size = [1, 1000]"],
        ["Tensor<[1, 1000]> self = ?", "List[int] size = [1, 1000, 1, 1]"],
        ["Tensor<[1, 768, 14, 14]> self = ?", "List[int] size = [1, 768, 196]"],
        ["Tensor<[1, 197, 768]> self = ?", "List[int] size = [197, 768]"],
        ["Tensor<[197, 768]> self = ?", "List[int] size = [1, 197, 768]"],
        ["Tensor<[1, 197, 768]> self = ?", "List[int] size = [1, 197, 12, 64]"],
        ["Tensor<[1, 12, 197, 64]> self = ?", "List[int] size = [12, 197, 64]"],
        ["Tensor<[1, 12, 64, 197]> self = ?", "List[int] size = [12, 64, 197]"],
        ["Tensor<[12, 197, 197]> self = ?", "List[int] size = [1, 12, 197, 197]"],
        ["Tensor<[1, 12, 197, 197]> self = ?", "List[int] size = [12, 197, 197]"],
        ["Tensor<[12, 197, 64]> self = ?", "List[int] size = [1, 12, 197, 64]"],
        ["Tensor<[1, 197, 12, 64]> self = ?", "List[int] size = [1, 197, 768]"],
        ["Tensor<[197, 3072]> self = ?", "List[int] size = [1, 197, 3072]"],
        ["Tensor<[1, 197, 3072]> self = ?", "List[int] size = [197, 3072]"],
        ["Tensor<[12, 64, 197]> self = ?", "List[int] size = [1, 12, 64, 197]"],
        ["Tensor<[1, 768, 196]> self = ?", "List[int] size = [1, 768, 14, 14]"],
        ["Tensor<[1, 1280]> self = ?", "List[int] size = [1, 1280, 1, 1]"],
        ["Tensor<[1, 1536, 1, 1]> self = ?", "List[int] size = [1, 1536]"],
        ["Tensor<[1, 1536]> self = ?", "List[int] size = [1, 1536, 1, 1]"],
        ["Tensor<[197, 197]> self = ?", "List[int] size = [-1]"],
        ["Tensor<[38809, 12]> self = ?", "List[int] size = [197, 197, -1]"],
        ["Tensor<[197, 197, 12]> self = ?", "List[int] size = [38809, 12]"],
        ["Tensor<[1, 1024, 14, 14]> self = ?", "List[int] size = [1, 1024, 196]"],
        ["Tensor<[1, 197, 1024]> self = ?", "List[int] size = [197, 1024]"],
        ["Tensor<[197, 1024]> self = ?", "List[int] size = [1, 197, 1024]"],
        ["Tensor<[1, 197, 1024]> self = ?", "List[int] size = [1, 197, 16, 64]"],
        ["Tensor<[1, 16, 197, 64]> self = ?", "List[int] size = [16, 197, 64]"],
        ["Tensor<[1, 16, 64, 197]> self = ?", "List[int] size = [16, 64, 197]"],
        ["Tensor<[16, 197, 197]> self = ?", "List[int] size = [1, 16, 197, 197]"],
        ["Tensor<[38809, 16]> self = ?", "List[int] size = [197, 197, -1]"],
        ["Tensor<[1, 16, 197, 197]> self = ?", "List[int] size = [16, 197, 197]"],
        ["Tensor<[16, 197, 64]> self = ?", "List[int] size = [1, 16, 197, 64]"],
        ["Tensor<[1, 197, 16, 64]> self = ?", "List[int] size = [1, 197, 1024]"],
        ["Tensor<[197, 4096]> self = ?", "List[int] size = [1, 197, 4096]"],
        ["Tensor<[1, 197, 4096]> self = ?", "List[int] size = [197, 4096]"],
        ["Tensor<[1, 1, 1024]> self = ?", "List[int] size = [1024]"],
        ["Tensor<[1, 4096]> self = ?", "List[int] size = [4096]"],
        ["Tensor<[197, 197, 16]> self = ?", "List[int] size = [38809, 16]"],
        ["Tensor<[16, 64, 197]> self = ?", "List[int] size = [1, 16, 64, 197]"],
        ["Tensor<[1, 1024, 196]> self = ?", "List[int] size = [1, 1024, 14, 14]"],
        ["Tensor<[1, 768, 196]> self = ?", "List[int] size = [768, 196]"],
        ["Tensor<[768, 384]> self = ?", "List[int] size = [1, 768, 384]"],
        ["Tensor<[1, 768, 384]> self = ?", "List[int] size = [768, 384]"],
        ["Tensor<[768, 196]> self = ?", "List[int] size = [1, 768, 196]"],
        ["Tensor<[1, 196, 768]> self = ?", "List[int] size = [196, 768]"],
        ["Tensor<[196, 3072]> self = ?", "List[int] size = [1, 196, 3072]"],
        ["Tensor<[1, 196, 3072]> self = ?", "List[int] size = [196, 3072]"],
        ["Tensor<[196, 768]> self = ?", "List[int] size = [1, 196, 768]"],
        ["Tensor<[1, 21843]> self = ?", "List[int] size = [21843]"],
        ["Tensor<[1, 196]> self = ?", "List[int] size = [196]"],
        ["Tensor<[1, 1, 384]> self = ?", "List[int] size = [384]"],
        ["Tensor<[1, 960, 1, 1]> self = ?", "List[int] size = [1, 960]"],
        ["Tensor<[1, 576, 1, 1]> self = ?", "List[int] size = [1, 576]"],
        ["Tensor<[1, 912, 1, 1]> self = ?", "List[int] size = [1, 912]"],
        ["Tensor<[1, 2520, 1, 1]> self = ?", "List[int] size = [1, 2520]"],
        ["Tensor<[1, 1008, 1, 1]> self = ?", "List[int] size = [1, 1008]"],
        ["Tensor<[1, 400, 1, 1]> self = ?", "List[int] size = [1, 400]"],
        ["Tensor<[1, 672, 1, 1]> self = ?", "List[int] size = [1, 672]"],
        ["Tensor<[1, 7392, 1, 1]> self = ?", "List[int] size = [1, 7392]"],
        ["Tensor<[1, 3024, 1, 1]> self = ?", "List[int] size = [1, 3024]"],
        ["Tensor<[1, 888, 1, 1]> self = ?", "List[int] size = [1, 888]"],
        ["Tensor<[1, 3712, 1, 1]> self = ?", "List[int] size = [1, 3712]"],
        ["Tensor<[1, 1512, 1, 1]> self = ?", "List[int] size = [1, 1512]"],
        ["Tensor<[1, 440, 1, 1]> self = ?", "List[int] size = [1, 440]"],
        ["Tensor<[1, 784, 1, 1]> self = ?", "List[int] size = [1, 784]"],
        ["Tensor<[1, 2016, 1, 1]> self = ?", "List[int] size = [1, 2016]"],
        ["Tensor<[1, 819, 100, 136]> self = ?", "List[int] size = [1, -1, 91, 100, 136]"],
        ["Tensor<[1, 819, 50, 68]> self = ?", "List[int] size = [1, -1, 91, 50, 68]"],
        ["Tensor<[1, 819, 25, 34]> self = ?", "List[int] size = [1, -1, 91, 25, 34]"],
        ["Tensor<[1, 819, 13, 17]> self = ?", "List[int] size = [1, -1, 91, 13, 17]"],
        ["Tensor<[1, 819, 7, 9]> self = ?", "List[int] size = [1, -1, 91, 7, 9]"],
        ["Tensor<[1, 36, 100, 136]> self = ?", "List[int] size = [1, -1, 4, 100, 136]"],
        ["Tensor<[1, 36, 50, 68]> self = ?", "List[int] size = [1, -1, 4, 50, 68]"],
        ["Tensor<[1, 36, 25, 34]> self = ?", "List[int] size = [1, -1, 4, 25, 34]"],
        ["Tensor<[1, 36, 13, 17]> self = ?", "List[int] size = [1, -1, 4, 13, 17]"],
        ["Tensor<[1, 36, 7, 9]> self = ?", "List[int] size = [1, -1, 4, 7, 9]"],
        ["Tensor<[100]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[136]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[13600, 4]> self = ?", "List[int] size = [-1, 1, 4]"],
        ["Tensor<[9, 4]> self = ?", "List[int] size = [1, -1, 4]"],
        ["Tensor<[13600, 9, 4]> self = ?", "List[int] size = [-1, 4]"],
        ["Tensor<[50]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[68]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[3400, 4]> self = ?", "List[int] size = [-1, 1, 4]"],
        ["Tensor<[3400, 9, 4]> self = ?", "List[int] size = [-1, 4]"],
        ["Tensor<[25]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[34]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[850, 4]> self = ?", "List[int] size = [-1, 1, 4]"],
        ["Tensor<[850, 9, 4]> self = ?", "List[int] size = [-1, 4]"],
        ["Tensor<[13]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[17]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[221, 4]> self = ?", "List[int] size = [-1, 1, 4]"],
        ["Tensor<[221, 9, 4]> self = ?", "List[int] size = [-1, 4]"],
        ["Tensor<[7]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[9]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[63, 4]> self = ?", "List[int] size = [-1, 1, 4]"],
        ["Tensor<[63, 9, 4]> self = ?", "List[int] size = [-1, 4]"],
        ["Tensor<[1, 24, 768]> self = ?", "List[int] size = [24, 768]"],
        ["Tensor<[24, 768]> self = ?", "List[int] size = [1, 24, 768]"],
        ["Tensor<[1, 24, 768]> self = ?", "List[int] size = [1, -1, 12, 64]"],
        ["Tensor<[1, 24, 768]> self = ?", "List[int] size = [1, 24, 12, 64]"],
        ["Tensor<[1, 12, 24, 64]> self = ?", "List[int] size = [12, -1, 64]"],
        ["Tensor<[12, 24, 64]> self = ?", "List[int] size = [12, -1, 64]"],
        ["Tensor<[24, 12, 64]> self = ?", "List[int] size = [24, 12, 64]"],
        ["Tensor<[24, 64, 24]> self = ?", "List[int] size = [24, 64, 24]"],
        ["Tensor<[24, 12, 24]> self = ?", "List[int] size = [24, 12, 24]"],
        ["Tensor<[12, 24, 24]> self = ?", "List[int] size = [12, 24, 24]"],
        ["Tensor<[12, 24, 24]> self = ?", "List[int] size = [1, 12, 24, 24]"],
        ["Tensor<[1, 12, 24, 24]> self = ?", "List[int] size = [12, 24, 24]"],
        ["Tensor<[12, 24, 64]> self = ?", "List[int] size = [1, 12, 24, 64]"],
        ["Tensor<[24, 3072]> self = ?", "List[int] size = [1, 24, 3072]"],
        ["Tensor<[1, 24, 3072]> self = ?", "List[int] size = [24, 3072]"],
        ["Tensor<[1, 16, 38, 38]> self = ?", "List[int] size = [1, -1, 4, 38, 38]"],
        ["Tensor<[1, 24, 19, 19]> self = ?", "List[int] size = [1, -1, 4, 19, 19]"],
        ["Tensor<[1, 16, 3, 3]> self = ?", "List[int] size = [1, -1, 4, 3, 3]"],
        ["Tensor<[1, 16, 1, 1]> self = ?", "List[int] size = [1, -1, 4, 1, 1]"],
        ["Tensor<[1, 1, 1, 4, 4]> self = ?", "List[int] size = [1, -1, 4]"],
        ["Tensor<[1, 364, 38, 38]> self = ?", "List[int] size = [1, -1, 91, 38, 38]"],
        ["Tensor<[1, 546, 19, 19]> self = ?", "List[int] size = [1, -1, 91, 19, 19]"],
        ["Tensor<[1, 364, 3, 3]> self = ?", "List[int] size = [1, -1, 91, 3, 3]"],
        ["Tensor<[1, 364, 1, 1]> self = ?", "List[int] size = [1, -1, 91, 1, 1]"],
        ["Tensor<[1, 1, 1, 4, 91]> self = ?", "List[int] size = [1, -1, 91]"],
        ["Tensor<[38]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[38]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[1444, 8]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[19]> self = ?", "List[int] size = [-1, 1]"],
        ["Tensor<[19]> self = ?", "List[int] size = [1, -1]"],
        ["Tensor<[361, 12]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[9, 8]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[1, 8]> self = ?", "List[int] size = [-1, 2]"],
        ["Tensor<[2401, 4]> self = ?", "List[int] size = [49, 49, -1]"],
        ["Tensor<[1, 56, 56, 128]> self = ?", "List[int] size = [1, 8, 7, 8, 7, 128]"],
        ["Tensor<[64, 49, 128]> self = ?", "List[int] size = [3136, 128]"],
        ["Tensor<[3136, 384]> self = ?", "List[int] size = [64, 49, 384]"],
        ["Tensor<[64, 49, 384]> self = ?", "List[int] size = [64, 49, 3, 4, 32]"],
        ["Tensor<[256, 49, 49]> self = ?", "List[int] size = [64, 4, 49, 49]"],
        ["Tensor<[64, 4, 49, 49]> self = ?", "List[int] size = [256, 49, 49]"],
        ["Tensor<[256, 49, 32]> self = ?", "List[int] size = [64, 4, 49, 32]"],
        ["Tensor<[3136, 128]> self = ?", "List[int] size = [64, 49, 128]"],
        ["Tensor<[64, 49, 128]> self = ?", "List[int] size = [1, 8, 8, 7, 7, 128]"],
        ["Tensor<[1, 56, 56, 128]> self = ?", "List[int] size = [3136, 128]"],
        ["Tensor<[3136, 512]> self = ?", "List[int] size = [1, 56, 56, 512]"],
        ["Tensor<[1, 56, 56, 512]> self = ?", "List[int] size = [3136, 512]"],
        ["Tensor<[3136, 128]> self = ?", "List[int] size = [1, 56, 56, 128]"],
        ["Tensor<[56, 56]> self = ?", "List[int] size = [8, 7, 8, 7]"],
        ["Tensor<[64, 4, 49, 49]> self = ?", "List[int] size = [1, 64, 4, 49, 49]"],
        ["Tensor<[1, 64, 4, 49, 49]> self = ?", "List[int] size = [-1, 4, 49, 49]"],
        ["Tensor<[1, 28, 28, 512]> self = ?", "List[int] size = [784, 512]"],
        ["Tensor<[784, 256]> self = ?", "List[int] size = [1, 28, 28, 256]"],
        ["Tensor<[2401, 8]> self = ?", "List[int] size = [49, 49, -1]"],
        ["Tensor<[1, 28, 28, 256]> self = ?", "List[int] size = [1, 4, 7, 4, 7, 256]"],
        ["Tensor<[16, 49, 256]> self = ?", "List[int] size = [784, 256]"],
        ["Tensor<[784, 768]> self = ?", "List[int] size = [16, 49, 768]"],
        ["Tensor<[16, 49, 768]> self = ?", "List[int] size = [16, 49, 3, 8, 32]"],
        ["Tensor<[128, 49, 49]> self = ?", "List[int] size = [16, 8, 49, 49]"],
        ["Tensor<[16, 8, 49, 49]> self = ?", "List[int] size = [128, 49, 49]"],
        ["Tensor<[128, 49, 32]> self = ?", "List[int] size = [16, 8, 49, 32]"],
        ["Tensor<[784, 256]> self = ?", "List[int] size = [16, 49, 256]"],
        ["Tensor<[16, 49, 256]> self = ?", "List[int] size = [1, 4, 4, 7, 7, 256]"],
        ["Tensor<[1, 28, 28, 256]> self = ?", "List[int] size = [784, 256]"],
        ["Tensor<[784, 1024]> self = ?", "List[int] size = [1, 28, 28, 1024]"],
        ["Tensor<[1, 28, 28, 1024]> self = ?", "List[int] size = [784, 1024]"],
        ["Tensor<[28, 28]> self = ?", "List[int] size = [4, 7, 4, 7]"],
        ["Tensor<[16, 8, 49, 49]> self = ?", "List[int] size = [1, 16, 8, 49, 49]"],
        ["Tensor<[1, 16, 8, 49, 49]> self = ?", "List[int] size = [-1, 8, 49, 49]"],
        ["Tensor<[1, 14, 14, 1024]> self = ?", "List[int] size = [196, 1024]"],
        ["Tensor<[196, 512]> self = ?", "List[int] size = [1, 14, 14, 512]"],
        ["Tensor<[2401, 16]> self = ?", "List[int] size = [49, 49, -1]"],
        ["Tensor<[1, 14, 14, 512]> self = ?", "List[int] size = [1, 2, 7, 2, 7, 512]"],
        ["Tensor<[4, 49, 512]> self = ?", "List[int] size = [196, 512]"],
        ["Tensor<[196, 1536]> self = ?", "List[int] size = [4, 49, 1536]"],
        ["Tensor<[4, 49, 1536]> self = ?", "List[int] size = [4, 49, 3, 16, 32]"],
        ["Tensor<[64, 49, 49]> self = ?", "List[int] size = [4, 16, 49, 49]"],
        ["Tensor<[4, 16, 49, 49]> self = ?", "List[int] size = [64, 49, 49]"],
        ["Tensor<[64, 49, 32]> self = ?", "List[int] size = [4, 16, 49, 32]"],
        ["Tensor<[196, 512]> self = ?", "List[int] size = [4, 49, 512]"],
        ["Tensor<[4, 49, 512]> self = ?", "List[int] size = [1, 2, 2, 7, 7, 512]"],
        ["Tensor<[1, 14, 14, 512]> self = ?", "List[int] size = [196, 512]"],
        ["Tensor<[196, 2048]> self = ?", "List[int] size = [1, 14, 14, 2048]"],
        ["Tensor<[1, 14, 14, 2048]> self = ?", "List[int] size = [196, 2048]"],
        ["Tensor<[14, 14]> self = ?", "List[int] size = [2, 7, 2, 7]"],
        ["Tensor<[4, 16, 49, 49]> self = ?", "List[int] size = [1, 4, 16, 49, 49]"],
        ["Tensor<[1, 4, 16, 49, 49]> self = ?", "List[int] size = [-1, 16, 49, 49]"],
        ["Tensor<[1, 7, 7, 2048]> self = ?", "List[int] size = [49, 2048]"],
        ["Tensor<[49, 1024]> self = ?", "List[int] size = [1, 7, 7, 1024]"],
        ["Tensor<[2401, 32]> self = ?", "List[int] size = [49, 49, -1]"],
        ["Tensor<[1, 7, 7, 1024]> self = ?", "List[int] size = [1, 1, 7, 1, 7, 1024]"],
        ["Tensor<[1, 1, 1, 7, 7, 1024]> self = ?", "List[int] size = [1, 49, 1024]"],
        ["Tensor<[1, 49, 1024]> self = ?", "List[int] size = [49, 1024]"],
        ["Tensor<[49, 3072]> self = ?", "List[int] size = [1, 49, 3072]"],
        ["Tensor<[1, 49, 3072]> self = ?", "List[int] size = [1, 49, 3, 32, 32]"],
        ["Tensor<[1, 32, 49, 32]> self = ?", "List[int] size = [32, 49, 32]"],
        ["Tensor<[1, 32, 32, 49]> self = ?", "List[int] size = [32, 32, 49]"],
        ["Tensor<[32, 49, 49]> self = ?", "List[int] size = [1, 32, 49, 49]"],
        ["Tensor<[1, 32, 49, 49]> self = ?", "List[int] size = [32, 49, 49]"],
        ["Tensor<[32, 49, 32]> self = ?", "List[int] size = [1, 32, 49, 32]"],
        ["Tensor<[49, 1024]> self = ?", "List[int] size = [1, 49, 1024]"],
        ["Tensor<[1, 49, 1024]> self = ?", "List[int] size = [1, 1, 1, 7, 7, 1024]"],
        ["Tensor<[1, 1, 7, 1, 7, 1024]> self = ?", "List[int] size = [1, 7, 7, 1024]"],
        ["Tensor<[1, 7, 7, 1024]> self = ?", "List[int] size = [49, 1024]"],
        ["Tensor<[49, 4096]> self = ?", "List[int] size = [1, 7, 7, 4096]"],
        ["Tensor<[1, 7, 7, 4096]> self = ?", "List[int] size = [49, 4096]"],
        ["Tensor<[2401, 3]> self = ?", "List[int] size = [49, 49, -1]"],
        ["Tensor<[1, 56, 56, 96]> self = ?", "List[int] size = [1, 8, 7, 8, 7, 96]"],
        ["Tensor<[64, 49, 96]> self = ?", "List[int] size = [3136, 96]"],
        ["Tensor<[3136, 288]> self = ?", "List[int] size = [64, 49, 288]"],
        ["Tensor<[64, 49, 288]> self = ?", "List[int] size = [64, 49, 3, 3, 32]"],
        ["Tensor<[192, 49, 49]> self = ?", "List[int] size = [64, 3, 49, 49]"],
        ["Tensor<[64, 3, 49, 49]> self = ?", "List[int] size = [192, 49, 49]"],
        ["Tensor<[192, 49, 32]> self = ?", "List[int] size = [64, 3, 49, 32]"],
        ["Tensor<[3136, 96]> self = ?", "List[int] size = [64, 49, 96]"],
        ["Tensor<[64, 49, 96]> self = ?", "List[int] size = [1, 8, 8, 7, 7, 96]"],
        ["Tensor<[1, 56, 56, 96]> self = ?", "List[int] size = [3136, 96]"],
        ["Tensor<[3136, 384]> self = ?", "List[int] size = [1, 56, 56, 384]"],
        ["Tensor<[1, 56, 56, 384]> self = ?", "List[int] size = [3136, 384]"],
        ["Tensor<[3136, 96]> self = ?", "List[int] size = [1, 56, 56, 96]"],
        ["Tensor<[64, 3, 49, 49]> self = ?", "List[int] size = [1, 64, 3, 49, 49]"],
        ["Tensor<[1, 64, 3, 49, 49]> self = ?", "List[int] size = [-1, 3, 49, 49]"],
        ["Tensor<[1, 28, 28, 384]> self = ?", "List[int] size = [784, 384]"],
        ["Tensor<[784, 192]> self = ?", "List[int] size = [1, 28, 28, 192]"],
        ["Tensor<[2401, 6]> self = ?", "List[int] size = [49, 49, -1]"],
        ["Tensor<[1, 28, 28, 192]> self = ?", "List[int] size = [1, 4, 7, 4, 7, 192]"],
        ["Tensor<[16, 49, 192]> self = ?", "List[int] size = [784, 192]"],
        ["Tensor<[784, 576]> self = ?", "List[int] size = [16, 49, 576]"],
        ["Tensor<[16, 49, 576]> self = ?", "List[int] size = [16, 49, 3, 6, 32]"],
        ["Tensor<[96, 49, 49]> self = ?", "List[int] size = [16, 6, 49, 49]"],
        ["Tensor<[16, 6, 49, 49]> self = ?", "List[int] size = [96, 49, 49]"],
        ["Tensor<[96, 49, 32]> self = ?", "List[int] size = [16, 6, 49, 32]"],
        ["Tensor<[784, 192]> self = ?", "List[int] size = [16, 49, 192]"],
        ["Tensor<[16, 49, 192]> self = ?", "List[int] size = [1, 4, 4, 7, 7, 192]"],
        ["Tensor<[1, 28, 28, 192]> self = ?", "List[int] size = [784, 192]"],
        ["Tensor<[784, 768]> self = ?", "List[int] size = [1, 28, 28, 768]"],
        ["Tensor<[1, 28, 28, 768]> self = ?", "List[int] size = [784, 768]"],
        ["Tensor<[16, 6, 49, 49]> self = ?", "List[int] size = [1, 16, 6, 49, 49]"],
        ["Tensor<[1, 16, 6, 49, 49]> self = ?", "List[int] size = [-1, 6, 49, 49]"],
        ["Tensor<[1, 14, 14, 768]> self = ?", "List[int] size = [196, 768]"],
        ["Tensor<[196, 384]> self = ?", "List[int] size = [1, 14, 14, 384]"],
        ["Tensor<[2401, 12]> self = ?", "List[int] size = [49, 49, -1]"],
        ["Tensor<[1, 14, 14, 384]> self = ?", "List[int] size = [1, 2, 7, 2, 7, 384]"],
        ["Tensor<[4, 49, 384]> self = ?", "List[int] size = [196, 384]"],
        ["Tensor<[196, 1152]> self = ?", "List[int] size = [4, 49, 1152]"],
        ["Tensor<[4, 49, 1152]> self = ?", "List[int] size = [4, 49, 3, 12, 32]"],
        ["Tensor<[48, 49, 49]> self = ?", "List[int] size = [4, 12, 49, 49]"],
        ["Tensor<[4, 12, 49, 49]> self = ?", "List[int] size = [48, 49, 49]"],
        ["Tensor<[48, 49, 32]> self = ?", "List[int] size = [4, 12, 49, 32]"],
        ["Tensor<[196, 384]> self = ?", "List[int] size = [4, 49, 384]"],
        ["Tensor<[4, 49, 384]> self = ?", "List[int] size = [1, 2, 2, 7, 7, 384]"],
        ["Tensor<[1, 14, 14, 384]> self = ?", "List[int] size = [196, 384]"],
        ["Tensor<[196, 1536]> self = ?", "List[int] size = [1, 14, 14, 1536]"],
        ["Tensor<[1, 14, 14, 1536]> self = ?", "List[int] size = [196, 1536]"],
        ["Tensor<[4, 12, 49, 49]> self = ?", "List[int] size = [1, 4, 12, 49, 49]"],
        ["Tensor<[1, 4, 12, 49, 49]> self = ?", "List[int] size = [-1, 12, 49, 49]"],
        ["Tensor<[1, 7, 7, 1536]> self = ?", "List[int] size = [49, 1536]"],
        ["Tensor<[49, 768]> self = ?", "List[int] size = [1, 7, 7, 768]"],
        ["Tensor<[2401, 24]> self = ?", "List[int] size = [49, 49, -1]"],
        ["Tensor<[1, 7, 7, 768]> self = ?", "List[int] size = [1, 1, 7, 1, 7, 768]"],
        ["Tensor<[1, 1, 1, 7, 7, 768]> self = ?", "List[int] size = [1, 49, 768]"],
        ["Tensor<[1, 49, 768]> self = ?", "List[int] size = [49, 768]"],
        ["Tensor<[49, 2304]> self = ?", "List[int] size = [1, 49, 2304]"],
        ["Tensor<[1, 49, 2304]> self = ?", "List[int] size = [1, 49, 3, 24, 32]"],
        ["Tensor<[1, 24, 49, 32]> self = ?", "List[int] size = [24, 49, 32]"],
        ["Tensor<[1, 24, 32, 49]> self = ?", "List[int] size = [24, 32, 49]"],
        ["Tensor<[24, 49, 49]> self = ?", "List[int] size = [1, 24, 49, 49]"],
        ["Tensor<[1, 24, 49, 49]> self = ?", "List[int] size = [24, 49, 49]"],
        ["Tensor<[24, 49, 32]> self = ?", "List[int] size = [1, 24, 49, 32]"],
        ["Tensor<[49, 768]> self = ?", "List[int] size = [1, 49, 768]"],
        ["Tensor<[1, 49, 768]> self = ?", "List[int] size = [1, 1, 1, 7, 7, 768]"],
        ["Tensor<[1, 1, 7, 1, 7, 768]> self = ?", "List[int] size = [1, 7, 7, 768]"],
        ["Tensor<[1, 7, 7, 768]> self = ?", "List[int] size = [49, 768]"],
        ["Tensor<[49, 3072]> self = ?", "List[int] size = [1, 7, 7, 3072]"],
        ["Tensor<[1, 7, 7, 3072]> self = ?", "List[int] size = [49, 3072]"],
        ["Tensor<[1, 768, 1, 1]> self = ?", "List[int] size = [1, 768]"],
        ["Tensor<[1, 15, 15, 2]> self = ?", "List[int] size = [225, 2]"],
        ["Tensor<[225, 512]> self = ?", "List[int] size = [1, 15, 15, 512]"],
        ["Tensor<[1, 15, 15, 512]> self = ?", "List[int] size = [225, 512]"],
        ["Tensor<[225, 4]> self = ?", "List[int] size = [1, 15, 15, 4]"],
        ["Tensor<[1, 15, 15, 4]> self = ?", "List[int] size = [-1, 4]"],
        ["Tensor<[4096, 4]> self = ?", "List[int] size = [64, 64, -1]"],
        ["Tensor<[1, 64, 64, 128]> self = ?", "List[int] size = [1, 8, 8, 8, 8, 128]"],
        ["Tensor<[64, 64, 128]> self = ?", "List[int] size = [4096, 128]"],
        ["Tensor<[4096, 384]> self = ?", "List[int] size = [64, 64, 384]"],
        ["Tensor<[64, 64, 384]> self = ?", "List[int] size = [64, 64, 3, 4, 32]"],
        ["Tensor<[256, 64, 64]> self = ?", "List[int] size = [64, 4, 64, 64]"],
        ["Tensor<[64, 4, 64, 64]> self = ?", "List[int] size = [256, 64, 64]"],
        ["Tensor<[256, 64, 32]> self = ?", "List[int] size = [64, 4, 64, 32]"],
        ["Tensor<[4096, 128]> self = ?", "List[int] size = [64, 64, 128]"],
        ["Tensor<[64, 64, 128]> self = ?", "List[int] size = [1, 8, 8, 8, 8, 128]"],
        ["Tensor<[1, 64, 64, 128]> self = ?", "List[int] size = [4096, 128]"],
        ["Tensor<[4096, 512]> self = ?", "List[int] size = [1, 64, 64, 512]"],
        ["Tensor<[1, 64, 64, 512]> self = ?", "List[int] size = [4096, 512]"],
        ["Tensor<[4096, 128]> self = ?", "List[int] size = [1, 64, 64, 128]"],
        ["Tensor<[64, 64]> self = ?", "List[int] size = [8, 8, 8, 8]"],
        ["Tensor<[64, 4, 64, 64]> self = ?", "List[int] size = [1, 64, 4, 64, 64]"],
        ["Tensor<[1, 64, 4, 64, 64]> self = ?", "List[int] size = [-1, 4, 64, 64]"],
        ["Tensor<[1, 32, 32, 512]> self = ?", "List[int] size = [1024, 512]"],
        ["Tensor<[1024, 256]> self = ?", "List[int] size = [1, 32, 32, 256]"],
        ["Tensor<[225, 8]> self = ?", "List[int] size = [1, 15, 15, 8]"],
        ["Tensor<[1, 15, 15, 8]> self = ?", "List[int] size = [-1, 8]"],
        ["Tensor<[4096, 8]> self = ?", "List[int] size = [64, 64, -1]"],
        ["Tensor<[1, 32, 32, 256]> self = ?", "List[int] size = [1, 4, 8, 4, 8, 256]"],
        ["Tensor<[16, 64, 256]> self = ?", "List[int] size = [1024, 256]"],
        ["Tensor<[1024, 768]> self = ?", "List[int] size = [16, 64, 768]"],
        ["Tensor<[16, 64, 768]> self = ?", "List[int] size = [16, 64, 3, 8, 32]"],
        ["Tensor<[128, 64, 64]> self = ?", "List[int] size = [16, 8, 64, 64]"],
        ["Tensor<[16, 8, 64, 64]> self = ?", "List[int] size = [128, 64, 64]"],
        ["Tensor<[128, 64, 32]> self = ?", "List[int] size = [16, 8, 64, 32]"],
        ["Tensor<[1024, 256]> self = ?", "List[int] size = [16, 64, 256]"],
        ["Tensor<[16, 64, 256]> self = ?", "List[int] size = [1, 4, 4, 8, 8, 256]"],
        ["Tensor<[1, 32, 32, 256]> self = ?", "List[int] size = [1024, 256]"],
        ["Tensor<[1024, 1024]> self = ?", "List[int] size = [1, 32, 32, 1024]"],
        ["Tensor<[1, 32, 32, 1024]> self = ?", "List[int] size = [1024, 1024]"],
        ["Tensor<[32, 32]> self = ?", "List[int] size = [4, 8, 4, 8]"],
        ["Tensor<[16, 8, 64, 64]> self = ?", "List[int] size = [1, 16, 8, 64, 64]"],
        ["Tensor<[1, 16, 8, 64, 64]> self = ?", "List[int] size = [-1, 8, 64, 64]"],
        ["Tensor<[1, 16, 16, 1024]> self = ?", "List[int] size = [256, 1024]"],
        ["Tensor<[256, 512]> self = ?", "List[int] size = [1, 16, 16, 512]"],
        ["Tensor<[225, 16]> self = ?", "List[int] size = [1, 15, 15, 16]"],
        ["Tensor<[1, 15, 15, 16]> self = ?", "List[int] size = [-1, 16]"],
        ["Tensor<[4096, 16]> self = ?", "List[int] size = [64, 64, -1]"],
        ["Tensor<[1, 16, 16, 512]> self = ?", "List[int] size = [1, 2, 8, 2, 8, 512]"],
        ["Tensor<[4, 64, 512]> self = ?", "List[int] size = [256, 512]"],
        ["Tensor<[256, 1536]> self = ?", "List[int] size = [4, 64, 1536]"],
        ["Tensor<[4, 64, 1536]> self = ?", "List[int] size = [4, 64, 3, 16, 32]"],
        ["Tensor<[64, 64, 64]> self = ?", "List[int] size = [4, 16, 64, 64]"],
        ["Tensor<[4, 16, 64, 64]> self = ?", "List[int] size = [64, 64, 64]"],
        ["Tensor<[64, 64, 32]> self = ?", "List[int] size = [4, 16, 64, 32]"],
        ["Tensor<[256, 512]> self = ?", "List[int] size = [4, 64, 512]"],
        ["Tensor<[4, 64, 512]> self = ?", "List[int] size = [1, 2, 2, 8, 8, 512]"],
        ["Tensor<[1, 16, 16, 512]> self = ?", "List[int] size = [256, 512]"],
        ["Tensor<[256, 2048]> self = ?", "List[int] size = [1, 16, 16, 2048]"],
        ["Tensor<[1, 16, 16, 2048]> self = ?", "List[int] size = [256, 2048]"],
        ["Tensor<[16, 16]> self = ?", "List[int] size = [2, 8, 2, 8]"],
        ["Tensor<[4, 16, 64, 64]> self = ?", "List[int] size = [1, 4, 16, 64, 64]"],
        ["Tensor<[1, 4, 16, 64, 64]> self = ?", "List[int] size = [-1, 16, 64, 64]"],
        ["Tensor<[1, 8, 8, 2048]> self = ?", "List[int] size = [64, 2048]"],
        ["Tensor<[64, 1024]> self = ?", "List[int] size = [1, 8, 8, 1024]"],
        ["Tensor<[225, 32]> self = ?", "List[int] size = [1, 15, 15, 32]"],
        ["Tensor<[1, 15, 15, 32]> self = ?", "List[int] size = [-1, 32]"],
        ["Tensor<[4096, 32]> self = ?", "List[int] size = [64, 64, -1]"],
        ["Tensor<[1, 8, 8, 1024]> self = ?", "List[int] size = [1, 1, 8, 1, 8, 1024]"],
        ["Tensor<[1, 1, 1, 8, 8, 1024]> self = ?", "List[int] size = [1, 64, 1024]"],
        ["Tensor<[1, 64, 1024]> self = ?", "List[int] size = [64, 1024]"],
        ["Tensor<[64, 3072]> self = ?", "List[int] size = [1, 64, 3072]"],
        ["Tensor<[1, 64, 3072]> self = ?", "List[int] size = [1, 64, 3, 32, 32]"],
        ["Tensor<[1, 32, 64, 32]> self = ?", "List[int] size = [32, 64, 32]"],
        ["Tensor<[1, 32, 32, 64]> self = ?", "List[int] size = [32, 32, 64]"],
        ["Tensor<[32, 64, 64]> self = ?", "List[int] size = [1, 32, 64, 64]"],
        ["Tensor<[1, 32, 64, 64]> self = ?", "List[int] size = [32, 64, 64]"],
        ["Tensor<[32, 64, 32]> self = ?", "List[int] size = [1, 32, 64, 32]"],
        ["Tensor<[64, 1024]> self = ?", "List[int] size = [1, 64, 1024]"],
        ["Tensor<[1, 64, 1024]> self = ?", "List[int] size = [1, 1, 1, 8, 8, 1024]"],
        ["Tensor<[1, 1, 8, 1, 8, 1024]> self = ?", "List[int] size = [1, 8, 8, 1024]"],
        ["Tensor<[1, 8, 8, 1024]> self = ?", "List[int] size = [64, 1024]"],
        ["Tensor<[64, 4096]> self = ?", "List[int] size = [1, 8, 8, 4096]"],
        ["Tensor<[1, 8, 8, 4096]> self = ?", "List[int] size = [64, 4096]"],
        ["Tensor<[225, 3]> self = ?", "List[int] size = [1, 15, 15, 3]"],
        ["Tensor<[1, 15, 15, 3]> self = ?", "List[int] size = [-1, 3]"],
        ["Tensor<[4096, 3]> self = ?", "List[int] size = [64, 64, -1]"],
        ["Tensor<[1, 64, 64, 96]> self = ?", "List[int] size = [1, 8, 8, 8, 8, 96]"],
        ["Tensor<[64, 64, 96]> self = ?", "List[int] size = [4096, 96]"],
        ["Tensor<[4096, 288]> self = ?", "List[int] size = [64, 64, 288]"],
        ["Tensor<[64, 64, 288]> self = ?", "List[int] size = [64, 64, 3, 3, 32]"],
        ["Tensor<[192, 64, 64]> self = ?", "List[int] size = [64, 3, 64, 64]"],
        ["Tensor<[64, 3, 64, 64]> self = ?", "List[int] size = [192, 64, 64]"],
        ["Tensor<[192, 64, 32]> self = ?", "List[int] size = [64, 3, 64, 32]"],
        ["Tensor<[4096, 96]> self = ?", "List[int] size = [64, 64, 96]"],
        ["Tensor<[64, 64, 96]> self = ?", "List[int] size = [1, 8, 8, 8, 8, 96]"],
        ["Tensor<[1, 64, 64, 96]> self = ?", "List[int] size = [4096, 96]"],
        ["Tensor<[4096, 384]> self = ?", "List[int] size = [1, 64, 64, 384]"],
        ["Tensor<[1, 64, 64, 384]> self = ?", "List[int] size = [4096, 384]"],
        ["Tensor<[4096, 96]> self = ?", "List[int] size = [1, 64, 64, 96]"],
        ["Tensor<[64, 3, 64, 64]> self = ?", "List[int] size = [1, 64, 3, 64, 64]"],
        ["Tensor<[1, 64, 3, 64, 64]> self = ?", "List[int] size = [-1, 3, 64, 64]"],
        ["Tensor<[1, 32, 32, 384]> self = ?", "List[int] size = [1024, 384]"],
        ["Tensor<[1024, 192]> self = ?", "List[int] size = [1, 32, 32, 192]"],
        ["Tensor<[225, 6]> self = ?", "List[int] size = [1, 15, 15, 6]"],
        ["Tensor<[1, 15, 15, 6]> self = ?", "List[int] size = [-1, 6]"],
        ["Tensor<[4096, 6]> self = ?", "List[int] size = [64, 64, -1]"],
        ["Tensor<[1, 32, 32, 192]> self = ?", "List[int] size = [1, 4, 8, 4, 8, 192]"],
        ["Tensor<[16, 64, 192]> self = ?", "List[int] size = [1024, 192]"],
        ["Tensor<[1024, 576]> self = ?", "List[int] size = [16, 64, 576]"],
        ["Tensor<[16, 64, 576]> self = ?", "List[int] size = [16, 64, 3, 6, 32]"],
        ["Tensor<[96, 64, 64]> self = ?", "List[int] size = [16, 6, 64, 64]"],
        ["Tensor<[16, 6, 64, 64]> self = ?", "List[int] size = [96, 64, 64]"],
        ["Tensor<[96, 64, 32]> self = ?", "List[int] size = [16, 6, 64, 32]"],
        ["Tensor<[1024, 192]> self = ?", "List[int] size = [16, 64, 192]"],
        ["Tensor<[16, 64, 192]> self = ?", "List[int] size = [1, 4, 4, 8, 8, 192]"],
        ["Tensor<[1, 32, 32, 192]> self = ?", "List[int] size = [1024, 192]"],
        ["Tensor<[1024, 768]> self = ?", "List[int] size = [1, 32, 32, 768]"],
        ["Tensor<[1, 32, 32, 768]> self = ?", "List[int] size = [1024, 768]"],
        ["Tensor<[16, 6, 64, 64]> self = ?", "List[int] size = [1, 16, 6, 64, 64]"],
        ["Tensor<[1, 16, 6, 64, 64]> self = ?", "List[int] size = [-1, 6, 64, 64]"],
        ["Tensor<[1, 16, 16, 768]> self = ?", "List[int] size = [256, 768]"],
        ["Tensor<[256, 384]> self = ?", "List[int] size = [1, 16, 16, 384]"],
        ["Tensor<[225, 12]> self = ?", "List[int] size = [1, 15, 15, 12]"],
        ["Tensor<[1, 15, 15, 12]> self = ?", "List[int] size = [-1, 12]"],
        ["Tensor<[4096, 12]> self = ?", "List[int] size = [64, 64, -1]"],
        ["Tensor<[1, 16, 16, 384]> self = ?", "List[int] size = [1, 2, 8, 2, 8, 384]"],
        ["Tensor<[4, 64, 384]> self = ?", "List[int] size = [256, 384]"],
        ["Tensor<[256, 1152]> self = ?", "List[int] size = [4, 64, 1152]"],
        ["Tensor<[4, 64, 1152]> self = ?", "List[int] size = [4, 64, 3, 12, 32]"],
        ["Tensor<[48, 64, 64]> self = ?", "List[int] size = [4, 12, 64, 64]"],
        ["Tensor<[4, 12, 64, 64]> self = ?", "List[int] size = [48, 64, 64]"],
        ["Tensor<[48, 64, 32]> self = ?", "List[int] size = [4, 12, 64, 32]"],
        ["Tensor<[256, 384]> self = ?", "List[int] size = [4, 64, 384]"],
        ["Tensor<[4, 64, 384]> self = ?", "List[int] size = [1, 2, 2, 8, 8, 384]"],
        ["Tensor<[1, 16, 16, 384]> self = ?", "List[int] size = [256, 384]"],
        ["Tensor<[256, 1536]> self = ?", "List[int] size = [1, 16, 16, 1536]"],
        ["Tensor<[1, 16, 16, 1536]> self = ?", "List[int] size = [256, 1536]"],
        ["Tensor<[4, 12, 64, 64]> self = ?", "List[int] size = [1, 4, 12, 64, 64]"],
        ["Tensor<[1, 4, 12, 64, 64]> self = ?", "List[int] size = [-1, 12, 64, 64]"],
        ["Tensor<[1, 8, 8, 1536]> self = ?", "List[int] size = [64, 1536]"],
        ["Tensor<[64, 768]> self = ?", "List[int] size = [1, 8, 8, 768]"],
        ["Tensor<[225, 24]> self = ?", "List[int] size = [1, 15, 15, 24]"],
        ["Tensor<[1, 15, 15, 24]> self = ?", "List[int] size = [-1, 24]"],
        ["Tensor<[4096, 24]> self = ?", "List[int] size = [64, 64, -1]"],
        ["Tensor<[1, 8, 8, 768]> self = ?", "List[int] size = [1, 1, 8, 1, 8, 768]"],
        ["Tensor<[1, 1, 1, 8, 8, 768]> self = ?", "List[int] size = [1, 64, 768]"],
        ["Tensor<[1, 64, 768]> self = ?", "List[int] size = [64, 768]"],
        ["Tensor<[64, 2304]> self = ?", "List[int] size = [1, 64, 2304]"],
        ["Tensor<[1, 64, 2304]> self = ?", "List[int] size = [1, 64, 3, 24, 32]"],
        ["Tensor<[1, 24, 64, 32]> self = ?", "List[int] size = [24, 64, 32]"],
        ["Tensor<[1, 24, 32, 64]> self = ?", "List[int] size = [24, 32, 64]"],
        ["Tensor<[24, 64, 64]> self = ?", "List[int] size = [1, 24, 64, 64]"],
        ["Tensor<[1, 24, 64, 64]> self = ?", "List[int] size = [24, 64, 64]"],
        ["Tensor<[24, 64, 32]> self = ?", "List[int] size = [1, 24, 64, 32]"],
        ["Tensor<[64, 768]> self = ?", "List[int] size = [1, 64, 768]"],
        ["Tensor<[1, 64, 768]> self = ?", "List[int] size = [1, 1, 1, 8, 8, 768]"],
        ["Tensor<[1, 1, 8, 1, 8, 768]> self = ?", "List[int] size = [1, 8, 8, 768]"],
        ["Tensor<[1, 8, 8, 768]> self = ?", "List[int] size = [64, 768]"],
        ["Tensor<[64, 3072]> self = ?", "List[int] size = [1, 8, 8, 3072]"],
        ["Tensor<[1, 8, 8, 3072]> self = ?", "List[int] size = [64, 3072]"],
        ["Tensor<[1, 10]> self = ?", "List[int] size = [-1, 10]"],
        ["Tensor<[1, 10, 768]> self = ?", "List[int] size = [1, -1, 12, 64]"],
        ["Tensor<[1, 10, 12, 64]> self = ?", "List[int] size = [1, -1, 768]"],
        ["Tensor<[1, 12, 64, 1]> self = ?", "List[int] size = [12, 64, 1]"],
        ["Tensor<[12, 1, 1]> self = ?", "List[int] size = [1, 12, 1, 1]"],
        ["Tensor<[1, 12, 1, 1]> self = ?", "List[int] size = [12, 1, 1]"],
        ["Tensor<[1, 1, 12, 64]> self = ?", "List[int] size = [1, -1, 768]"],
        ["Tensor<[12, 1, 10]> self = ?", "List[int] size = [1, 12, 1, 10]"],
        ["Tensor<[1, 12, 1, 10]> self = ?", "List[int] size = [12, 1, 10]"],
        ["Tensor<[1, 12, 64, 2]> self = ?", "List[int] size = [12, 64, 2]"],
        ["Tensor<[12, 1, 2]> self = ?", "List[int] size = [1, 12, 1, 2]"],
        ["Tensor<[1, 12, 1, 2]> self = ?", "List[int] size = [12, 1, 2]"],
        ["Tensor<[1, 12, 2, 64]> self = ?", "List[int] size = [12, 2, 64]"],
        ["Tensor<[1, 12, 64, s0 + 1]> self = ?", "List[int] size = [12, 64, <s0 + 1>]"],
        ["Tensor<[12, 1, s0 + 1]> self = ?", "List[int] size = [1, 12, 1, <s0 + 1>]"],
        ["Tensor<[1, 12, 1, s0 + 1]> self = ?", "List[int] size = [12, 1, <s0 + 1>]"],
        ["Tensor<[1, 12, s0 + 1, 64]> self = ?", "List[int] size = [12, <s0 + 1>, 64]"],
        ["Tensor<[1, 10, 1024]> self = ?", "List[int] size = [10, 1024]"],
        ["Tensor<[10, 1024]> self = ?", "List[int] size = [1, 10, 1024]"],
        ["Tensor<[1, 10, 1024]> self = ?", "List[int] size = [1, -1, 16, 64]"],
        ["Tensor<[1, 16, 10, 64]> self = ?", "List[int] size = [16, 10, 64]"],
        ["Tensor<[1, 16, 64, 10]> self = ?", "List[int] size = [16, 64, 10]"],
        ["Tensor<[16, 10, 10]> self = ?", "List[int] size = [1, 16, 10, 10]"],
        ["Tensor<[1, 16, 10, 10]> self = ?", "List[int] size = [16, 10, 10]"],
        ["Tensor<[16, 10, 64]> self = ?", "List[int] size = [1, 16, 10, 64]"],
        ["Tensor<[1, 10, 16, 64]> self = ?", "List[int] size = [1, -1, 1024]"],
        ["Tensor<[10, 4096]> self = ?", "List[int] size = [1, 10, 4096]"],
        ["Tensor<[1, 10, 4096]> self = ?", "List[int] size = [10, 4096]"],
        ["Tensor<[1, 16, 64, 1]> self = ?", "List[int] size = [16, 64, 1]"],
        ["Tensor<[16, 1, 1]> self = ?", "List[int] size = [1, 16, 1, 1]"],
        ["Tensor<[1, 16, 1, 1]> self = ?", "List[int] size = [16, 1, 1]"],
        ["Tensor<[1, 1, 16, 64]> self = ?", "List[int] size = [1, -1, 1024]"],
        ["Tensor<[16, 1, 10]> self = ?", "List[int] size = [1, 16, 1, 10]"],
        ["Tensor<[1, 16, 1, 10]> self = ?", "List[int] size = [16, 1, 10]"],
        ["Tensor<[1, 16, 64, 2]> self = ?", "List[int] size = [16, 64, 2]"],
        ["Tensor<[16, 1, 2]> self = ?", "List[int] size = [1, 16, 1, 2]"],
        ["Tensor<[1, 16, 1, 2]> self = ?", "List[int] size = [16, 1, 2]"],
        ["Tensor<[1, 16, 2, 64]> self = ?", "List[int] size = [16, 2, 64]"],
        ["Tensor<[1, 16, 64, s0 + 1]> self = ?", "List[int] size = [16, 64, <s0 + 1>]"],
        ["Tensor<[16, 1, s0 + 1]> self = ?", "List[int] size = [1, 16, 1, <s0 + 1>]"],
        ["Tensor<[1, 16, 1, s0 + 1]> self = ?", "List[int] size = [16, 1, <s0 + 1>]"],
        ["Tensor<[1, 16, s0 + 1, 64]> self = ?", "List[int] size = [16, <s0 + 1>, 64]"],
        ["Tensor<[1, 10, 512]> self = ?", "List[int] size = [10, 512]"],
        ["Tensor<[10, 512]> self = ?", "List[int] size = [1, 10, 512]"],
        ["Tensor<[1, 10, 512]> self = ?", "List[int] size = [1, -1, 8, 64]"],
        ["Tensor<[1, 8, 10, 64]> self = ?", "List[int] size = [8, 10, 64]"],
        ["Tensor<[1, 8, 64, 10]> self = ?", "List[int] size = [8, 64, 10]"],
        ["Tensor<[8, 10, 10]> self = ?", "List[int] size = [1, 8, 10, 10]"],
        ["Tensor<[1, 8, 10, 10]> self = ?", "List[int] size = [8, 10, 10]"],
        ["Tensor<[8, 10, 64]> self = ?", "List[int] size = [1, 8, 10, 64]"],
        ["Tensor<[1, 10, 8, 64]> self = ?", "List[int] size = [1, -1, 512]"],
        ["Tensor<[10, 2048]> self = ?", "List[int] size = [1, 10, 2048]"],
        ["Tensor<[1, 10, 2048]> self = ?", "List[int] size = [10, 2048]"],
        ["Tensor<[1, 1, 512]> self = ?", "List[int] size = [1, -1, 8, 64]"],
        ["Tensor<[1, 8, 1, 64]> self = ?", "List[int] size = [8, 1, 64]"],
        ["Tensor<[1, 8, 64, 1]> self = ?", "List[int] size = [8, 64, 1]"],
        ["Tensor<[8, 1, 1]> self = ?", "List[int] size = [1, 8, 1, 1]"],
        ["Tensor<[1, 8, 1, 1]> self = ?", "List[int] size = [8, 1, 1]"],
        ["Tensor<[8, 1, 64]> self = ?", "List[int] size = [1, 8, 1, 64]"],
        ["Tensor<[1, 1, 8, 64]> self = ?", "List[int] size = [1, -1, 512]"],
        ["Tensor<[8, 1, 10]> self = ?", "List[int] size = [1, 8, 1, 10]"],
        ["Tensor<[1, 8, 1, 10]> self = ?", "List[int] size = [8, 1, 10]"],
        ["Tensor<[1, 2048]> self = ?", "List[int] size = [1, 1, 2048]"],
        ["Tensor<[1, 1, 2048]> self = ?", "List[int] size = [1, 2048]"],
        ["Tensor<[1, 8, 64, 2]> self = ?", "List[int] size = [8, 64, 2]"],
        ["Tensor<[8, 1, 2]> self = ?", "List[int] size = [1, 8, 1, 2]"],
        ["Tensor<[1, 8, 1, 2]> self = ?", "List[int] size = [8, 1, 2]"],
        ["Tensor<[1, 8, 2, 64]> self = ?", "List[int] size = [8, 2, 64]"],
        ["Tensor<[1, 8, 64, s0 + 1]> self = ?", "List[int] size = [8, 64, <s0 + 1>]"],
        ["Tensor<[8, 1, s0 + 1]> self = ?", "List[int] size = [1, 8, 1, <s0 + 1>]"],
        ["Tensor<[1, 8, 1, s0 + 1]> self = ?", "List[int] size = [8, 1, <s0 + 1>]"],
        ["Tensor<[1, 8, s0 + 1, 64]> self = ?", "List[int] size = [8, <s0 + 1>, 64]"],
        ["Tensor<[1, 14, 128]> self = ?", "List[int] size = [14, 128]"],
        ["Tensor<[14, 768]> self = ?", "List[int] size = [1, 14, 768]"],
        ["Tensor<[1, 14, 768]> self = ?", "List[int] size = [14, 768]"],
        ["Tensor<[1, 14, 768]> self = ?", "List[int] size = [1, 14, 12, 64]"],
        ["Tensor<[1, 12, 14, 64]> self = ?", "List[int] size = [12, 14, 64]"],
        ["Tensor<[1, 12, 64, 14]> self = ?", "List[int] size = [12, 64, 14]"],
        ["Tensor<[12, 14, 14]> self = ?", "List[int] size = [1, 12, 14, 14]"],
        ["Tensor<[1, 12, 14, 14]> self = ?", "List[int] size = [12, 14, 14]"],
        ["Tensor<[12, 14, 64]> self = ?", "List[int] size = [1, 12, 14, 64]"],
        ["Tensor<[14, 3072]> self = ?", "List[int] size = [1, 14, 3072]"],
        ["Tensor<[1, 14, 3072]> self = ?", "List[int] size = [14, 3072]"],
        ["Tensor<[14, 2]> self = ?", "List[int] size = [1, 14, 2]"],
        ["Tensor<[1, 512, 7, 7]> self = ?", "List[int] size = [1, 25088]"],
        ["Tensor<[197, 1, 768]> self = ?", "List[int] size = [197, 768]"],
        ["Tensor<[197, 2304]> self = ?", "List[int] size = [197, 1, 2304]"],
        ["Tensor<[197, 1, 2304]> self = ?", "List[int] size = [197, 1, 3, 768]"],
        ["Tensor<[197, 1, 768]> self = ?", "List[int] size = [197, 12, 64]"],
        ["Tensor<[197, 1, 12, 64]> self = ?", "List[int] size = [197, 768]"],
        ["Tensor<[197, 768]> self = ?", "List[int] size = [197, 1, 768]"],
        ["Tensor<[50, 1, 768]> self = ?", "List[int] size = [50, 768]"],
        ["Tensor<[50, 2304]> self = ?", "List[int] size = [50, 1, 2304]"],
        ["Tensor<[50, 1, 2304]> self = ?", "List[int] size = [50, 1, 3, 768]"],
        ["Tensor<[50, 1, 768]> self = ?", "List[int] size = [50, 12, 64]"],
        ["Tensor<[50, 1, 12, 64]> self = ?", "List[int] size = [50, 768]"],
        ["Tensor<[50, 768]> self = ?", "List[int] size = [50, 1, 768]"],
        ["Tensor<[1, 1280, 37, 37]> self = ?", "List[int] size = [1, 1280, 1369]"],
        ["Tensor<[1370, 1, 1280]> self = ?", "List[int] size = [1370, 1280]"],
        ["Tensor<[1370, 3840]> self = ?", "List[int] size = [1370, 1, 3840]"],
        ["Tensor<[1370, 1, 3840]> self = ?", "List[int] size = [1370, 1, 3, 1280]"],
        ["Tensor<[1370, 1, 1280]> self = ?", "List[int] size = [1370, 16, 80]"],
        ["Tensor<[16, 1370, 80]> self = ?", "List[int] size = [1, 16, 1370, 80]"],
        ["Tensor<[1370, 1, 16, 80]> self = ?", "List[int] size = [1370, 1280]"],
        ["Tensor<[1370, 1280]> self = ?", "List[int] size = [1370, 1, 1280]"],
        ["Tensor<[1, 1370, 1280]> self = ?", "List[int] size = [1370, 1280]"],
        ["Tensor<[1370, 5120]> self = ?", "List[int] size = [1, 1370, 5120]"],
        ["Tensor<[1, 1370, 5120]> self = ?", "List[int] size = [1370, 5120]"],
        ["Tensor<[1370, 1280]> self = ?", "List[int] size = [1, 1370, 1280]"],
        ["Tensor<[197, 1, 1024]> self = ?", "List[int] size = [197, 1024]"],
        ["Tensor<[197, 3072]> self = ?", "List[int] size = [197, 1, 3072]"],
        ["Tensor<[197, 1, 3072]> self = ?", "List[int] size = [197, 1, 3, 1024]"],
        ["Tensor<[197, 1, 1024]> self = ?", "List[int] size = [197, 16, 64]"],
        ["Tensor<[197, 1, 16, 64]> self = ?", "List[int] size = [197, 1024]"],
        ["Tensor<[197, 1024]> self = ?", "List[int] size = [197, 1, 1024]"],
        ["Tensor<[1, 1024, 7, 7]> self = ?", "List[int] size = [1, 1024, 49]"],
        ["Tensor<[50, 1, 1024]> self = ?", "List[int] size = [50, 1024]"],
        ["Tensor<[50, 3072]> self = ?", "List[int] size = [50, 1, 3072]"],
        ["Tensor<[50, 1, 3072]> self = ?", "List[int] size = [50, 1, 3, 1024]"],
        ["Tensor<[50, 1, 1024]> self = ?", "List[int] size = [50, 16, 64]"],
        ["Tensor<[16, 50, 64]> self = ?", "List[int] size = [1, 16, 50, 64]"],
        ["Tensor<[50, 1, 16, 64]> self = ?", "List[int] size = [50, 1024]"],
        ["Tensor<[50, 1024]> self = ?", "List[int] size = [50, 1, 1024]"],
        ["Tensor<[1, 50, 1024]> self = ?", "List[int] size = [50, 1024]"],
        ["Tensor<[50, 4096]> self = ?", "List[int] size = [1, 50, 4096]"],
        ["Tensor<[1, 50, 4096]> self = ?", "List[int] size = [50, 4096]"],
        ["Tensor<[50, 1024]> self = ?", "List[int] size = [1, 50, 1024]"],
    ],
)
def test_aten(device, input_strings, input_var_only_native, input_var_check_accu, input_var_check_ttnn):
    metric = {
        "opname": "aten.view.default",
        "input_strings": input_strings,
        "native_run": "N/A",
        "run": "N/A",
        "accuracy": "N/A",
        "convert_to_ttnn": "N/A",
        "ttnn_fallbacks_to_host_count": "N/A",
    }
    m = AtenModule()
    input_args, input_kwargs, status = render_metric_string_list_to_input_args_kwargs(
        "aten.view.default", input_strings
    )
    if status == False:
        pytest.skip("Invalid input strings")
    try:
        result_before = m.forward(*input_args, **input_kwargs)
        metric["native_run"] = True
    except Exception as e:
        print(f"Failed to run native. Raised exception: {e}")
        metric["native_run"] = False

    if metric["native_run"] == True:
        result_after = None
        option = torch_ttnn.TorchTtnnOption(device=device)
        # option.gen_graphviz = True
        # The compilation is lazy, so we need to run forward once to trigger the compilation
        m = torch.compile(m, backend=torch_ttnn.backend, options=option)
        try:
            ttnn.graph.begin_graph_capture()
            result_after = m.forward(*input_args, **input_kwargs)
            # option._out_fx_graphs[0].print_tabular()
            metric["run"] = True
        except Exception as e:
            print(f"Failed to run. Raised exception: {e}")
            metric["run"] = False
        finally:
            trace = ttnn.graph.end_graph_capture()
            call_stack = ttnn.graph.extract_calltrace(trace)
            if metric["run"] == True:
                print(call_stack)
                expected_to_host_count = 0
                if result_after is None:
                    expected_to_host_count = 0
                elif isinstance(result_after, torch.Tensor):
                    expected_to_host_count = 1
                elif isinstance(result_after, (list, dict)):
                    expected_to_host_count = len(result_after)
                else:
                    print(f"Unexpected result_after type: {type(result_after)}")

                to_host_count = sum(["Tensor::cpu" in str(node) for node in call_stack])
                fallbacks_to_host_count = to_host_count - expected_to_host_count
                print(f"expected_to_host_count: {expected_to_host_count}")
                print(f"to_host_count: {to_host_count}")
                print(f"fallbacks_to_host_count: {fallbacks_to_host_count}")
                metric["ttnn_fallbacks_to_host_count"] = fallbacks_to_host_count

    if metric["run"] == True:
        try:
            # Check inference result
            metric["accuracy"] = calculate_accuracy(result_before, result_after)
        except Exception as e:
            print(f"Failed to check inference result. Raised exception: {e}")

        try:
            # Check the graph has be rewritten and contain ttnn ops
            nodes = list(option._out_fx_graphs[0].nodes)
            if not any(["aten." in str(node.target) for node in nodes]):
                metric["convert_to_ttnn"] = True
            else:
                metric["convert_to_ttnn"] = False
        except Exception as e:
            print(f"Failed to check the graph has ttnn op. Raised exception: {e}")

    metrics.append(metric)

    if not input_var_only_native:
        assert metric["run"] == True
        if input_var_check_accu:
            assert metric["accuracy"] >= 0.99
        if input_var_check_ttnn:
            assert metric["convert_to_ttnn"] == True
