# SPDX-FileCopyrightText: © 2025 Tenstorrent AI ULC
#
# SPDX-License-Identifier: Apache-2.0
import torch
import torch_ttnn
import pytest
import pickle
import ttnn
from pathlib import Path
from tests.utils import calculate_accuracy, render_metric_string_list_to_input_args_kwargs


class AtenModule(torch.nn.Module):
    def __init__(self):
        super().__init__()

    def forward(self, *args, **kwargs):
        return torch.ops.aten.convolution_backward.default(*args, **kwargs)


metrics = []


def save_pickle(obj, base_path, filename):
    p = Path(base_path)
    p.mkdir(parents=True, exist_ok=True)
    pickle_out_path = p / f"{filename}.pickle"
    with open(pickle_out_path, "wb") as f:
        pickle.dump(obj, f)


def teardown_module(module):
    print(metrics)
    save_pickle(metrics, "metrics-autogen-op/ghostnetv2_100.in1k-train", "aten.convolution_backward.default")


@pytest.mark.parametrize(
    "input_strings",
    [
        [
            "Tensor<[1, 1280, 1, 1]> grad_output = ?",
            "Tensor<[1, 960, 1, 1]> input = ?",
            "Tensor<[1280, 960, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [1280]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 960, 7, 7]> grad_output = ?",
            "Tensor<[1, 160, 7, 7]> input = ?",
            "Tensor<[960, 160, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 7, 7]> grad_output = ?",
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Tensor<[80, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 80",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 7, 7]> grad_output = ?",
            "Tensor<[1, 960, 7, 7]> input = ?",
            "Tensor<[80, 960, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 1, 1]> grad_output = ?",
            "Tensor<[1, 240, 1, 1]> input = ?",
            "Tensor<[960, 240, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [960]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 240, 1, 1]> grad_output = ?",
            "Tensor<[1, 960, 1, 1]> input = ?",
            "Tensor<[240, 960, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [240]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 480, 7, 7]> input = ?",
            "Tensor<[480, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 160, 7, 7]> input = ?",
            "Tensor<[480, 160, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 3, 3]> grad_output = ?",
            "Tensor<[1, 960, 3, 3]> input = ?",
            "Tensor<[960, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 960",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 3, 3]> grad_output = ?",
            "Tensor<[1, 960, 3, 3]> input = ?",
            "Tensor<[960, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 960",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 3, 3]> grad_output = ?",
            "Tensor<[1, 160, 3, 3]> input = ?",
            "Tensor<[960, 160, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 160, 7, 7]> grad_output = ?",
            "Tensor<[1, 112, 7, 7]> input = ?",
            "Tensor<[160, 112, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 7, 7]> grad_output = ?",
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Tensor<[112, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 112",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Tensor<[80, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 1, 1]> grad_output = ?",
            "Tensor<[1, 168, 1, 1]> input = ?",
            "Tensor<[672, 168, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [672]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 168, 1, 1]> grad_output = ?",
            "Tensor<[1, 672, 1, 1]> input = ?",
            "Tensor<[168, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [168]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Tensor<[672, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 336, 14, 14]> grad_output = ?",
            "Tensor<[1, 336, 14, 14]> input = ?",
            "Tensor<[336, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 336",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 336, 14, 14]> grad_output = ?",
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Tensor<[336, 112, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Tensor<[672, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Tensor<[672, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_output = ?",
            "Tensor<[1, 112, 7, 7]> input = ?",
            "Tensor<[672, 112, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 56, 14, 14]> grad_output = ?",
            "Tensor<[1, 56, 14, 14]> input = ?",
            "Tensor<[56, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 56",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 56, 14, 14]> grad_output = ?",
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Tensor<[56, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[112, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[80, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 80",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 56, 14, 14]> grad_output = ?",
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Tensor<[56, 480, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 1, 1]> grad_output = ?",
            "Tensor<[1, 120, 1, 1]> input = ?",
            "Tensor<[480, 120, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [480]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 120, 1, 1]> grad_output = ?",
            "Tensor<[1, 480, 1, 1]> input = ?",
            "Tensor<[120, 480, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [120]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Tensor<[240, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[240, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 480, 7, 7]> input = ?",
            "Tensor<[480, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 480, 7, 7]> input = ?",
            "Tensor<[480, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Tensor<[480, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Tensor<[40, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 40",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 184, 14, 14]> input = ?",
            "Tensor<[40, 184, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 92, 14, 14]> grad_output = ?",
            "Tensor<[1, 92, 14, 14]> input = ?",
            "Tensor<[92, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 92",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 92, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[92, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 184, 7, 7]> grad_output = ?",
            "Tensor<[1, 184, 7, 7]> input = ?",
            "Tensor<[184, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 184",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 184, 7, 7]> grad_output = ?",
            "Tensor<[1, 184, 7, 7]> input = ?",
            "Tensor<[184, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 184",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 184, 7, 7]> grad_output = ?",
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Tensor<[184, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 200, 14, 14]> input = ?",
            "Tensor<[40, 200, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 100, 14, 14]> grad_output = ?",
            "Tensor<[1, 100, 14, 14]> input = ?",
            "Tensor<[100, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 100",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 100, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[100, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 200, 7, 7]> grad_output = ?",
            "Tensor<[1, 200, 7, 7]> input = ?",
            "Tensor<[200, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 200",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 200, 7, 7]> grad_output = ?",
            "Tensor<[1, 200, 7, 7]> input = ?",
            "Tensor<[200, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 200",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 200, 7, 7]> grad_output = ?",
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Tensor<[200, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Tensor<[80, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Tensor<[40, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 40",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Tensor<[40, 240, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 28, 28]> input = ?",
            "Tensor<[240, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 28, 28]> grad_output = ?",
            "Tensor<[1, 120, 28, 28]> input = ?",
            "Tensor<[120, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 120",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 28, 28]> grad_output = ?",
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Tensor<[120, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Tensor<[240, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Tensor<[240, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Tensor<[240, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 20, 28, 28]> input = ?",
            "Tensor<[20, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 20",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 120, 28, 28]> input = ?",
            "Tensor<[20, 120, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 1, 1]> grad_output = ?",
            "Tensor<[1, 32, 1, 1]> input = ?",
            "Tensor<[120, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [120]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 1, 1]> grad_output = ?",
            "Tensor<[1, 120, 1, 1]> input = ?",
            "Tensor<[32, 120, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [32]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 60, 28, 28]> grad_output = ?",
            "Tensor<[1, 60, 28, 28]> input = ?",
            "Tensor<[60, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 60",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 60, 28, 28]> grad_output = ?",
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Tensor<[60, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 14, 14]> grad_output = ?",
            "Tensor<[1, 120, 14, 14]> input = ?",
            "Tensor<[120, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 120",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 14, 14]> grad_output = ?",
            "Tensor<[1, 120, 14, 14]> input = ?",
            "Tensor<[120, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 120",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Tensor<[120, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 28, 28]> grad_output = ?",
            "Tensor<[1, 24, 28, 28]> input = ?",
            "Tensor<[40, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 28, 28]> grad_output = ?",
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Tensor<[24, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 24",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 72, 28, 28]> input = ?",
            "Tensor<[20, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 1, 1]> grad_output = ?",
            "Tensor<[1, 20, 1, 1]> input = ?",
            "Tensor<[72, 20, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [72]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 20, 1, 1]> grad_output = ?",
            "Tensor<[1, 72, 1, 1]> input = ?",
            "Tensor<[20, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [20]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 72, 28, 28]> grad_output = ?",
            "Tensor<[1, 72, 56, 56]> input = ?",
            "Tensor<[72, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 72",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 56, 56]> grad_output = ?",
            "Tensor<[1, 36, 56, 56]> input = ?",
            "Tensor<[36, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 36",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 56, 56]> grad_output = ?",
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Tensor<[36, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 28, 28]> grad_output = ?",
            "Tensor<[1, 72, 28, 28]> input = ?",
            "Tensor<[72, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 72",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 28, 28]> grad_output = ?",
            "Tensor<[1, 72, 28, 28]> input = ?",
            "Tensor<[72, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 72",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 28, 28]> grad_output = ?",
            "Tensor<[1, 24, 28, 28]> input = ?",
            "Tensor<[72, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 12, 56, 56]> grad_output = ?",
            "Tensor<[1, 12, 56, 56]> input = ?",
            "Tensor<[12, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 12",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 12, 56, 56]> grad_output = ?",
            "Tensor<[1, 72, 56, 56]> input = ?",
            "Tensor<[12, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 56, 56]> grad_output = ?",
            "Tensor<[1, 16, 56, 56]> input = ?",
            "Tensor<[24, 16, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 56, 56]> grad_output = ?",
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Tensor<[16, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 16",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 12, 56, 56]> grad_output = ?",
            "Tensor<[1, 48, 56, 56]> input = ?",
            "Tensor<[12, 48, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 48, 56, 56]> grad_output = ?",
            "Tensor<[1, 48, 112, 112]> input = ?",
            "Tensor<[48, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 48",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 112, 112]> grad_output = ?",
            "Tensor<[1, 24, 112, 112]> input = ?",
            "Tensor<[24, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 24",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 112, 112]> grad_output = ?",
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Tensor<[24, 16, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 8, 112, 112]> grad_output = ?",
            "Tensor<[1, 8, 112, 112]> input = ?",
            "Tensor<[8, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 8",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 8, 112, 112]> grad_output = ?",
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Tensor<[8, 16, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 112, 112]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[16, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
    ],
)
def test_aten(device, input_strings, input_var_only_native, input_var_check_accu, input_var_check_ttnn):
    metric = {
        "opname": "aten.convolution_backward.default",
        "input_strings": input_strings,
        "native_run": "N/A",
        "run": "N/A",
        "accuracy": "N/A",
        "convert_to_ttnn": "N/A",
        "ttnn_fallbacks_to_host_count": "N/A",
    }
    m = AtenModule()
    input_args, input_kwargs, status = render_metric_string_list_to_input_args_kwargs(
        "aten.convolution_backward.default", input_strings
    )
    if status == False:
        pytest.skip("Invalid input strings")
    try:
        result_before = m.forward(*input_args, **input_kwargs)
        metric["native_run"] = True
    except Exception as e:
        print(f"Failed to run native. Raised exception: {e}")
        metric["native_run"] = False

    if metric["native_run"] == True:
        result_after = None
        option = torch_ttnn.TorchTtnnOption(device=device)
        # option.gen_graphviz = True
        # The compilation is lazy, so we need to run forward once to trigger the compilation
        m = torch.compile(m, backend=torch_ttnn.backend, options=option)
        try:
            ttnn.graph.begin_graph_capture()
            result_after = m.forward(*input_args, **input_kwargs)
            # option._out_fx_graphs[0].print_tabular()
            metric["run"] = True
        except Exception as e:
            print(f"Failed to run. Raised exception: {e}")
            metric["run"] = False
        finally:
            trace = ttnn.graph.end_graph_capture()
            call_stack = ttnn.graph.extract_calltrace(trace)
            if metric["run"] == True:
                print(call_stack)
                expected_to_host_count = 0
                if result_after is None:
                    expected_to_host_count = 0
                elif isinstance(result_after, torch.Tensor):
                    expected_to_host_count = 1
                elif isinstance(result_after, (list, dict)):
                    expected_to_host_count = len(result_after)
                else:
                    print(f"Unexpected result_after type: {type(result_after)}")

                to_host_count = sum(["Tensor::cpu" in str(node) for node in call_stack])
                fallbacks_to_host_count = to_host_count - expected_to_host_count
                print(f"expected_to_host_count: {expected_to_host_count}")
                print(f"to_host_count: {to_host_count}")
                print(f"fallbacks_to_host_count: {fallbacks_to_host_count}")
                metric["ttnn_fallbacks_to_host_count"] = fallbacks_to_host_count

    if metric["run"] == True:
        try:
            # Check inference result
            metric["accuracy"] = calculate_accuracy(result_before, result_after)
        except Exception as e:
            print(f"Failed to check inference result. Raised exception: {e}")

        try:
            # Check the graph has be rewritten and contain ttnn ops
            nodes = list(option._out_fx_graphs[0].nodes)
            if not any(["aten." in str(node.target) for node in nodes]):
                metric["convert_to_ttnn"] = True
            else:
                metric["convert_to_ttnn"] = False
        except Exception as e:
            print(f"Failed to check the graph has ttnn op. Raised exception: {e}")

    metrics.append(metric)

    if not input_var_only_native:
        assert metric["run"] == True
        if input_var_check_accu:
            assert metric["accuracy"] >= 0.99
        if input_var_check_ttnn:
            assert metric["convert_to_ttnn"] == True
