# SPDX-FileCopyrightText: © 2025 Tenstorrent AI ULC
#
# SPDX-License-Identifier: Apache-2.0
import torch
import torch_ttnn
import pytest
import pickle
import ttnn
from pathlib import Path
from tests.utils import calculate_accuracy, render_metric_string_list_to_input_args_kwargs


class AtenModule(torch.nn.Module):
    def __init__(self):
        super().__init__()

    def forward(self, *args, **kwargs):
        return torch.ops.aten.cat.default(*args, **kwargs)


metrics = []


def save_pickle(obj, base_path, filename):
    p = Path(base_path)
    p.mkdir(parents=True, exist_ok=True)
    pickle_out_path = p / f"{filename}.pickle"
    with open(pickle_out_path, "wb") as f:
        pickle.dump(obj, f)


def teardown_module(module):
    print(metrics)
    save_pickle(metrics, "metrics-autogen-op/ALL", "aten.cat.default")


@pytest.mark.parametrize(
    "input_strings",
    [
        ["List[Tensor] tensors = [<[1, 1, 768]>, <[1, 49, 768]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 23, 40, 128]>, <[1, 23, 40, 128]>]", "int dim = 3"],
        ["List[Tensor] tensors = [_folded_ones_1, <[1, 1, 1]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 6, 1, 64]>, <[1, 6, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 1, s0]>, <[1, 1, 1]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 6, s0, 64]>, <[1, 6, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [_folded_ones, <[1, 1, 1]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 6, 16, 64]>, <[1, 6, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 64, 30, 40]>, <[1, 64, 30, 40]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 64, 60, 80]>, <[1, 64, 60, 80]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 64, 120, 160]>, <[1, 64, 120, 160]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 45]>, <[1, 1]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 12, 45, 64]>, <[1, 12, 1, 64]>]", "int dim = -2"],
        ["List[Tensor] tensors = [<[1, s24]>, <[1, 1]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 12, s10, 64]>, <[1, 12, 1, 64]>]", "int dim = -2"],
        ["List[Tensor] tensors = [<[1, 14, 56, 56]>, <[1, 64, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 14, 56, 56]>, <[1, 24, 56, 56]>, <[1, 64, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 14, 56, 56]>, <[1, 40, 56, 56]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 14, 56, 56]>, <[1, 24, 56, 56]>, <[1, 40, 56, 56]>, <[1, 64, 56, 56]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 14, 56, 56]>, <[1, 14, 56, 56]>, <[1, 14, 56, 56]>, <[1, 14, 56, 56]>, <[1, 68, 56, 56]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 16, 28, 28]>, <[1, 128, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 16, 28, 28]>, <[1, 28, 28, 28]>, <[1, 128, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 16, 28, 28]>, <[1, 46, 28, 28]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 16, 28, 28]>, <[1, 28, 28, 28]>, <[1, 46, 28, 28]>, <[1, 128, 28, 28]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 16, 28, 28]>, <[1, 78, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 16, 28, 28]>, <[1, 28, 28, 28]>, <[1, 78, 28, 28]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 16, 28, 28]>, <[1, 28, 28, 28]>, <[1, 46, 28, 28]>, <[1, 78, 28, 28]>, <[1, 128, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 16, 28, 28]>, <[1, 16, 28, 28]>, <[1, 16, 28, 28]>, <[1, 16, 28, 28]>, <[1, 16, 28, 28]>, <[1, 16, 28, 28]>, <[1, 16, 28, 28]>, <[1, 16, 28, 28]>, <[1, 134, 28, 28]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 20, 28, 28]>, <[1, 256, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 20, 28, 28]>, <[1, 34, 28, 28]>, <[1, 256, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 20, 28, 28]>, <[1, 58, 28, 28]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 20, 28, 28]>, <[1, 34, 28, 28]>, <[1, 58, 28, 28]>, <[1, 256, 28, 28]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 20, 28, 28]>, <[1, 98, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 20, 28, 28]>, <[1, 34, 28, 28]>, <[1, 98, 28, 28]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 20, 28, 28]>, <[1, 34, 28, 28]>, <[1, 58, 28, 28]>, <[1, 98, 28, 28]>, <[1, 256, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 20, 28, 28]>, <[1, 20, 28, 28]>, <[1, 20, 28, 28]>, <[1, 20, 28, 28]>, <[1, 20, 28, 28]>, <[1, 20, 28, 28]>, <[1, 20, 28, 28]>, <[1, 20, 28, 28]>, <[1, 168, 28, 28]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 40, 14, 14]>, <[1, 320, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 40, 14, 14]>, <[1, 68, 14, 14]>, <[1, 320, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 40, 14, 14]>, <[1, 116, 14, 14]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 40, 14, 14]>, <[1, 68, 14, 14]>, <[1, 116, 14, 14]>, <[1, 320, 14, 14]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 40, 14, 14]>, <[1, 196, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 40, 14, 14]>, <[1, 68, 14, 14]>, <[1, 196, 14, 14]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 40, 14, 14]>, <[1, 68, 14, 14]>, <[1, 116, 14, 14]>, <[1, 196, 14, 14]>, <[1, 320, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 40, 14, 14]>, <[1, 40, 14, 14]>, <[1, 40, 14, 14]>, <[1, 40, 14, 14]>, <[1, 40, 14, 14]>, <[1, 40, 14, 14]>, <[1, 40, 14, 14]>, <[1, 40, 14, 14]>, <[1, 334, 14, 14]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 160, 7, 7]>, <[1, 640, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 160, 7, 7]>, <[1, 272, 7, 7]>, <[1, 640, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 160, 7, 7]>, <[1, 160, 7, 7]>, <[1, 462, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 32, 64]>, <[1, 32, 64]>]", "int dim = -1"],
        [
            "List[Tensor] tensors = [<[1, 2400, 4]>, <[1, 600, 4]>, <[1, 150, 4]>, <[1, 54, 4]>, <[1, 24, 4]>, <[1, 6, 4]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 2400, 91]>, <[1, 600, 91]>, <[1, 150, 91]>, <[1, 54, 91]>, <[1, 24, 91]>, <[1, 6, 91]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[2400, 2]>, <[2400, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[600, 2]>, <[600, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[150, 2]>, <[150, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[54, 2]>, <[54, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[24, 2]>, <[24, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[6, 2]>, <[6, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[2400, 4]>, <[600, 4]>, <[150, 4]>, <[54, 4]>, <[24, 4]>, <[6, 4]>]"],
        ["List[Tensor] tensors = [<[3234, 2]>, <[3234, 2]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 59]>, <[1, 1]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 16, 59, 64]>, <[1, 16, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, s48]>, <[1, 1]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 16, s10, 64]>, <[1, 16, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 19, 28, 28]>, <[1, 38, 28, 28]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 256, 128, 128]>, <[1, 256, 128, 128]>, <[1, 256, 128, 128]>, <[1, 256, 128, 128]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 160]>, <[1, 160]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 256, 32, 32]>, <[1, 256, 32, 32]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 128, 64, 64]>, <[1, 128, 64, 64]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 64, 128, 128]>, <[1, 64, 128, 128]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 32, 256, 256]>, <[1, 32, 256, 256]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 512, 28, 28]>, <[1, 512, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 256, 56, 56]>, <[1, 256, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 128, 112, 112]>, <[1, 128, 112, 112]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 64, 224, 224]>, <[1, 64, 224, 224]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 768, 12, 16]>]"],
        ["List[Tensor] tensors = [_folded_mul, _folded_mul_1, _folded_mul_2, _folded_mul_3]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 12, 4, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s2, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s3, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s4, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s5, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s6, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s7, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s8, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s9, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s10, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s11, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s12, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s13, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s14, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s15, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s16, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s17, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s18, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s19, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s20, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s21, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s22, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s23, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s24, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s25, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 1, 192]>, <[1, 1344, 192]>, <[1, 100, 192]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 256, 32, 32]>, <[1, 512, 32, 32]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 128, 64, 64]>, <[1, 256, 64, 64]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 5, 16, 32]>, <[1, 5, 16, 32]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 5]>, <[1, 1]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 1, 16, 32]>, <[1, 1, 16, 32]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 16, 5, 64]>, <[1, 16, 1, 64]>]", "int dim = -2"],
        ["List[Tensor] tensors = [<[1, s40]>, <[1, 1]>]", "int dim = -1"],
        ["List[Tensor] tensors = [<[1, 16, s10, 64]>, <[1, 16, 1, 64]>]", "int dim = -2"],
        ["List[Tensor] tensors = [<[1, 64, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 64, 56, 56]>, <[1, 32, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 64, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 64, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 64, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 64, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 64, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>, <[1, 32, 56, 56]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 128, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 256, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 512, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 96, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 96, 56, 56]>, <[1, 48, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 96, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 96, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 96, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 96, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 96, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>, <[1, 48, 56, 56]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 192, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>, <[1, 48, 28, 28]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 384, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>, <[1, 48, 14, 14]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 1056, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 1056, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>, <[1, 48, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 640, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 640, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>, <[1, 32, 14, 14]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 896, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 896, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>, <[1, 32, 7, 7]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 64, 56, 56]>, <[1, 64, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 128, 28, 28]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 128, 28, 28]>, <[1, 64, 28, 28]>, <[1, 128, 28, 28]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 256, 14, 14]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 256, 14, 14]>, <[1, 128, 14, 14]>, <[1, 256, 14, 14]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 512, 7, 7]>, <[1, 512, 7, 7]>, <[1, 256, 7, 7]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 64, 56, 56]>, <[1, 128, 56, 56]>, <[1, 128, 56, 56]>, <[1, 128, 56, 56]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 28, 28]>, <[1, 160, 28, 28]>, <[1, 160, 28, 28]>, <[1, 160, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 512, 14, 14]>, <[1, 192, 14, 14]>, <[1, 192, 14, 14]>, <[1, 192, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 768, 7, 7]>, <[1, 224, 7, 7]>, <[1, 224, 7, 7]>, <[1, 224, 7, 7]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 1, 768]>, <[1, 196, 768]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 8, 112, 112]>, <[1, 8, 112, 112]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 24, 112, 112]>, <[1, 24, 112, 112]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 12, 56, 56]>, <[1, 12, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 36, 56, 56]>, <[1, 36, 56, 56]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 20, 28, 28]>, <[1, 20, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 60, 28, 28]>, <[1, 60, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 120, 28, 28]>, <[1, 120, 28, 28]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 40, 14, 14]>, <[1, 40, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 100, 14, 14]>, <[1, 100, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 92, 14, 14]>, <[1, 92, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 240, 14, 14]>, <[1, 240, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 56, 14, 14]>, <[1, 56, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 336, 14, 14]>, <[1, 336, 14, 14]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 80, 7, 7]>, <[1, 80, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 480, 7, 7]>, <[1, 480, 7, 7]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 1, 224, 224]>, <[1, 1, 224, 224]>, <[1, 1, 224, 224]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 64, 28, 28]>, <[1, 128, 28, 28]>, <[1, 32, 28, 28]>, <[1, 32, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 28, 28]>, <[1, 192, 28, 28]>, <[1, 96, 28, 28]>, <[1, 64, 28, 28]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 192, 14, 14]>, <[1, 208, 14, 14]>, <[1, 48, 14, 14]>, <[1, 64, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 160, 14, 14]>, <[1, 224, 14, 14]>, <[1, 64, 14, 14]>, <[1, 64, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 128, 14, 14]>, <[1, 256, 14, 14]>, <[1, 64, 14, 14]>, <[1, 64, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 112, 14, 14]>, <[1, 288, 14, 14]>, <[1, 64, 14, 14]>, <[1, 64, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 14, 14]>, <[1, 320, 14, 14]>, <[1, 128, 14, 14]>, <[1, 128, 14, 14]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 256, 7, 7]>, <[1, 320, 7, 7]>, <[1, 128, 7, 7]>, <[1, 128, 7, 7]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 384, 7, 7]>, <[1, 384, 7, 7]>, <[1, 128, 7, 7]>, <[1, 128, 7, 7]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 64, 73, 73]>, <[1, 96, 73, 73]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 96, 71, 71]>, <[1, 96, 71, 71]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 192, 35, 35]>, <[1, 192, 35, 35]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 96, 35, 35]>, <[1, 96, 35, 35]>, <[1, 96, 35, 35]>, <[1, 96, 35, 35]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 384, 17, 17]>, <[1, 256, 17, 17]>, <[1, 384, 17, 17]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 384, 17, 17]>, <[1, 256, 17, 17]>, <[1, 256, 17, 17]>, <[1, 128, 17, 17]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 192, 8, 8]>, <[1, 320, 8, 8]>, <[1, 1024, 8, 8]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 256, 8, 8]>, <[1, 256, 8, 8]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 256, 8, 8]>, <[1, 512, 8, 8]>, <[1, 512, 8, 8]>, <[1, 256, 8, 8]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[1, 1, 1024]>, <[1, 196, 1024]>]", "int dim = 1"],
        [
            "List[Tensor] tensors = [<[1, 122400, 91]>, <[1, 30600, 91]>, <[1, 7650, 91]>, <[1, 1989, 91]>, <[1, 567, 91]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 122400, 4]>, <[1, 30600, 4]>, <[1, 7650, 4]>, <[1, 1989, 4]>, <[1, 567, 4]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[122400, 4]>, <[30600, 4]>, <[7650, 4]>, <[1989, 4]>, <[567, 4]>]"],
        [
            "List[Tensor] tensors = [<[1, 5776, 4]>, <[1, 2166, 4]>, <[1, 600, 4]>, <[1, 150, 4]>, <[1, 36, 4]>, <[1, 4, 4]>]",
            "int dim = 1",
        ],
        [
            "List[Tensor] tensors = [<[1, 5776, 91]>, <[1, 2166, 91]>, <[1, 600, 91]>, <[1, 150, 91]>, <[1, 36, 91]>, <[1, 4, 91]>]",
            "int dim = 1",
        ],
        ["List[Tensor] tensors = [<[5776, 2]>, <[5776, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[2166, 2]>, <[2166, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[36, 2]>, <[36, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[4, 2]>, <[4, 2]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[5776, 4]>, <[2166, 4]>, <[600, 4]>, <[150, 4]>, <[36, 4]>, <[4, 4]>]"],
        ["List[Tensor] tensors = [<[8732, 2]>, <[8732, 2]>]", "int dim = -1"],
        [
            "List[Tensor] tensors = [<[1, 28, 28, 128]>, <[1, 28, 28, 128]>, <[1, 28, 28, 128]>, <[1, 28, 28, 128]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 14, 14, 256]>, <[1, 14, 14, 256]>, <[1, 14, 14, 256]>, <[1, 14, 14, 256]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 7, 7, 512]>, <[1, 7, 7, 512]>, <[1, 7, 7, 512]>, <[1, 7, 7, 512]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 28, 28, 96]>, <[1, 28, 28, 96]>, <[1, 28, 28, 96]>, <[1, 28, 28, 96]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 14, 14, 192]>, <[1, 14, 14, 192]>, <[1, 14, 14, 192]>, <[1, 14, 14, 192]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 7, 7, 384]>, <[1, 7, 7, 384]>, <[1, 7, 7, 384]>, <[1, 7, 7, 384]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 32, 32, 128]>, <[1, 32, 32, 128]>, <[1, 32, 32, 128]>, <[1, 32, 32, 128]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 16, 16, 256]>, <[1, 16, 16, 256]>, <[1, 16, 16, 256]>, <[1, 16, 16, 256]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 8, 8, 512]>, <[1, 8, 8, 512]>, <[1, 8, 8, 512]>, <[1, 8, 8, 512]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 32, 32, 96]>, <[1, 32, 32, 96]>, <[1, 32, 32, 96]>, <[1, 32, 32, 96]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 16, 16, 192]>, <[1, 16, 16, 192]>, <[1, 16, 16, 192]>, <[1, 16, 16, 192]>]",
            "int dim = -1",
        ],
        [
            "List[Tensor] tensors = [<[1, 8, 8, 384]>, <[1, 8, 8, 384]>, <[1, 8, 8, 384]>, <[1, 8, 8, 384]>]",
            "int dim = -1",
        ],
        ["List[Tensor] tensors = [<[1, 12, 1, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 12, s0, 64]>, <[1, 12, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 16, 1, 64]>, <[1, 16, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 16, s0, 64]>, <[1, 16, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 8, 1, 64]>, <[1, 8, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 8, s0, 64]>, <[1, 8, 1, 64]>]", "int dim = 2"],
        ["List[Tensor] tensors = [<[1, 1, 1280]>, <[1, 1369, 1280]>]", "int dim = 1"],
        ["List[Tensor] tensors = [<[1, 1, 1024]>, <[1, 49, 1024]>]", "int dim = 1"],
    ],
)
def test_aten(device, input_strings, input_var_only_native, input_var_check_accu, input_var_check_ttnn):
    metric = {
        "opname": "aten.cat.default",
        "input_strings": input_strings,
        "native_run": "N/A",
        "run": "N/A",
        "accuracy": "N/A",
        "convert_to_ttnn": "N/A",
        "ttnn_fallbacks_to_host_count": "N/A",
    }
    m = AtenModule()
    input_args, input_kwargs, status = render_metric_string_list_to_input_args_kwargs("aten.cat.default", input_strings)
    if status == False:
        pytest.skip("Invalid input strings")
    try:
        result_before = m.forward(*input_args, **input_kwargs)
        metric["native_run"] = True
    except Exception as e:
        print(f"Failed to run native. Raised exception: {e}")
        metric["native_run"] = False

    if metric["native_run"] == True:
        result_after = None
        option = torch_ttnn.TorchTtnnOption(device=device)
        # option.gen_graphviz = True
        # The compilation is lazy, so we need to run forward once to trigger the compilation
        m = torch.compile(m, backend=torch_ttnn.backend, options=option)
        try:
            ttnn.graph.begin_graph_capture()
            result_after = m.forward(*input_args, **input_kwargs)
            # option._out_fx_graphs[0].print_tabular()
            metric["run"] = True
        except Exception as e:
            print(f"Failed to run. Raised exception: {e}")
            metric["run"] = False
        finally:
            trace = ttnn.graph.end_graph_capture()
            call_stack = ttnn.graph.extract_calltrace(trace)
            if metric["run"] == True:
                print(call_stack)
                expected_to_host_count = 0
                if result_after is None:
                    expected_to_host_count = 0
                elif isinstance(result_after, torch.Tensor):
                    expected_to_host_count = 1
                elif isinstance(result_after, (list, dict)):
                    expected_to_host_count = len(result_after)
                else:
                    print(f"Unexpected result_after type: {type(result_after)}")

                to_host_count = sum(["Tensor::cpu" in str(node) for node in call_stack])
                fallbacks_to_host_count = to_host_count - expected_to_host_count
                print(f"expected_to_host_count: {expected_to_host_count}")
                print(f"to_host_count: {to_host_count}")
                print(f"fallbacks_to_host_count: {fallbacks_to_host_count}")
                metric["ttnn_fallbacks_to_host_count"] = fallbacks_to_host_count

    if metric["run"] == True:
        try:
            # Check inference result
            metric["accuracy"] = calculate_accuracy(result_before, result_after)
        except Exception as e:
            print(f"Failed to check inference result. Raised exception: {e}")

        try:
            # Check the graph has be rewritten and contain ttnn ops
            nodes = list(option._out_fx_graphs[0].nodes)
            if not any(["aten." in str(node.target) for node in nodes]):
                metric["convert_to_ttnn"] = True
            else:
                metric["convert_to_ttnn"] = False
        except Exception as e:
            print(f"Failed to check the graph has ttnn op. Raised exception: {e}")

    metrics.append(metric)

    if not input_var_only_native:
        assert metric["run"] == True
        if input_var_check_accu:
            assert metric["accuracy"] >= 0.99
        if input_var_check_ttnn:
            assert metric["convert_to_ttnn"] == True
