# SPDX-FileCopyrightText: © 2025 Tenstorrent AI ULC
#
# SPDX-License-Identifier: Apache-2.0
import torch
import torch_ttnn
import pytest
import pickle
import ttnn
from pathlib import Path
from tests.utils import calculate_accuracy, render_metric_string_list_to_input_args_kwargs


class AtenModule(torch.nn.Module):
    def __init__(self):
        super().__init__()

    def forward(self, *args, **kwargs):
        return torch.ops.aten.convolution_backward.default(*args, **kwargs)


metrics = []


def save_pickle(obj, base_path, filename):
    p = Path(base_path)
    p.mkdir(parents=True, exist_ok=True)
    pickle_out_path = p / f"{filename}.pickle"
    with open(pickle_out_path, "wb") as f:
        pickle.dump(obj, f)


def teardown_module(module):
    print(metrics)
    save_pickle(metrics, "metrics-autogen-op/ALL", "aten.convolution_backward.default")


@pytest.mark.parametrize(
    "input_strings",
    [
        [
            "Tensor<[1, 1, 28, 28]> grad_output = ?",
            "Tensor<[1, 16, 14, 14]> input = ?",
            "Tensor<[16, 1, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [1]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 16, 14, 14]> grad_output = ?",
            "Tensor<[1, 4, 7, 7]> input = ?",
            "Tensor<[4, 16, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [16]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 4, 14, 14]> grad_output = ?",
            "Tensor<[1, 16, 14, 14]> input = ?",
            "Tensor<[4, 16, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [4]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 16, 28, 28]> grad_output = ?",
            "Tensor<[1, 1, 28, 28]> input = ?",
            "Tensor<[16, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [16]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 768, 7, 7]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[768, 3, 32, 32]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [32, 32]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 7, 7]> grad_output = ?",
            "Tensor<[1, 782, 7, 7]> input = ?",
            "Tensor<[1024, 782, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 462, 7, 7]> grad_output = ?",
            "Tensor<[1, 1072, 7, 7]> input = ?",
            "Tensor<[462, 1072, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 160, 7, 7]> grad_output = ?",
            "Tensor<[1, 272, 7, 7]> input = ?",
            "Tensor<[160, 272, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 272, 7, 7]> grad_output = ?",
            "Tensor<[1, 800, 7, 7]> input = ?",
            "Tensor<[272, 800, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 160, 7, 7]> grad_output = ?",
            "Tensor<[1, 640, 7, 7]> input = ?",
            "Tensor<[160, 640, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 640, 14, 14]> grad_output = ?",
            "Tensor<[1, 654, 14, 14]> input = ?",
            "Tensor<[640, 654, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 334, 14, 14]> grad_output = ?",
            "Tensor<[1, 740, 14, 14]> input = ?",
            "Tensor<[334, 740, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 68, 14, 14]> input = ?",
            "Tensor<[40, 68, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 68, 14, 14]> grad_output = ?",
            "Tensor<[1, 156, 14, 14]> input = ?",
            "Tensor<[68, 156, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 116, 14, 14]> input = ?",
            "Tensor<[40, 116, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 116, 14, 14]> grad_output = ?",
            "Tensor<[1, 304, 14, 14]> input = ?",
            "Tensor<[116, 304, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 68, 14, 14]> grad_output = ?",
            "Tensor<[1, 236, 14, 14]> input = ?",
            "Tensor<[68, 236, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 196, 14, 14]> input = ?",
            "Tensor<[40, 196, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 196, 14, 14]> grad_output = ?",
            "Tensor<[1, 544, 14, 14]> input = ?",
            "Tensor<[196, 544, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 116, 14, 14]> grad_output = ?",
            "Tensor<[1, 428, 14, 14]> input = ?",
            "Tensor<[116, 428, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 68, 14, 14]> grad_output = ?",
            "Tensor<[1, 360, 14, 14]> input = ?",
            "Tensor<[68, 360, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 320, 14, 14]> input = ?",
            "Tensor<[40, 320, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 320, 28, 28]> grad_output = ?",
            "Tensor<[1, 328, 28, 28]> input = ?",
            "Tensor<[320, 328, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 168, 28, 28]> grad_output = ?",
            "Tensor<[1, 466, 28, 28]> input = ?",
            "Tensor<[168, 466, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 34, 28, 28]> input = ?",
            "Tensor<[20, 34, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 34, 28, 28]> grad_output = ?",
            "Tensor<[1, 78, 28, 28]> input = ?",
            "Tensor<[34, 78, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 58, 28, 28]> input = ?",
            "Tensor<[20, 58, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 58, 28, 28]> grad_output = ?",
            "Tensor<[1, 152, 28, 28]> input = ?",
            "Tensor<[58, 152, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 34, 28, 28]> grad_output = ?",
            "Tensor<[1, 118, 28, 28]> input = ?",
            "Tensor<[34, 118, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 98, 28, 28]> input = ?",
            "Tensor<[20, 98, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 98, 28, 28]> grad_output = ?",
            "Tensor<[1, 368, 28, 28]> input = ?",
            "Tensor<[98, 368, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 58, 28, 28]> grad_output = ?",
            "Tensor<[1, 310, 28, 28]> input = ?",
            "Tensor<[58, 310, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 34, 28, 28]> grad_output = ?",
            "Tensor<[1, 276, 28, 28]> input = ?",
            "Tensor<[34, 276, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[20, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 28, 28]> grad_output = ?",
            "Tensor<[1, 262, 28, 28]> input = ?",
            "Tensor<[256, 262, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 134, 28, 28]> grad_output = ?",
            "Tensor<[1, 296, 28, 28]> input = ?",
            "Tensor<[134, 296, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 28, 28]> grad_output = ?",
            "Tensor<[1, 28, 28, 28]> input = ?",
            "Tensor<[16, 28, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 28, 28, 28]> grad_output = ?",
            "Tensor<[1, 62, 28, 28]> input = ?",
            "Tensor<[28, 62, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 28, 28]> grad_output = ?",
            "Tensor<[1, 46, 28, 28]> input = ?",
            "Tensor<[16, 46, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 46, 28, 28]> grad_output = ?",
            "Tensor<[1, 122, 28, 28]> input = ?",
            "Tensor<[46, 122, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 28, 28, 28]> grad_output = ?",
            "Tensor<[1, 94, 28, 28]> input = ?",
            "Tensor<[28, 94, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 28, 28]> grad_output = ?",
            "Tensor<[1, 78, 28, 28]> input = ?",
            "Tensor<[16, 78, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 78, 28, 28]> grad_output = ?",
            "Tensor<[1, 218, 28, 28]> input = ?",
            "Tensor<[78, 218, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 46, 28, 28]> grad_output = ?",
            "Tensor<[1, 172, 28, 28]> input = ?",
            "Tensor<[46, 172, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 28, 28, 28]> grad_output = ?",
            "Tensor<[1, 144, 28, 28]> input = ?",
            "Tensor<[28, 144, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[16, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 56, 56]> grad_output = ?",
            "Tensor<[1, 124, 56, 56]> input = ?",
            "Tensor<[128, 124, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 68, 56, 56]> grad_output = ?",
            "Tensor<[1, 142, 56, 56]> input = ?",
            "Tensor<[68, 142, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 14, 56, 56]> grad_output = ?",
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Tensor<[14, 24, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 56, 56]> grad_output = ?",
            "Tensor<[1, 54, 56, 56]> input = ?",
            "Tensor<[24, 54, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 14, 56, 56]> grad_output = ?",
            "Tensor<[1, 40, 56, 56]> input = ?",
            "Tensor<[14, 40, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 56, 56]> grad_output = ?",
            "Tensor<[1, 102, 56, 56]> input = ?",
            "Tensor<[40, 102, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 56, 56]> grad_output = ?",
            "Tensor<[1, 78, 56, 56]> input = ?",
            "Tensor<[24, 78, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 14, 56, 56]> grad_output = ?",
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Tensor<[14, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 112, 112]> grad_output = ?",
            "Tensor<[1, 32, 112, 112]> input = ?",
            "Tensor<[64, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 112, 112]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[32, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 512]> grad_output = ?",
            "Tensor<[1, 1024, 512]> input = ?",
            "Tensor<[256, 1024, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [256]",
            "List[int] stride = [1]",
            "List[int] padding = [0]",
            "List[int] dilation = [1]",
            "bool transposed = False",
            "List[int] output_padding = [0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1024, 512]> grad_output = ?",
            "Tensor<[1, 256, 512]> input = ?",
            "Tensor<[1024, 256, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [1024]",
            "List[int] stride = [1]",
            "List[int] padding = [0]",
            "List[int] dilation = [1]",
            "bool transposed = False",
            "List[int] output_padding = [0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 24, 24]> grad_output = ?",
            "Tensor<[1, 32, 26, 26]> input = ?",
            "Tensor<[64, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [64]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 26, 26]> grad_output = ?",
            "Tensor<[1, 1, 28, 28]> input = ?",
            "Tensor<[32, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [32]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 38, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[38, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [38]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[128, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [128]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 19, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[19, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [19]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[128, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [128]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [2, 2]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[128, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [128]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 185, 28, 28]> input = ?",
            "Tensor<[128, 185, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [128]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 38, 28, 28]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[38, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [38]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[512, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [512]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 19, 28, 28]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[19, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [19]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[128, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[128, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 128",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[128, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [128]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[512, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[512, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 512",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[512, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [2, 2]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 512",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[512, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 28, 28]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[256, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 256",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 28, 28]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[256, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[256, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 56, 56]> input = ?",
            "Tensor<[128, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 128",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 56, 56]> grad_output = ?",
            "Tensor<[1, 128, 56, 56]> input = ?",
            "Tensor<[128, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 56, 56]> grad_output = ?",
            "Tensor<[1, 128, 56, 56]> input = ?",
            "Tensor<[128, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 128",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 56, 56]> grad_output = ?",
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Tensor<[128, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 56, 56]> grad_output = ?",
            "Tensor<[1, 64, 112, 112]> input = ?",
            "Tensor<[64, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 64",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 112, 112]> grad_output = ?",
            "Tensor<[1, 32, 112, 112]> input = ?",
            "Tensor<[64, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 112, 112]> grad_output = ?",
            "Tensor<[1, 32, 112, 112]> input = ?",
            "Tensor<[32, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 32",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 7, 7]> grad_output = ?",
            "Tensor<[1, 512, 7, 7]> input = ?",
            "Tensor<[512, 512, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 7, 7]> grad_output = ?",
            "Tensor<[1, 256, 14, 14]> input = ?",
            "Tensor<[512, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 7, 7]> grad_output = ?",
            "Tensor<[1, 256, 14, 14]> input = ?",
            "Tensor<[512, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_output = ?",
            "Tensor<[1, 256, 14, 14]> input = ?",
            "Tensor<[256, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[256, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[256, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[128, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Tensor<[128, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Tensor<[128, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 56, 56]> grad_output = ?",
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Tensor<[64, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 112, 112]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[64, 3, 7, 7]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [3, 3]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 2048, 7, 7]> grad_output = ?",
            "Tensor<[1, 512, 7, 7]> input = ?",
            "Tensor<[2048, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 7, 7]> grad_output = ?",
            "Tensor<[1, 2048, 7, 7]> input = ?",
            "Tensor<[512, 2048, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 2048, 7, 7]> grad_output = ?",
            "Tensor<[1, 1024, 14, 14]> input = ?",
            "Tensor<[2048, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 7, 7]> grad_output = ?",
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Tensor<[512, 512, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 14, 14]> grad_output = ?",
            "Tensor<[1, 1024, 14, 14]> input = ?",
            "Tensor<[512, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 14, 14]> grad_output = ?",
            "Tensor<[1, 256, 14, 14]> input = ?",
            "Tensor<[1024, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_output = ?",
            "Tensor<[1, 1024, 14, 14]> input = ?",
            "Tensor<[256, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 14, 14]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[1024, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[256, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 28, 28]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[256, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Tensor<[512, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[128, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 256, 56, 56]> input = ?",
            "Tensor<[512, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 56, 56]> input = ?",
            "Tensor<[128, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 56, 56]> grad_output = ?",
            "Tensor<[1, 256, 56, 56]> input = ?",
            "Tensor<[128, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 56, 56]> grad_output = ?",
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Tensor<[256, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 56, 56]> grad_output = ?",
            "Tensor<[1, 256, 56, 56]> input = ?",
            "Tensor<[64, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 56, 56]> grad_output = ?",
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Tensor<[64, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 150, 128, 128]> grad_output = ?",
            "Tensor<[1, 256, 128, 128]> input = ?",
            "Tensor<[150, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [150]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 128, 128]> grad_output = ?",
            "Tensor<[1, 1024, 128, 128]> input = ?",
            "Tensor<[256, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 16, 16]> grad_output = ?",
            "Tensor<[1, 1024, 16, 16]> input = ?",
            "Tensor<[1024, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [1024]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1024",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 16, 16]> grad_output = ?",
            "Tensor<[1, 160, 32, 32]> input = ?",
            "Tensor<[256, 160, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [256]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 640, 32, 32]> grad_output = ?",
            "Tensor<[1, 640, 32, 32]> input = ?",
            "Tensor<[640, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [640]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 640",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 160, 16, 16]> grad_output = ?",
            "Tensor<[1, 160, 32, 32]> input = ?",
            "Tensor<[160, 160, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [160]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 160, 32, 32]> grad_output = ?",
            "Tensor<[1, 64, 64, 64]> input = ?",
            "Tensor<[160, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [160]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 64, 64]> grad_output = ?",
            "Tensor<[1, 256, 64, 64]> input = ?",
            "Tensor<[256, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [256]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 256",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 16, 16]> grad_output = ?",
            "Tensor<[1, 64, 64, 64]> input = ?",
            "Tensor<[64, 64, 4, 4]> weight = ?",
            "Optional[List[int]] bias_sizes = [64]",
            "List[int] stride = [4, 4]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 64, 64]> grad_output = ?",
            "Tensor<[1, 32, 128, 128]> input = ?",
            "Tensor<[64, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [64]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 128, 128]> grad_output = ?",
            "Tensor<[1, 128, 128, 128]> input = ?",
            "Tensor<[128, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [128]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 128",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 16, 16]> grad_output = ?",
            "Tensor<[1, 32, 128, 128]> input = ?",
            "Tensor<[32, 32, 8, 8]> weight = ?",
            "Optional[List[int]] bias_sizes = [32]",
            "List[int] stride = [8, 8]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 128, 128]> grad_output = ?",
            "Tensor<[1, 3, 512, 512]> input = ?",
            "Tensor<[32, 3, 7, 7]> weight = ?",
            "Optional[List[int]] bias_sizes = [32]",
            "List[int] stride = [4, 4]",
            "List[int] padding = [3, 3]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1, 256, 256]> grad_output = ?",
            "Tensor<[1, 32, 256, 256]> input = ?",
            "Tensor<[1, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [1]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 256, 256]> grad_output = ?",
            "Tensor<[1, 32, 256, 256]> input = ?",
            "Tensor<[32, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 256, 256]> grad_output = ?",
            "Tensor<[1, 64, 256, 256]> input = ?",
            "Tensor<[32, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 256, 256]> grad_output = ?",
            "Tensor<[1, 64, 128, 128]> input = ?",
            "Tensor<[64, 32, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [32]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 128, 128]> grad_output = ?",
            "Tensor<[1, 64, 128, 128]> input = ?",
            "Tensor<[64, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 128, 128]> grad_output = ?",
            "Tensor<[1, 128, 128, 128]> input = ?",
            "Tensor<[64, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 128, 128]> grad_output = ?",
            "Tensor<[1, 128, 64, 64]> input = ?",
            "Tensor<[128, 64, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [64]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 64, 64]> grad_output = ?",
            "Tensor<[1, 128, 64, 64]> input = ?",
            "Tensor<[128, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 64, 64]> grad_output = ?",
            "Tensor<[1, 256, 64, 64]> input = ?",
            "Tensor<[128, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 64, 64]> grad_output = ?",
            "Tensor<[1, 256, 32, 32]> input = ?",
            "Tensor<[256, 128, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [128]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 32, 32]> grad_output = ?",
            "Tensor<[1, 256, 32, 32]> input = ?",
            "Tensor<[256, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 32, 32]> grad_output = ?",
            "Tensor<[1, 512, 32, 32]> input = ?",
            "Tensor<[256, 512, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 32, 32]> grad_output = ?",
            "Tensor<[1, 512, 16, 16]> input = ?",
            "Tensor<[512, 256, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [256]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 16, 16]> grad_output = ?",
            "Tensor<[1, 512, 16, 16]> input = ?",
            "Tensor<[512, 512, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 16, 16]> grad_output = ?",
            "Tensor<[1, 256, 16, 16]> input = ?",
            "Tensor<[512, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 32, 32]> grad_output = ?",
            "Tensor<[1, 128, 32, 32]> input = ?",
            "Tensor<[256, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 64, 64]> grad_output = ?",
            "Tensor<[1, 64, 64, 64]> input = ?",
            "Tensor<[128, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 128, 128]> grad_output = ?",
            "Tensor<[1, 32, 128, 128]> input = ?",
            "Tensor<[64, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 256, 256]> grad_output = ?",
            "Tensor<[1, 3, 256, 256]> input = ?",
            "Tensor<[32, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1, 224, 224]> grad_output = ?",
            "Tensor<[1, 64, 224, 224]> input = ?",
            "Tensor<[1, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [1]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 64, 224, 224]> grad_output = ?",
            "Tensor<[1, 64, 224, 224]> input = ?",
            "Tensor<[64, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 224, 224]> grad_output = ?",
            "Tensor<[1, 128, 224, 224]> input = ?",
            "Tensor<[64, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 224, 224]> grad_output = ?",
            "Tensor<[1, 128, 112, 112]> input = ?",
            "Tensor<[128, 64, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [64]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 128, 112, 112]> grad_output = ?",
            "Tensor<[1, 128, 112, 112]> input = ?",
            "Tensor<[128, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 112, 112]> grad_output = ?",
            "Tensor<[1, 256, 112, 112]> input = ?",
            "Tensor<[128, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 112, 112]> grad_output = ?",
            "Tensor<[1, 256, 56, 56]> input = ?",
            "Tensor<[256, 128, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [128]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 56, 56]> grad_output = ?",
            "Tensor<[1, 256, 56, 56]> input = ?",
            "Tensor<[256, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 56, 56]> grad_output = ?",
            "Tensor<[1, 512, 56, 56]> input = ?",
            "Tensor<[256, 512, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 56, 56]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[512, 256, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [256]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Tensor<[512, 512, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 1024, 28, 28]> input = ?",
            "Tensor<[512, 1024, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 1024, 14, 14]> input = ?",
            "Tensor<[1024, 512, 2, 2]> weight = ?",
            "Optional[List[int]] bias_sizes = [512]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = True",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1024, 14, 14]> grad_output = ?",
            "Tensor<[1, 1024, 14, 14]> input = ?",
            "Tensor<[1024, 1024, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 14, 14]> grad_output = ?",
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Tensor<[1024, 512, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[512, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 56, 56]> grad_output = ?",
            "Tensor<[1, 128, 56, 56]> input = ?",
            "Tensor<[256, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 112, 112]> grad_output = ?",
            "Tensor<[1, 64, 112, 112]> input = ?",
            "Tensor<[128, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 224, 224]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[64, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1000, 1, 1]> grad_output = ?",
            "Tensor<[1, 512, 1, 1]> input = ?",
            "Tensor<[1000, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [1000]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 7, 7]> grad_output = ?",
            "Tensor<[1, 1280, 7, 7]> input = ?",
            "Tensor<[512, 1280, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 7, 7]> grad_output = ?",
            "Tensor<[1, 256, 7, 7]> input = ?",
            "Tensor<[512, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_output = ?",
            "Tensor<[1, 896, 14, 14]> input = ?",
            "Tensor<[256, 896, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_output = ?",
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Tensor<[256, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_output = ?",
            "Tensor<[1, 128, 14, 14]> input = ?",
            "Tensor<[256, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 448, 28, 28]> input = ?",
            "Tensor<[128, 448, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[128, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 28, 28]> grad_output = ?",
            "Tensor<[1, 64, 28, 28]> input = ?",
            "Tensor<[128, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 56, 56]> grad_output = ?",
            "Tensor<[1, 128, 56, 56]> input = ?",
            "Tensor<[64, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 56, 56]> grad_output = ?",
            "Tensor<[1, 32, 112, 112]> input = ?",
            "Tensor<[64, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 56, 56]> grad_output = ?",
            "Tensor<[1, 32, 56, 56]> input = ?",
            "Tensor<[64, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 112, 112]> grad_output = ?",
            "Tensor<[1, 16, 224, 224]> input = ?",
            "Tensor<[32, 16, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 224, 224]> grad_output = ?",
            "Tensor<[1, 16, 224, 224]> input = ?",
            "Tensor<[16, 16, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 224, 224]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[16, 3, 7, 7]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [3, 3]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 1, 1]> grad_output = ?",
            "Tensor<[1, 1024, 1, 1]> input = ?",
            "Tensor<[1024, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [1024]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1024, 7, 7]> grad_output = ?",
            "Tensor<[1, 1440, 7, 7]> input = ?",
            "Tensor<[1024, 1440, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 224, 7, 7]> grad_output = ?",
            "Tensor<[1, 224, 7, 7]> input = ?",
            "Tensor<[224, 224, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 224, 7, 7]> grad_output = ?",
            "Tensor<[1, 224, 7, 7]> input = ?",
            "Tensor<[224, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 224",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 224, 7, 7]> grad_output = ?",
            "Tensor<[1, 768, 7, 7]> input = ?",
            "Tensor<[224, 768, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 768, 1, 1]> grad_output = ?",
            "Tensor<[1, 768, 1, 1]> input = ?",
            "Tensor<[768, 768, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [768]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 768, 14, 14]> grad_output = ?",
            "Tensor<[1, 1088, 14, 14]> input = ?",
            "Tensor<[768, 1088, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 14, 14]> grad_output = ?",
            "Tensor<[1, 192, 14, 14]> input = ?",
            "Tensor<[192, 192, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 14, 14]> grad_output = ?",
            "Tensor<[1, 192, 14, 14]> input = ?",
            "Tensor<[192, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 192",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 14, 14]> grad_output = ?",
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Tensor<[192, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 1, 1]> grad_output = ?",
            "Tensor<[1, 512, 1, 1]> input = ?",
            "Tensor<[512, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [512]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 28, 28]> grad_output = ?",
            "Tensor<[1, 736, 28, 28]> input = ?",
            "Tensor<[512, 736, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 160, 28, 28]> grad_output = ?",
            "Tensor<[1, 160, 28, 28]> input = ?",
            "Tensor<[160, 160, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 160, 28, 28]> grad_output = ?",
            "Tensor<[1, 160, 28, 28]> input = ?",
            "Tensor<[160, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 160",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 160, 28, 28]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[160, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 1, 1]> grad_output = ?",
            "Tensor<[1, 256, 1, 1]> input = ?",
            "Tensor<[256, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [256]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 56, 56]> grad_output = ?",
            "Tensor<[1, 448, 56, 56]> input = ?",
            "Tensor<[256, 448, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 112, 112]> grad_output = ?",
            "Tensor<[1, 64, 112, 112]> input = ?",
            "Tensor<[64, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 112, 112]> grad_output = ?",
            "Tensor<[1, 64, 112, 112]> input = ?",
            "Tensor<[64, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 64",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 112, 112]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[64, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 768, 14, 14]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[768, 3, 16, 16]> weight = ?",
            "Optional[List[int]] bias_sizes = [768]",
            "List[int] stride = [16, 16]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1280, 1, 1]> grad_output = ?",
            "Tensor<[1, 960, 1, 1]> input = ?",
            "Tensor<[1280, 960, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [1280]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 960, 7, 7]> grad_output = ?",
            "Tensor<[1, 160, 7, 7]> input = ?",
            "Tensor<[960, 160, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 7, 7]> grad_output = ?",
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Tensor<[80, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 80",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 7, 7]> grad_output = ?",
            "Tensor<[1, 960, 7, 7]> input = ?",
            "Tensor<[80, 960, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 1, 1]> grad_output = ?",
            "Tensor<[1, 240, 1, 1]> input = ?",
            "Tensor<[960, 240, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [960]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 240, 1, 1]> grad_output = ?",
            "Tensor<[1, 960, 1, 1]> input = ?",
            "Tensor<[240, 960, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [240]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 480, 7, 7]> input = ?",
            "Tensor<[480, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 160, 7, 7]> input = ?",
            "Tensor<[480, 160, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 160, 7, 7]> grad_output = ?",
            "Tensor<[1, 112, 7, 7]> input = ?",
            "Tensor<[160, 112, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 7, 7]> grad_output = ?",
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Tensor<[112, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 112",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Tensor<[80, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 1, 1]> grad_output = ?",
            "Tensor<[1, 168, 1, 1]> input = ?",
            "Tensor<[672, 168, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [672]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 168, 1, 1]> grad_output = ?",
            "Tensor<[1, 672, 1, 1]> input = ?",
            "Tensor<[168, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [168]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Tensor<[672, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 336, 14, 14]> grad_output = ?",
            "Tensor<[1, 336, 14, 14]> input = ?",
            "Tensor<[336, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 336",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 336, 14, 14]> grad_output = ?",
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Tensor<[336, 112, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 56, 14, 14]> grad_output = ?",
            "Tensor<[1, 56, 14, 14]> input = ?",
            "Tensor<[56, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 56",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 56, 14, 14]> grad_output = ?",
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Tensor<[56, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[112, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[80, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 80",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 56, 14, 14]> grad_output = ?",
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Tensor<[56, 480, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 1, 1]> grad_output = ?",
            "Tensor<[1, 120, 1, 1]> input = ?",
            "Tensor<[480, 120, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [480]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 120, 1, 1]> grad_output = ?",
            "Tensor<[1, 480, 1, 1]> input = ?",
            "Tensor<[120, 480, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [120]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Tensor<[240, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[240, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Tensor<[40, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 40",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 184, 14, 14]> input = ?",
            "Tensor<[40, 184, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 92, 14, 14]> grad_output = ?",
            "Tensor<[1, 92, 14, 14]> input = ?",
            "Tensor<[92, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 92",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 92, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[92, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 200, 14, 14]> input = ?",
            "Tensor<[40, 200, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 100, 14, 14]> grad_output = ?",
            "Tensor<[1, 100, 14, 14]> input = ?",
            "Tensor<[100, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 100",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 100, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[100, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Tensor<[80, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Tensor<[40, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 40",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Tensor<[40, 240, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 28, 28]> input = ?",
            "Tensor<[240, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 28, 28]> grad_output = ?",
            "Tensor<[1, 120, 28, 28]> input = ?",
            "Tensor<[120, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 120",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 28, 28]> grad_output = ?",
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Tensor<[120, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 20, 28, 28]> input = ?",
            "Tensor<[20, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 20",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 120, 28, 28]> input = ?",
            "Tensor<[20, 120, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 1, 1]> grad_output = ?",
            "Tensor<[1, 32, 1, 1]> input = ?",
            "Tensor<[120, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [120]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 32, 1, 1]> grad_output = ?",
            "Tensor<[1, 120, 1, 1]> input = ?",
            "Tensor<[32, 120, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [32]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 60, 28, 28]> grad_output = ?",
            "Tensor<[1, 60, 28, 28]> input = ?",
            "Tensor<[60, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 60",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 60, 28, 28]> grad_output = ?",
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Tensor<[60, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 28, 28]> grad_output = ?",
            "Tensor<[1, 24, 28, 28]> input = ?",
            "Tensor<[40, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 28, 28]> grad_output = ?",
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Tensor<[24, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 24",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 20, 28, 28]> grad_output = ?",
            "Tensor<[1, 72, 28, 28]> input = ?",
            "Tensor<[20, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 1, 1]> grad_output = ?",
            "Tensor<[1, 20, 1, 1]> input = ?",
            "Tensor<[72, 20, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [72]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 20, 1, 1]> grad_output = ?",
            "Tensor<[1, 72, 1, 1]> input = ?",
            "Tensor<[20, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [20]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 72, 28, 28]> grad_output = ?",
            "Tensor<[1, 72, 56, 56]> input = ?",
            "Tensor<[72, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 72",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 56, 56]> grad_output = ?",
            "Tensor<[1, 36, 56, 56]> input = ?",
            "Tensor<[36, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 36",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 56, 56]> grad_output = ?",
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Tensor<[36, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 12, 56, 56]> grad_output = ?",
            "Tensor<[1, 12, 56, 56]> input = ?",
            "Tensor<[12, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 12",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 12, 56, 56]> grad_output = ?",
            "Tensor<[1, 72, 56, 56]> input = ?",
            "Tensor<[12, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 56, 56]> grad_output = ?",
            "Tensor<[1, 16, 56, 56]> input = ?",
            "Tensor<[24, 16, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 56, 56]> grad_output = ?",
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Tensor<[16, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 16",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 12, 56, 56]> grad_output = ?",
            "Tensor<[1, 48, 56, 56]> input = ?",
            "Tensor<[12, 48, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 48, 56, 56]> grad_output = ?",
            "Tensor<[1, 48, 112, 112]> input = ?",
            "Tensor<[48, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 48",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 112, 112]> grad_output = ?",
            "Tensor<[1, 24, 112, 112]> input = ?",
            "Tensor<[24, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 24",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 112, 112]> grad_output = ?",
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Tensor<[24, 16, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 8, 112, 112]> grad_output = ?",
            "Tensor<[1, 8, 112, 112]> input = ?",
            "Tensor<[8, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 8",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 8, 112, 112]> grad_output = ?",
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Tensor<[8, 16, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 112, 112]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[16, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 3, 3]> grad_output = ?",
            "Tensor<[1, 960, 3, 3]> input = ?",
            "Tensor<[960, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 960",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 3, 3]> grad_output = ?",
            "Tensor<[1, 960, 3, 3]> input = ?",
            "Tensor<[960, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 960",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 3, 3]> grad_output = ?",
            "Tensor<[1, 160, 3, 3]> input = ?",
            "Tensor<[960, 160, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Tensor<[672, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Tensor<[672, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_output = ?",
            "Tensor<[1, 112, 7, 7]> input = ?",
            "Tensor<[672, 112, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 480, 7, 7]> input = ?",
            "Tensor<[480, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 480, 7, 7]> input = ?",
            "Tensor<[480, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 7, 7]> grad_output = ?",
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Tensor<[480, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 184, 7, 7]> grad_output = ?",
            "Tensor<[1, 184, 7, 7]> input = ?",
            "Tensor<[184, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 184",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 184, 7, 7]> grad_output = ?",
            "Tensor<[1, 184, 7, 7]> input = ?",
            "Tensor<[184, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 184",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 184, 7, 7]> grad_output = ?",
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Tensor<[184, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 200, 7, 7]> grad_output = ?",
            "Tensor<[1, 200, 7, 7]> input = ?",
            "Tensor<[200, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 200",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 200, 7, 7]> grad_output = ?",
            "Tensor<[1, 200, 7, 7]> input = ?",
            "Tensor<[200, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 200",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 200, 7, 7]> grad_output = ?",
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Tensor<[200, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Tensor<[240, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Tensor<[240, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Tensor<[240, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 14, 14]> grad_output = ?",
            "Tensor<[1, 120, 14, 14]> input = ?",
            "Tensor<[120, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 120",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 14, 14]> grad_output = ?",
            "Tensor<[1, 120, 14, 14]> input = ?",
            "Tensor<[120, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 120",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 14, 14]> grad_output = ?",
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Tensor<[120, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 28, 28]> grad_output = ?",
            "Tensor<[1, 72, 28, 28]> input = ?",
            "Tensor<[72, 1, 5, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 72",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 28, 28]> grad_output = ?",
            "Tensor<[1, 72, 28, 28]> input = ?",
            "Tensor<[72, 1, 1, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 72",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 28, 28]> grad_output = ?",
            "Tensor<[1, 24, 28, 28]> input = ?",
            "Tensor<[72, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 2048, 7, 7]> grad_output = ?",
            "Tensor<[1, 1024, 7, 7]> input = ?",
            "Tensor<[2048, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [2048]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1024, 7, 7]> grad_output = ?",
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Tensor<[1024, 512, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [1024]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1024, 7, 7]> grad_output = ?",
            "Tensor<[1, 144, 7, 7]> input = ?",
            "Tensor<[1024, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 7, 7]> grad_output = ?",
            "Tensor<[1, 256, 7, 7]> input = ?",
            "Tensor<[1024, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 7, 7]> grad_output = ?",
            "Tensor<[1, 256, 7, 7]> input = ?",
            "Tensor<[256, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 7, 7]> grad_output = ?",
            "Tensor<[1, 144, 7, 7]> input = ?",
            "Tensor<[256, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 14, 14]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[512, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [512]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 512, 14, 14]> grad_output = ?",
            "Tensor<[1, 72, 14, 14]> input = ?",
            "Tensor<[512, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 14, 14]> grad_output = ?",
            "Tensor<[1, 128, 14, 14]> input = ?",
            "Tensor<[512, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 14, 14]> grad_output = ?",
            "Tensor<[1, 128, 14, 14]> input = ?",
            "Tensor<[128, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 14, 14]> grad_output = ?",
            "Tensor<[1, 72, 14, 14]> input = ?",
            "Tensor<[128, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 28, 28]> grad_output = ?",
            "Tensor<[1, 128, 56, 56]> input = ?",
            "Tensor<[256, 128, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [256]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 256, 28, 28]> grad_output = ?",
            "Tensor<[1, 36, 28, 28]> input = ?",
            "Tensor<[256, 36, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 28, 28]> grad_output = ?",
            "Tensor<[1, 64, 28, 28]> input = ?",
            "Tensor<[256, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 28, 28]> grad_output = ?",
            "Tensor<[1, 64, 28, 28]> input = ?",
            "Tensor<[64, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 28, 28]> grad_output = ?",
            "Tensor<[1, 36, 28, 28]> input = ?",
            "Tensor<[64, 36, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 56, 56]> grad_output = ?",
            "Tensor<[1, 18, 56, 56]> input = ?",
            "Tensor<[128, 18, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 56, 56]> grad_output = ?",
            "Tensor<[1, 32, 56, 56]> input = ?",
            "Tensor<[128, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 56, 56]> grad_output = ?",
            "Tensor<[1, 32, 56, 56]> input = ?",
            "Tensor<[32, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 56, 56]> grad_output = ?",
            "Tensor<[1, 18, 56, 56]> input = ?",
            "Tensor<[32, 18, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 7, 7]> grad_output = ?",
            "Tensor<[1, 72, 14, 14]> input = ?",
            "Tensor<[144, 72, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 7, 7]> grad_output = ?",
            "Tensor<[1, 36, 14, 14]> input = ?",
            "Tensor<[144, 36, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 14, 14]> grad_output = ?",
            "Tensor<[1, 36, 28, 28]> input = ?",
            "Tensor<[36, 36, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 7, 7]> grad_output = ?",
            "Tensor<[1, 18, 14, 14]> input = ?",
            "Tensor<[144, 18, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 18, 14, 14]> grad_output = ?",
            "Tensor<[1, 18, 28, 28]> input = ?",
            "Tensor<[18, 18, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 18, 28, 28]> grad_output = ?",
            "Tensor<[1, 18, 56, 56]> input = ?",
            "Tensor<[18, 18, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 7, 7]> grad_output = ?",
            "Tensor<[1, 144, 7, 7]> input = ?",
            "Tensor<[72, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 14, 14]> grad_output = ?",
            "Tensor<[1, 36, 28, 28]> input = ?",
            "Tensor<[72, 36, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 14, 14]> grad_output = ?",
            "Tensor<[1, 18, 28, 28]> input = ?",
            "Tensor<[72, 18, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 7, 7]> grad_output = ?",
            "Tensor<[1, 144, 7, 7]> input = ?",
            "Tensor<[36, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 14, 14]> grad_output = ?",
            "Tensor<[1, 72, 14, 14]> input = ?",
            "Tensor<[36, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 28, 28]> grad_output = ?",
            "Tensor<[1, 18, 56, 56]> input = ?",
            "Tensor<[36, 18, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 18, 7, 7]> grad_output = ?",
            "Tensor<[1, 144, 7, 7]> input = ?",
            "Tensor<[18, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 18, 14, 14]> grad_output = ?",
            "Tensor<[1, 72, 14, 14]> input = ?",
            "Tensor<[18, 72, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 18, 28, 28]> grad_output = ?",
            "Tensor<[1, 36, 28, 28]> input = ?",
            "Tensor<[18, 36, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 7, 7]> grad_output = ?",
            "Tensor<[1, 144, 7, 7]> input = ?",
            "Tensor<[144, 144, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 72, 14, 14]> grad_output = ?",
            "Tensor<[1, 72, 14, 14]> input = ?",
            "Tensor<[72, 72, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 28, 28]> grad_output = ?",
            "Tensor<[1, 36, 28, 28]> input = ?",
            "Tensor<[36, 36, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 18, 56, 56]> grad_output = ?",
            "Tensor<[1, 18, 56, 56]> input = ?",
            "Tensor<[18, 18, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 36, 28, 28]> grad_output = ?",
            "Tensor<[1, 256, 56, 56]> input = ?",
            "Tensor<[36, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 18, 56, 56]> grad_output = ?",
            "Tensor<[1, 256, 56, 56]> input = ?",
            "Tensor<[18, 256, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 56, 56]> grad_output = ?",
            "Tensor<[1, 64, 112, 112]> input = ?",
            "Tensor<[64, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 8, 8]> grad_output = ?",
            "Tensor<[1, 1536, 8, 8]> input = ?",
            "Tensor<[256, 1536, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 8, 8]> grad_output = ?",
            "Tensor<[1, 512, 8, 8]> input = ?",
            "Tensor<[256, 512, 3, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 8, 8]> grad_output = ?",
            "Tensor<[1, 512, 8, 8]> input = ?",
            "Tensor<[256, 512, 1, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 8, 8]> grad_output = ?",
            "Tensor<[1, 448, 8, 8]> input = ?",
            "Tensor<[512, 448, 1, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 448, 8, 8]> grad_output = ?",
            "Tensor<[1, 384, 8, 8]> input = ?",
            "Tensor<[448, 384, 3, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 384, 8, 8]> grad_output = ?",
            "Tensor<[1, 1536, 8, 8]> input = ?",
            "Tensor<[384, 1536, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 8, 8]> grad_output = ?",
            "Tensor<[1, 384, 8, 8]> input = ?",
            "Tensor<[256, 384, 3, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 8, 8]> grad_output = ?",
            "Tensor<[1, 384, 8, 8]> input = ?",
            "Tensor<[256, 384, 1, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 320, 8, 8]> grad_output = ?",
            "Tensor<[1, 320, 17, 17]> input = ?",
            "Tensor<[320, 320, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 320, 17, 17]> grad_output = ?",
            "Tensor<[1, 256, 17, 17]> input = ?",
            "Tensor<[320, 256, 7, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [3, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 17, 17]> grad_output = ?",
            "Tensor<[1, 256, 17, 17]> input = ?",
            "Tensor<[256, 256, 1, 7]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 3]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 17, 17]> grad_output = ?",
            "Tensor<[1, 1024, 17, 17]> input = ?",
            "Tensor<[256, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 8, 8]> grad_output = ?",
            "Tensor<[1, 192, 17, 17]> input = ?",
            "Tensor<[192, 192, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 17, 17]> grad_output = ?",
            "Tensor<[1, 1024, 17, 17]> input = ?",
            "Tensor<[192, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 17, 17]> grad_output = ?",
            "Tensor<[1, 1024, 17, 17]> input = ?",
            "Tensor<[128, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 17, 17]> grad_output = ?",
            "Tensor<[1, 224, 17, 17]> input = ?",
            "Tensor<[256, 224, 1, 7]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 3]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 224, 17, 17]> grad_output = ?",
            "Tensor<[1, 224, 17, 17]> input = ?",
            "Tensor<[224, 224, 7, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [3, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 224, 17, 17]> grad_output = ?",
            "Tensor<[1, 192, 17, 17]> input = ?",
            "Tensor<[224, 192, 1, 7]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 3]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 17, 17]> grad_output = ?",
            "Tensor<[1, 192, 17, 17]> input = ?",
            "Tensor<[192, 192, 7, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [3, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 17, 17]> grad_output = ?",
            "Tensor<[1, 224, 17, 17]> input = ?",
            "Tensor<[256, 224, 7, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [3, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 384, 17, 17]> grad_output = ?",
            "Tensor<[1, 1024, 17, 17]> input = ?",
            "Tensor<[384, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 17, 17]> grad_output = ?",
            "Tensor<[1, 224, 35, 35]> input = ?",
            "Tensor<[256, 224, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 224, 35, 35]> grad_output = ?",
            "Tensor<[1, 192, 35, 35]> input = ?",
            "Tensor<[224, 192, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 35, 35]> grad_output = ?",
            "Tensor<[1, 384, 35, 35]> input = ?",
            "Tensor<[192, 384, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 384, 17, 17]> grad_output = ?",
            "Tensor<[1, 384, 35, 35]> input = ?",
            "Tensor<[384, 384, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 35, 35]> grad_output = ?",
            "Tensor<[1, 384, 35, 35]> input = ?",
            "Tensor<[96, 384, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 35, 35]> grad_output = ?",
            "Tensor<[1, 96, 35, 35]> input = ?",
            "Tensor<[96, 96, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 35, 35]> grad_output = ?",
            "Tensor<[1, 64, 35, 35]> input = ?",
            "Tensor<[96, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 35, 35]> grad_output = ?",
            "Tensor<[1, 384, 35, 35]> input = ?",
            "Tensor<[64, 384, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 35, 35]> grad_output = ?",
            "Tensor<[1, 192, 71, 71]> input = ?",
            "Tensor<[192, 192, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 71, 71]> grad_output = ?",
            "Tensor<[1, 64, 73, 73]> input = ?",
            "Tensor<[96, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 73, 73]> grad_output = ?",
            "Tensor<[1, 64, 73, 73]> input = ?",
            "Tensor<[64, 64, 7, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [3, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 73, 73]> grad_output = ?",
            "Tensor<[1, 64, 73, 73]> input = ?",
            "Tensor<[64, 64, 1, 7]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 3]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 73, 73]> grad_output = ?",
            "Tensor<[1, 160, 73, 73]> input = ?",
            "Tensor<[64, 160, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 73, 73]> grad_output = ?",
            "Tensor<[1, 64, 147, 147]> input = ?",
            "Tensor<[96, 64, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 147, 147]> grad_output = ?",
            "Tensor<[1, 32, 147, 147]> input = ?",
            "Tensor<[64, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 147, 147]> grad_output = ?",
            "Tensor<[1, 32, 149, 149]> input = ?",
            "Tensor<[32, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 149, 149]> grad_output = ?",
            "Tensor<[1, 3, 299, 299]> input = ?",
            "Tensor<[32, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 14, 14]> grad_output = ?",
            "Tensor<[1, 3, 224, 224]> input = ?",
            "Tensor<[1024, 3, 16, 16]> weight = ?",
            "Optional[List[int]] bias_sizes = [1024]",
            "List[int] stride = [16, 16]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, True]",
        ],
        [
            "Tensor<[1, 1024, 7, 7]> grad_output = ?",
            "Tensor<[1, 1024, 7, 7]> input = ?",
            "Tensor<[1024, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 7, 7]> grad_output = ?",
            "Tensor<[1, 1024, 7, 7]> input = ?",
            "Tensor<[1024, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1024",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 7, 7]> grad_output = ?",
            "Tensor<[1, 512, 7, 7]> input = ?",
            "Tensor<[1024, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 7, 7]> grad_output = ?",
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Tensor<[512, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 512",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 14, 14]> grad_output = ?",
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Tensor<[512, 512, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 14, 14]> grad_output = ?",
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Tensor<[512, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 512",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 512, 14, 14]> grad_output = ?",
            "Tensor<[1, 256, 14, 14]> input = ?",
            "Tensor<[512, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 14, 14]> grad_output = ?",
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Tensor<[256, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 256",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1280, 7, 7]> grad_output = ?",
            "Tensor<[1, 320, 7, 7]> input = ?",
            "Tensor<[1280, 320, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 320, 7, 7]> grad_output = ?",
            "Tensor<[1, 1152, 7, 7]> input = ?",
            "Tensor<[320, 1152, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1152, 7, 7]> grad_output = ?",
            "Tensor<[1, 1152, 7, 7]> input = ?",
            "Tensor<[1152, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1152",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1152, 7, 7]> grad_output = ?",
            "Tensor<[1, 192, 7, 7]> input = ?",
            "Tensor<[1152, 192, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 7, 7]> grad_output = ?",
            "Tensor<[1, 1152, 7, 7]> input = ?",
            "Tensor<[192, 1152, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1152, 7, 7]> grad_output = ?",
            "Tensor<[1, 1152, 7, 7]> input = ?",
            "Tensor<[1152, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1152",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Tensor<[192, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 7, 7]> grad_output = ?",
            "Tensor<[1, 672, 17, 17]> input = ?",
            "Tensor<[672, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 14, 14]> grad_output = ?",
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Tensor<[672, 112, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 14, 14]> grad_output = ?",
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Tensor<[112, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 14, 14]> grad_output = ?",
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Tensor<[672, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 14, 14]> grad_output = ?",
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Tensor<[112, 480, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 14, 14]> grad_output = ?",
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Tensor<[480, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 14, 14]> grad_output = ?",
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Tensor<[480, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 14, 14]> grad_output = ?",
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Tensor<[80, 480, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 14, 14]> grad_output = ?",
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Tensor<[480, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Tensor<[80, 240, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 14, 14]> grad_output = ?",
            "Tensor<[1, 240, 29, 29]> input = ?",
            "Tensor<[240, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 28, 28]> grad_output = ?",
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Tensor<[240, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 28, 28]> grad_output = ?",
            "Tensor<[1, 240, 28, 28]> input = ?",
            "Tensor<[40, 240, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 28, 28]> grad_output = ?",
            "Tensor<[1, 240, 28, 28]> input = ?",
            "Tensor<[240, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 28, 28]> grad_output = ?",
            "Tensor<[1, 144, 28, 28]> input = ?",
            "Tensor<[40, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 28, 28]> grad_output = ?",
            "Tensor<[1, 144, 59, 59]> input = ?",
            "Tensor<[144, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 144",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 56, 56]> grad_output = ?",
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Tensor<[144, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 56, 56]> grad_output = ?",
            "Tensor<[1, 144, 56, 56]> input = ?",
            "Tensor<[24, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 56, 56]> grad_output = ?",
            "Tensor<[1, 144, 56, 56]> input = ?",
            "Tensor<[144, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 144",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 56, 56]> grad_output = ?",
            "Tensor<[1, 96, 56, 56]> input = ?",
            "Tensor<[24, 96, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 56, 56]> grad_output = ?",
            "Tensor<[1, 96, 113, 113]> input = ?",
            "Tensor<[96, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 96",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 112, 112]> grad_output = ?",
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Tensor<[96, 16, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 112, 112]> grad_output = ?",
            "Tensor<[1, 32, 112, 112]> input = ?",
            "Tensor<[16, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 112, 112]> grad_output = ?",
            "Tensor<[1, 3, 225, 225]> input = ?",
            "Tensor<[32, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1280, 8, 8]> grad_output = ?",
            "Tensor<[1, 320, 8, 8]> input = ?",
            "Tensor<[1280, 320, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 320, 8, 8]> grad_output = ?",
            "Tensor<[1, 1152, 8, 8]> input = ?",
            "Tensor<[320, 1152, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1152, 8, 8]> grad_output = ?",
            "Tensor<[1, 1152, 8, 8]> input = ?",
            "Tensor<[1152, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1152",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1152, 8, 8]> grad_output = ?",
            "Tensor<[1, 192, 8, 8]> input = ?",
            "Tensor<[1152, 192, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 8, 8]> grad_output = ?",
            "Tensor<[1, 1152, 8, 8]> input = ?",
            "Tensor<[192, 1152, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1152, 8, 8]> grad_output = ?",
            "Tensor<[1, 1152, 8, 8]> input = ?",
            "Tensor<[1152, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1152",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 8, 8]> grad_output = ?",
            "Tensor<[1, 672, 8, 8]> input = ?",
            "Tensor<[192, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 8, 8]> grad_output = ?",
            "Tensor<[1, 672, 19, 19]> input = ?",
            "Tensor<[672, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 15, 15]> grad_output = ?",
            "Tensor<[1, 112, 15, 15]> input = ?",
            "Tensor<[672, 112, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 15, 15]> grad_output = ?",
            "Tensor<[1, 672, 15, 15]> input = ?",
            "Tensor<[112, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 15, 15]> grad_output = ?",
            "Tensor<[1, 672, 15, 15]> input = ?",
            "Tensor<[672, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 15, 15]> grad_output = ?",
            "Tensor<[1, 480, 15, 15]> input = ?",
            "Tensor<[112, 480, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 15, 15]> grad_output = ?",
            "Tensor<[1, 480, 15, 15]> input = ?",
            "Tensor<[480, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 15, 15]> grad_output = ?",
            "Tensor<[1, 80, 15, 15]> input = ?",
            "Tensor<[480, 80, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 15, 15]> grad_output = ?",
            "Tensor<[1, 480, 15, 15]> input = ?",
            "Tensor<[80, 480, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 480, 15, 15]> grad_output = ?",
            "Tensor<[1, 480, 15, 15]> input = ?",
            "Tensor<[480, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 480",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 80, 15, 15]> grad_output = ?",
            "Tensor<[1, 240, 15, 15]> input = ?",
            "Tensor<[80, 240, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 15, 15]> grad_output = ?",
            "Tensor<[1, 240, 31, 31]> input = ?",
            "Tensor<[240, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 30, 30]> grad_output = ?",
            "Tensor<[1, 40, 30, 30]> input = ?",
            "Tensor<[240, 40, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 30, 30]> grad_output = ?",
            "Tensor<[1, 240, 30, 30]> input = ?",
            "Tensor<[40, 240, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 240, 30, 30]> grad_output = ?",
            "Tensor<[1, 240, 30, 30]> input = ?",
            "Tensor<[240, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 240",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 40, 30, 30]> grad_output = ?",
            "Tensor<[1, 144, 30, 30]> input = ?",
            "Tensor<[40, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 30, 30]> grad_output = ?",
            "Tensor<[1, 144, 63, 63]> input = ?",
            "Tensor<[144, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 144",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 60, 60]> grad_output = ?",
            "Tensor<[1, 24, 60, 60]> input = ?",
            "Tensor<[144, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 60, 60]> grad_output = ?",
            "Tensor<[1, 144, 60, 60]> input = ?",
            "Tensor<[24, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 60, 60]> grad_output = ?",
            "Tensor<[1, 144, 60, 60]> input = ?",
            "Tensor<[144, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 144",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 60, 60]> grad_output = ?",
            "Tensor<[1, 96, 60, 60]> input = ?",
            "Tensor<[24, 96, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 60, 60]> grad_output = ?",
            "Tensor<[1, 96, 121, 121]> input = ?",
            "Tensor<[96, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 96",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 120, 120]> grad_output = ?",
            "Tensor<[1, 16, 120, 120]> input = ?",
            "Tensor<[96, 16, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 120, 120]> grad_output = ?",
            "Tensor<[1, 32, 120, 120]> input = ?",
            "Tensor<[16, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 120, 120]> grad_output = ?",
            "Tensor<[1, 32, 120, 120]> input = ?",
            "Tensor<[32, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 32",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 120, 120]> grad_output = ?",
            "Tensor<[1, 3, 241, 241]> input = ?",
            "Tensor<[32, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1280, 9, 9]> grad_output = ?",
            "Tensor<[1, 352, 9, 9]> input = ?",
            "Tensor<[1280, 352, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 352, 9, 9]> grad_output = ?",
            "Tensor<[1, 1248, 9, 9]> input = ?",
            "Tensor<[352, 1248, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1248, 9, 9]> grad_output = ?",
            "Tensor<[1, 1248, 9, 9]> input = ?",
            "Tensor<[1248, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1248",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1248, 9, 9]> grad_output = ?",
            "Tensor<[1, 208, 9, 9]> input = ?",
            "Tensor<[1248, 208, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 208, 9, 9]> grad_output = ?",
            "Tensor<[1, 1248, 9, 9]> input = ?",
            "Tensor<[208, 1248, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1248, 9, 9]> grad_output = ?",
            "Tensor<[1, 1248, 9, 9]> input = ?",
            "Tensor<[1248, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1248",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 208, 9, 9]> grad_output = ?",
            "Tensor<[1, 720, 9, 9]> input = ?",
            "Tensor<[208, 720, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 720, 9, 9]> grad_output = ?",
            "Tensor<[1, 720, 21, 21]> input = ?",
            "Tensor<[720, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 720",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 720, 17, 17]> grad_output = ?",
            "Tensor<[1, 120, 17, 17]> input = ?",
            "Tensor<[720, 120, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 17, 17]> grad_output = ?",
            "Tensor<[1, 720, 17, 17]> input = ?",
            "Tensor<[120, 720, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 720, 17, 17]> grad_output = ?",
            "Tensor<[1, 720, 17, 17]> input = ?",
            "Tensor<[720, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 720",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 120, 17, 17]> grad_output = ?",
            "Tensor<[1, 528, 17, 17]> input = ?",
            "Tensor<[120, 528, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 528, 17, 17]> grad_output = ?",
            "Tensor<[1, 528, 17, 17]> input = ?",
            "Tensor<[528, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 528",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 528, 17, 17]> grad_output = ?",
            "Tensor<[1, 88, 17, 17]> input = ?",
            "Tensor<[528, 88, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 88, 17, 17]> grad_output = ?",
            "Tensor<[1, 528, 17, 17]> input = ?",
            "Tensor<[88, 528, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 528, 17, 17]> grad_output = ?",
            "Tensor<[1, 528, 17, 17]> input = ?",
            "Tensor<[528, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 528",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 88, 17, 17]> grad_output = ?",
            "Tensor<[1, 288, 17, 17]> input = ?",
            "Tensor<[88, 288, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 288, 17, 17]> grad_output = ?",
            "Tensor<[1, 288, 35, 35]> input = ?",
            "Tensor<[288, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 288",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 288, 33, 33]> grad_output = ?",
            "Tensor<[1, 48, 33, 33]> input = ?",
            "Tensor<[288, 48, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 48, 33, 33]> grad_output = ?",
            "Tensor<[1, 288, 33, 33]> input = ?",
            "Tensor<[48, 288, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 288, 33, 33]> grad_output = ?",
            "Tensor<[1, 288, 33, 33]> input = ?",
            "Tensor<[288, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 288",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 48, 33, 33]> grad_output = ?",
            "Tensor<[1, 144, 33, 33]> input = ?",
            "Tensor<[48, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 33, 33]> grad_output = ?",
            "Tensor<[1, 144, 69, 69]> input = ?",
            "Tensor<[144, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 144",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 65, 65]> grad_output = ?",
            "Tensor<[1, 24, 65, 65]> input = ?",
            "Tensor<[144, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 65, 65]> grad_output = ?",
            "Tensor<[1, 144, 65, 65]> input = ?",
            "Tensor<[24, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 65, 65]> grad_output = ?",
            "Tensor<[1, 144, 65, 65]> input = ?",
            "Tensor<[144, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 144",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 65, 65]> grad_output = ?",
            "Tensor<[1, 96, 65, 65]> input = ?",
            "Tensor<[24, 96, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 65, 65]> grad_output = ?",
            "Tensor<[1, 96, 131, 131]> input = ?",
            "Tensor<[96, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 96",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 130, 130]> grad_output = ?",
            "Tensor<[1, 16, 130, 130]> input = ?",
            "Tensor<[96, 16, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 16, 130, 130]> grad_output = ?",
            "Tensor<[1, 32, 130, 130]> input = ?",
            "Tensor<[16, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 130, 130]> grad_output = ?",
            "Tensor<[1, 32, 130, 130]> input = ?",
            "Tensor<[32, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 32",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 130, 130]> grad_output = ?",
            "Tensor<[1, 3, 261, 261]> input = ?",
            "Tensor<[32, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1280, 10, 10]> grad_output = ?",
            "Tensor<[1, 384, 10, 10]> input = ?",
            "Tensor<[1280, 384, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 384, 10, 10]> grad_output = ?",
            "Tensor<[1, 1392, 10, 10]> input = ?",
            "Tensor<[384, 1392, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1392, 10, 10]> grad_output = ?",
            "Tensor<[1, 1392, 10, 10]> input = ?",
            "Tensor<[1392, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1392",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1392, 10, 10]> grad_output = ?",
            "Tensor<[1, 232, 10, 10]> input = ?",
            "Tensor<[1392, 232, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 232, 10, 10]> grad_output = ?",
            "Tensor<[1, 1392, 10, 10]> input = ?",
            "Tensor<[232, 1392, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1392, 10, 10]> grad_output = ?",
            "Tensor<[1, 1392, 10, 10]> input = ?",
            "Tensor<[1392, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1392",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 232, 10, 10]> grad_output = ?",
            "Tensor<[1, 816, 10, 10]> input = ?",
            "Tensor<[232, 816, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 816, 10, 10]> grad_output = ?",
            "Tensor<[1, 816, 23, 23]> input = ?",
            "Tensor<[816, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 816",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 816, 19, 19]> grad_output = ?",
            "Tensor<[1, 136, 19, 19]> input = ?",
            "Tensor<[816, 136, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 136, 19, 19]> grad_output = ?",
            "Tensor<[1, 816, 19, 19]> input = ?",
            "Tensor<[136, 816, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 816, 19, 19]> grad_output = ?",
            "Tensor<[1, 816, 19, 19]> input = ?",
            "Tensor<[816, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 816",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 136, 19, 19]> grad_output = ?",
            "Tensor<[1, 576, 19, 19]> input = ?",
            "Tensor<[136, 576, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 576, 19, 19]> grad_output = ?",
            "Tensor<[1, 576, 19, 19]> input = ?",
            "Tensor<[576, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 576",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 576, 19, 19]> grad_output = ?",
            "Tensor<[1, 96, 19, 19]> input = ?",
            "Tensor<[576, 96, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 19, 19]> grad_output = ?",
            "Tensor<[1, 576, 19, 19]> input = ?",
            "Tensor<[96, 576, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 576, 19, 19]> grad_output = ?",
            "Tensor<[1, 576, 19, 19]> input = ?",
            "Tensor<[576, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 576",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 96, 19, 19]> grad_output = ?",
            "Tensor<[1, 288, 19, 19]> input = ?",
            "Tensor<[96, 288, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 288, 19, 19]> grad_output = ?",
            "Tensor<[1, 288, 39, 39]> input = ?",
            "Tensor<[288, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 288",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 288, 38, 38]> grad_output = ?",
            "Tensor<[1, 48, 38, 38]> input = ?",
            "Tensor<[288, 48, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 48, 38, 38]> grad_output = ?",
            "Tensor<[1, 288, 38, 38]> input = ?",
            "Tensor<[48, 288, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 288, 38, 38]> grad_output = ?",
            "Tensor<[1, 288, 38, 38]> input = ?",
            "Tensor<[288, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 288",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 48, 38, 38]> grad_output = ?",
            "Tensor<[1, 192, 38, 38]> input = ?",
            "Tensor<[48, 192, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 38, 38]> grad_output = ?",
            "Tensor<[1, 192, 79, 79]> input = ?",
            "Tensor<[192, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 192",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 75, 75]> grad_output = ?",
            "Tensor<[1, 32, 75, 75]> input = ?",
            "Tensor<[192, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 75, 75]> grad_output = ?",
            "Tensor<[1, 192, 75, 75]> input = ?",
            "Tensor<[32, 192, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 75, 75]> grad_output = ?",
            "Tensor<[1, 192, 75, 75]> input = ?",
            "Tensor<[192, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 192",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 75, 75]> grad_output = ?",
            "Tensor<[1, 144, 75, 75]> input = ?",
            "Tensor<[32, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 75, 75]> grad_output = ?",
            "Tensor<[1, 144, 151, 151]> input = ?",
            "Tensor<[144, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 144",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 150, 150]> grad_output = ?",
            "Tensor<[1, 24, 150, 150]> input = ?",
            "Tensor<[144, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 150, 150]> grad_output = ?",
            "Tensor<[1, 32, 150, 150]> input = ?",
            "Tensor<[24, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 150, 150]> grad_output = ?",
            "Tensor<[1, 32, 150, 150]> input = ?",
            "Tensor<[32, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 32",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 150, 150]> grad_output = ?",
            "Tensor<[1, 3, 301, 301]> input = ?",
            "Tensor<[32, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1280, 12, 12]> grad_output = ?",
            "Tensor<[1, 448, 12, 12]> input = ?",
            "Tensor<[1280, 448, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 448, 12, 12]> grad_output = ?",
            "Tensor<[1, 1632, 12, 12]> input = ?",
            "Tensor<[448, 1632, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1632, 12, 12]> grad_output = ?",
            "Tensor<[1, 1632, 12, 12]> input = ?",
            "Tensor<[1632, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1632",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1632, 12, 12]> grad_output = ?",
            "Tensor<[1, 272, 12, 12]> input = ?",
            "Tensor<[1632, 272, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 272, 12, 12]> grad_output = ?",
            "Tensor<[1, 1632, 12, 12]> input = ?",
            "Tensor<[272, 1632, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1632, 12, 12]> grad_output = ?",
            "Tensor<[1, 1632, 12, 12]> input = ?",
            "Tensor<[1632, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1632",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 272, 12, 12]> grad_output = ?",
            "Tensor<[1, 960, 12, 12]> input = ?",
            "Tensor<[272, 960, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 12, 12]> grad_output = ?",
            "Tensor<[1, 960, 27, 27]> input = ?",
            "Tensor<[960, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 960",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 24, 24]> grad_output = ?",
            "Tensor<[1, 160, 24, 24]> input = ?",
            "Tensor<[960, 160, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 160, 24, 24]> grad_output = ?",
            "Tensor<[1, 960, 24, 24]> input = ?",
            "Tensor<[160, 960, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 960, 24, 24]> grad_output = ?",
            "Tensor<[1, 960, 24, 24]> input = ?",
            "Tensor<[960, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 960",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 160, 24, 24]> grad_output = ?",
            "Tensor<[1, 672, 24, 24]> input = ?",
            "Tensor<[160, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 24, 24]> grad_output = ?",
            "Tensor<[1, 672, 24, 24]> input = ?",
            "Tensor<[672, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 24, 24]> grad_output = ?",
            "Tensor<[1, 112, 24, 24]> input = ?",
            "Tensor<[672, 112, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 24, 24]> grad_output = ?",
            "Tensor<[1, 672, 24, 24]> input = ?",
            "Tensor<[112, 672, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 672, 24, 24]> grad_output = ?",
            "Tensor<[1, 672, 24, 24]> input = ?",
            "Tensor<[672, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 672",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 112, 24, 24]> grad_output = ?",
            "Tensor<[1, 336, 24, 24]> input = ?",
            "Tensor<[112, 336, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 336, 24, 24]> grad_output = ?",
            "Tensor<[1, 336, 49, 49]> input = ?",
            "Tensor<[336, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 336",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 336, 48, 48]> grad_output = ?",
            "Tensor<[1, 56, 48, 48]> input = ?",
            "Tensor<[336, 56, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 56, 48, 48]> grad_output = ?",
            "Tensor<[1, 336, 48, 48]> input = ?",
            "Tensor<[56, 336, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 336, 48, 48]> grad_output = ?",
            "Tensor<[1, 336, 48, 48]> input = ?",
            "Tensor<[336, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [2, 2]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 336",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 56, 48, 48]> grad_output = ?",
            "Tensor<[1, 192, 48, 48]> input = ?",
            "Tensor<[56, 192, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 48, 48]> grad_output = ?",
            "Tensor<[1, 192, 99, 99]> input = ?",
            "Tensor<[192, 1, 5, 5]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 192",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 95, 95]> grad_output = ?",
            "Tensor<[1, 32, 95, 95]> input = ?",
            "Tensor<[192, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 95, 95]> grad_output = ?",
            "Tensor<[1, 192, 95, 95]> input = ?",
            "Tensor<[32, 192, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 192, 95, 95]> grad_output = ?",
            "Tensor<[1, 192, 95, 95]> input = ?",
            "Tensor<[192, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 192",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 95, 95]> grad_output = ?",
            "Tensor<[1, 144, 95, 95]> input = ?",
            "Tensor<[32, 144, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 95, 95]> grad_output = ?",
            "Tensor<[1, 144, 191, 191]> input = ?",
            "Tensor<[144, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 144",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 144, 190, 190]> grad_output = ?",
            "Tensor<[1, 24, 190, 190]> input = ?",
            "Tensor<[144, 24, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 24, 190, 190]> grad_output = ?",
            "Tensor<[1, 32, 190, 190]> input = ?",
            "Tensor<[24, 32, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 190, 190]> grad_output = ?",
            "Tensor<[1, 32, 190, 190]> input = ?",
            "Tensor<[32, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 32",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 190, 190]> grad_output = ?",
            "Tensor<[1, 3, 381, 381]> input = ?",
            "Tensor<[32, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 2048, 10, 10]> grad_output = ?",
            "Tensor<[1, 1536, 10, 10]> input = ?",
            "Tensor<[2048, 1536, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1536, 10, 10]> grad_output = ?",
            "Tensor<[1, 1536, 10, 10]> input = ?",
            "Tensor<[1536, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1536",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1536, 10, 10]> grad_output = ?",
            "Tensor<[1, 1536, 10, 10]> input = ?",
            "Tensor<[1536, 1536, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1536, 10, 10]> grad_output = ?",
            "Tensor<[1, 1024, 10, 10]> input = ?",
            "Tensor<[1536, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 10, 10]> grad_output = ?",
            "Tensor<[1, 1024, 10, 10]> input = ?",
            "Tensor<[1024, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1024",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 10, 10]> grad_output = ?",
            "Tensor<[1, 728, 19, 19]> input = ?",
            "Tensor<[1024, 728, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 10, 10]> grad_output = ?",
            "Tensor<[1, 1024, 10, 10]> input = ?",
            "Tensor<[1024, 1024, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 10, 10]> grad_output = ?",
            "Tensor<[1, 1024, 19, 19]> input = ?",
            "Tensor<[1024, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1024",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 1024, 19, 19]> grad_output = ?",
            "Tensor<[1, 728, 19, 19]> input = ?",
            "Tensor<[1024, 728, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 728, 19, 19]> grad_output = ?",
            "Tensor<[1, 728, 19, 19]> input = ?",
            "Tensor<[728, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 728",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 728, 19, 19]> grad_output = ?",
            "Tensor<[1, 728, 19, 19]> input = ?",
            "Tensor<[728, 728, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 728, 19, 19]> grad_output = ?",
            "Tensor<[1, 728, 38, 38]> input = ?",
            "Tensor<[728, 728, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 728, 19, 19]> grad_output = ?",
            "Tensor<[1, 728, 38, 38]> input = ?",
            "Tensor<[728, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 728",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 728, 38, 38]> grad_output = ?",
            "Tensor<[1, 728, 38, 38]> input = ?",
            "Tensor<[728, 728, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 728, 38, 38]> grad_output = ?",
            "Tensor<[1, 728, 38, 38]> input = ?",
            "Tensor<[728, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 728",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 728, 38, 38]> grad_output = ?",
            "Tensor<[1, 256, 38, 38]> input = ?",
            "Tensor<[728, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 38, 38]> grad_output = ?",
            "Tensor<[1, 256, 38, 38]> input = ?",
            "Tensor<[256, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 256",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 38, 38]> grad_output = ?",
            "Tensor<[1, 256, 75, 75]> input = ?",
            "Tensor<[256, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 38, 38]> grad_output = ?",
            "Tensor<[1, 256, 38, 38]> input = ?",
            "Tensor<[256, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 38, 38]> grad_output = ?",
            "Tensor<[1, 256, 75, 75]> input = ?",
            "Tensor<[256, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 256",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 75, 75]> grad_output = ?",
            "Tensor<[1, 256, 75, 75]> input = ?",
            "Tensor<[256, 256, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 75, 75]> grad_output = ?",
            "Tensor<[1, 256, 75, 75]> input = ?",
            "Tensor<[256, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 256",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 256, 75, 75]> grad_output = ?",
            "Tensor<[1, 128, 75, 75]> input = ?",
            "Tensor<[256, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 75, 75]> grad_output = ?",
            "Tensor<[1, 128, 75, 75]> input = ?",
            "Tensor<[128, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 128",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 75, 75]> grad_output = ?",
            "Tensor<[1, 64, 150, 150]> input = ?",
            "Tensor<[128, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 75, 75]> grad_output = ?",
            "Tensor<[1, 128, 75, 75]> input = ?",
            "Tensor<[128, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 75, 75]> grad_output = ?",
            "Tensor<[1, 128, 150, 150]> input = ?",
            "Tensor<[128, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 128",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 150, 150]> grad_output = ?",
            "Tensor<[1, 128, 150, 150]> input = ?",
            "Tensor<[128, 128, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 150, 150]> grad_output = ?",
            "Tensor<[1, 128, 150, 150]> input = ?",
            "Tensor<[128, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 128",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 128, 150, 150]> grad_output = ?",
            "Tensor<[1, 64, 150, 150]> input = ?",
            "Tensor<[128, 64, 1, 1]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [0, 0]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 150, 150]> grad_output = ?",
            "Tensor<[1, 64, 150, 150]> input = ?",
            "Tensor<[64, 1, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 64",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 64, 150, 150]> grad_output = ?",
            "Tensor<[1, 32, 150, 150]> input = ?",
            "Tensor<[64, 32, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [1, 1]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
        [
            "Tensor<[1, 32, 150, 150]> grad_output = ?",
            "Tensor<[1, 3, 299, 299]> input = ?",
            "Tensor<[32, 3, 3, 3]> weight = ?",
            "Optional[List[int]] bias_sizes = [0]",
            "List[int] stride = [2, 2]",
            "List[int] padding = [1, 1]",
            "List[int] dilation = [1, 1]",
            "bool transposed = False",
            "List[int] output_padding = [0, 0]",
            "int groups = 1",
            "List[bool] output_mask = [True, True, False]",
        ],
    ],
)
def test_aten(device, input_strings, input_var_only_native, input_var_check_accu, input_var_check_ttnn):
    metric = {
        "opname": "aten.convolution_backward.default",
        "input_strings": input_strings,
        "native_run": "N/A",
        "run": "N/A",
        "accuracy": "N/A",
        "convert_to_ttnn": "N/A",
        "ttnn_fallbacks_to_host_count": "N/A",
    }
    m = AtenModule()
    input_args, input_kwargs, status = render_metric_string_list_to_input_args_kwargs(
        "aten.convolution_backward.default", input_strings
    )
    if status == False:
        pytest.skip("Invalid input strings")
    try:
        result_before = m.forward(*input_args, **input_kwargs)
        metric["native_run"] = True
    except Exception as e:
        print(f"Failed to run native. Raised exception: {e}")
        metric["native_run"] = False

    if metric["native_run"] == True:
        result_after = None
        option = torch_ttnn.TorchTtnnOption(device=device)
        # option.gen_graphviz = True
        # The compilation is lazy, so we need to run forward once to trigger the compilation
        m = torch.compile(m, backend=torch_ttnn.backend, options=option)
        try:
            ttnn.graph.begin_graph_capture()
            result_after = m.forward(*input_args, **input_kwargs)
            # option._out_fx_graphs[0].print_tabular()
            metric["run"] = True
        except Exception as e:
            print(f"Failed to run. Raised exception: {e}")
            metric["run"] = False
        finally:
            trace = ttnn.graph.end_graph_capture()
            call_stack = ttnn.graph.extract_calltrace(trace)
            if metric["run"] == True:
                print(call_stack)
                expected_to_host_count = 0
                if result_after is None:
                    expected_to_host_count = 0
                elif isinstance(result_after, torch.Tensor):
                    expected_to_host_count = 1
                elif isinstance(result_after, (list, dict)):
                    expected_to_host_count = len(result_after)
                else:
                    print(f"Unexpected result_after type: {type(result_after)}")

                to_host_count = sum(["Tensor::cpu" in str(node) for node in call_stack])
                fallbacks_to_host_count = to_host_count - expected_to_host_count
                print(f"expected_to_host_count: {expected_to_host_count}")
                print(f"to_host_count: {to_host_count}")
                print(f"fallbacks_to_host_count: {fallbacks_to_host_count}")
                metric["ttnn_fallbacks_to_host_count"] = fallbacks_to_host_count

    if metric["run"] == True:
        try:
            # Check inference result
            metric["accuracy"] = calculate_accuracy(result_before, result_after)
        except Exception as e:
            print(f"Failed to check inference result. Raised exception: {e}")

        try:
            # Check the graph has be rewritten and contain ttnn ops
            nodes = list(option._out_fx_graphs[0].nodes)
            if not any(["aten." in str(node.target) for node in nodes]):
                metric["convert_to_ttnn"] = True
            else:
                metric["convert_to_ttnn"] = False
        except Exception as e:
            print(f"Failed to check the graph has ttnn op. Raised exception: {e}")

    metrics.append(metric)

    if not input_var_only_native:
        assert metric["run"] == True
        if input_var_check_accu:
            assert metric["accuracy"] >= 0.99
        if input_var_check_ttnn:
            assert metric["convert_to_ttnn"] == True
